@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -61,12 +61,6 @@ guAlListBox::~guAlListBox()
     Disconnect( ID_ALBUM_COMMANDS, ID_ALBUM_COMMANDS + 99, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlListBox::OnCommandClicked ) );
 }
 
-// -------------------------------------------------------------------------------- //
-void guAlListBox::GetItemsList( void )
-{
-    m_Db->GetAlbums( ( guAlbumItems * ) m_Items );
-}
-
 // -------------------------------------------------------------------------------- //
 bool guAlListBox::SelectAlbumName( const wxString &AlbumName )
 {
@@ -98,11 +92,22 @@ void guAlListBox::DrawItem( wxDC &dc, const wxRect &rect, const int row, const i
 
     dc.DrawText( Item->m_Name, rect.x + 45, rect.y + 4 );
 
-    if( Item->m_Year > 0 )
+    if( Item->m_Year )
     {
+        int Pos;
+        dc.GetTextExtent( Item->m_Name, &Pos, NULL );
+        Pos += rect.x + 45;
         m_Attr.m_Font->SetPointSize( 7 );
         dc.SetFont( * m_Attr.m_Font );
-        dc.DrawText( wxString::Format( wxT( "%4u" ), Item->m_Year ), rect.x + 45, rect.y + 22 );
+        dc.DrawText( wxString::Format( wxT( " (%4u)" ), Item->m_Year ), Pos, rect.y + 7 );
+    }
+
+
+    if( !Item->m_ArtistName.IsEmpty() )
+    {
+        m_Attr.m_Font->SetPointSize( 8 );
+        dc.SetFont( * m_Attr.m_Font );
+        dc.DrawText( _( "by " ) + Item->m_ArtistName, rect.x + 45, rect.y + 22 );
     }
 
     if( Item->m_Thumb && Item->m_Thumb->IsOk() )
@@ -179,14 +184,18 @@ void guAlListBox::CreateContextMenu( wxMenu * Menu ) const
     wxMenuItem * MenuItem;
 
     MenuItem = new wxMenuItem( Menu, ID_ALBUM_PLAY, _( "Play" ), _( "Play current selected albums" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_playback_start ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_player_tiny_light_play ) );
     Menu->Append( MenuItem );
 
     MenuItem = new wxMenuItem( Menu, ID_ALBUM_ENQUEUE, _( "Enqueue" ), _( "Add current selected albums to the Playlist" ) );
     MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
     Menu->Append( MenuItem );
 
-    int SelCount = GetSelectedItems().Count();
+    MenuItem = new wxMenuItem( Menu, ID_ALBUM_ENQUEUE_ASNEXT, _( "Enqueue Next" ), _( "Add current selected albums to the Playlist as Next Tracks" ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
+    Menu->Append( MenuItem );
+
+    int SelCount = GetSelectedCount();
     if( SelCount )
     {
         Menu->AppendSeparator();
@@ -320,7 +329,9 @@ void guAlListBox::OnCommandClicked( wxCommandEvent &event )
                 count = AlbumPaths.Count();
                 for( index = 0; index < count; index++ )
                 {
-                    Paths += wxT( " \"" ) + AlbumPaths[ index ] + wxT( "\"" );
+                    //Paths += wxT( " \"" ) + AlbumPaths[ index ] + wxT( "\"" );
+                    AlbumPaths[ index ].Replace( wxT( " " ), wxT( "\\ " )  );
+                    Paths += wxT( " " ) + AlbumPaths[ index ];
                 }
                 CurCmd.Replace( wxT( "{bp}" ), Paths.Trim( false ) );
             }
@@ -378,6 +389,12 @@ int inline guAlListBox::GetItemId( const int row ) const
     return ( * m_Items )[ row ].m_Id;
 }
 
+// -------------------------------------------------------------------------------- //
+void guAlListBox::GetItemsList( void )
+{
+    m_Db->GetAlbums( ( guAlbumItems * ) m_Items );
+}
+
 // -------------------------------------------------------------------------------- //
 void guAlListBox::ReloadItems( bool reset )
 {
@@ -408,10 +425,10 @@ void  guAlListBox::SetSelectedItems( const wxArrayInt &selection )
 {
     guListView::SetSelectedItems( selection );
 
-    wxCommandEvent event( wxEVT_COMMAND_LISTBOX_SELECTED, GetId() );
-    event.SetEventObject( this );
-    event.SetInt( -1 );
-    (void) GetEventHandler()->ProcessEvent( event );
+//    wxCommandEvent event( wxEVT_COMMAND_LISTBOX_SELECTED, GetId() );
+//    event.SetEventObject( this );
+//    event.SetInt( -1 );
+//    (void) GetEventHandler()->ProcessEvent( event );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -422,13 +439,29 @@ int guAlListBox::GetDragFiles( wxFileDataObject * files )
     int count = GetSelectedSongs( &Songs );
     for( index = 0; index < count; index++ )
     {
-       wxString FileName = Songs[ index ].m_FileName;
-       FileName.Replace( wxT( "#" ), wxT( "%23" ) );
+       wxString FileName = guFileDnDEncode( Songs[ index ].m_FileName );
+       //FileName.Replace( wxT( "#" ), wxT( "%23" ) );
+
        //FileName.Replace( wxT( "%" ), wxT( "%25" ) );
-       //guLogMessage( wxT( "Adding song '%s'" ), Songs[ index ].m_FileName.c_str() );
+       //guLogMessage( wxT( "Adding song '%s'" ), FileName.c_str() );
        files->AddFile( FileName );
     }
     return count;
 }
 
 // -------------------------------------------------------------------------------- //
+int guAlListBox::FindAlbum( const int albumid )
+{
+    int Index;
+    int Count = m_Items->Count();
+    for( Index = 0; Index < Count; Index++ )
+    {
+        if( m_Items->Item( Index ).m_Id == albumid )
+        {
+            return Index;
+        }
+    }
+    return wxNOT_FOUND;
+}
+
+// -------------------------------------------------------------------------------- //
