@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -40,6 +40,12 @@
 WX_DEFINE_OBJARRAY( guAlbumBrowserItemArray );
 
 #define guALBUMBROWSER_REFRESH_DELAY    60
+#define guALBUMBROWSER_GRID_SIZE        180
+
+#define guALBUMBROWSER_TIMER_ID_REFRESH    3
+#define guALBUMBROWSER_TIMER_ID_TEXTSEARCH 4
+
+#define guALBUMBROWSER_TIMER_TEXTSEARCH_DELAY 500
 
 void AddAlbumCommands( wxMenu * Menu, int SelCount );
 
@@ -125,23 +131,30 @@ guAlbumBrowserItemPanel::guAlbumBrowserItemPanel( wxWindow * parent, const int i
     m_AlbumBrowserItem = albumitem;
     m_AlbumBrowser = ( guAlbumBrowser * ) parent;
 
+    wxFont CurrentFont = GetFont();
     // GUI
 	m_MainSizer = new wxBoxSizer( wxVERTICAL );
 
 	m_Bitmap = new wxStaticBitmap( this, wxID_ANY, wxNullBitmap, wxDefaultPosition, wxSize( 100, 100 ), 0 );
-	m_MainSizer->Add( m_Bitmap, 0, wxALIGN_CENTER_HORIZONTAL|wxTOP|wxRIGHT|wxLEFT, 5 );
+	m_MainSizer->Add( m_Bitmap, 0, wxTOP|wxRIGHT|wxLEFT, 5 );
 
-	m_ArtistLabel = new wxStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE );
-	m_ArtistLabel->Wrap( 120 );
-	m_MainSizer->Add( m_ArtistLabel, 0, wxALIGN_CENTER_HORIZONTAL|wxRIGHT|wxLEFT, 5 );
-
-	m_AlbumLabel = new wxStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE );
+	m_AlbumLabel = new wxStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT );
 	m_AlbumLabel->Wrap( 120 );
-	m_MainSizer->Add( m_AlbumLabel, 0, wxALIGN_CENTER_HORIZONTAL|wxRIGHT|wxLEFT, 5 );
+	CurrentFont.SetWeight( wxFONTWEIGHT_BOLD );
+	CurrentFont.SetPointSize( CurrentFont.GetPointSize() - 1 );
+	m_AlbumLabel->SetFont( CurrentFont );
+	m_MainSizer->Add( m_AlbumLabel, 0, wxRIGHT|wxLEFT|wxEXPAND, 5 );
 
-	m_TracksLabel = new wxStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE );
+	m_ArtistLabel = new wxStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT );
+	m_ArtistLabel->Wrap( 120 );
+	CurrentFont.SetWeight( wxFONTWEIGHT_NORMAL );
+	m_ArtistLabel->SetFont( CurrentFont );
+	m_MainSizer->Add( m_ArtistLabel, 0, wxRIGHT|wxLEFT|wxEXPAND, 5 );
+
+	m_TracksLabel = new wxStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT );
 	m_TracksLabel->Wrap( 120 );
-	m_MainSizer->Add( m_TracksLabel, 0, wxALIGN_CENTER_HORIZONTAL|wxBOTTOM|wxRIGHT|wxLEFT, 5 );
+	m_TracksLabel->SetFont( CurrentFont );
+	m_MainSizer->Add( m_TracksLabel, 0, wxBOTTOM|wxRIGHT|wxLEFT|wxEXPAND, 5 );
 
 	SetSizer( m_MainSizer );
 	Layout();
@@ -158,6 +171,7 @@ guAlbumBrowserItemPanel::guAlbumBrowserItemPanel( wxWindow * parent, const int i
 
     Connect( ID_ALBUMBROWSER_PLAY, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnPlayClicked ), NULL, this );
     Connect( ID_ALBUMBROWSER_ENQUEUE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnEnqueueClicked ), NULL, this );
+    Connect( ID_ALBUMBROWSER_ENQUEUE_ASNEXT, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnEnqueueAsNextClicked ), NULL, this );
     Connect( ID_ALBUMBROWSER_EDITLABELS, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnAlbumEditLabelsClicked ), NULL, this );
     Connect( ID_ALBUMBROWSER_EDITTRACKS, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnAlbumEditTracksClicked ), NULL, this );
     Connect( ID_ALBUMBROWSER_COPYTOCLIPBOARD, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnCopyToClipboard ), NULL, this );
@@ -189,6 +203,7 @@ guAlbumBrowserItemPanel::~guAlbumBrowserItemPanel()
 
     Disconnect( ID_ALBUMBROWSER_PLAY, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnPlayClicked ), NULL, this );
     Disconnect( ID_ALBUMBROWSER_ENQUEUE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnEnqueueClicked ), NULL, this );
+    Disconnect( ID_ALBUMBROWSER_ENQUEUE_ASNEXT, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnEnqueueAsNextClicked ), NULL, this );
     Disconnect( ID_ALBUMBROWSER_EDITLABELS, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnAlbumEditLabelsClicked ), NULL, this );
     Disconnect( ID_ALBUMBROWSER_EDITTRACKS, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnAlbumEditTracksClicked ), NULL, this );
     Disconnect( ID_ALBUMBROWSER_COPYTOCLIPBOARD, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowserItemPanel::OnCopyToClipboard ), NULL, this );
@@ -223,6 +238,7 @@ void guAlbumBrowserItemPanel::SetAlbumItem( const int index, guAlbumBrowserItem
     m_AlbumBrowserItem = albumitem;
     if( m_AlbumBrowserItem )
     {
+        m_Bitmap->Show();
         //guLogMessage( wxT( "Inserting item %i '%s'" ), index, albumitem->m_AlbumName.c_str() );
         if( m_AlbumBrowserItem->m_CoverBitmap )
             m_Bitmap->SetBitmap( * m_AlbumBrowserItem->m_CoverBitmap );
@@ -233,13 +249,19 @@ void guAlbumBrowserItemPanel::SetAlbumItem( const int index, guAlbumBrowserItem
         m_ArtistLabel->SetLabel( guCheckLabelString( m_AlbumBrowserItem->m_ArtistName ) );
         m_AlbumLabel->SetLabel( guCheckLabelString( m_AlbumBrowserItem->m_AlbumName ) );
 
-        m_TracksLabel->SetLabel( wxString::Format( wxT( "(%u) %u " ),
-            m_AlbumBrowserItem->m_Year,
-            m_AlbumBrowserItem->m_TrackCount ) + _( "Tracks" ) );
+        wxString Label;
+        if( m_AlbumBrowserItem->m_Year )
+        {
+            Label += wxString::Format( wxT( "(%u) " ), m_AlbumBrowserItem->m_Year );
+        }
+        Label += wxString::Format( wxT( "%u " ), m_AlbumBrowserItem->m_TrackCount ) + _( "Tracks" );
+
+        m_TracksLabel->SetLabel( Label );
     }
     else
     {
         m_Bitmap->SetBitmap( * blankcd );
+        m_Bitmap->Hide();
         m_ArtistLabel->SetLabel( wxEmptyString );
         m_AlbumLabel->SetLabel( wxEmptyString );
         m_TracksLabel->SetLabel( wxEmptyString );
@@ -252,9 +274,14 @@ void guAlbumBrowserItemPanel::UpdateDetails( void )
 {
     if( m_AlbumBrowserItem )
     {
-        m_TracksLabel->SetLabel( wxString::Format( wxT( "(%u) %u " ),
-            m_AlbumBrowserItem->m_Year,
-            m_AlbumBrowserItem->m_TrackCount ) + _( "Tracks" ) );
+        wxString Label;
+        if( m_AlbumBrowserItem->m_Year )
+        {
+            Label += wxString::Format( wxT( "(%u) " ), m_AlbumBrowserItem->m_Year );
+        }
+        Label += wxString::Format( wxT( "%u " ), m_AlbumBrowserItem->m_TrackCount ) + _( "Tracks" );
+
+        m_TracksLabel->SetLabel( Label );
         m_MainSizer->Layout();
     }
 }
@@ -284,13 +311,17 @@ void guAlbumBrowserItemPanel::OnContextMenu( wxContextMenuEvent &event )
     if( m_AlbumBrowserItem->m_AlbumId != ( size_t ) wxNOT_FOUND )
     {
         MenuItem = new wxMenuItem( &Menu, ID_ALBUMBROWSER_PLAY, _( "Play" ), _( "Play the album tracks" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_playback_start ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_player_tiny_light_play ) );
         Menu.Append( MenuItem );
 
         MenuItem = new wxMenuItem( &Menu, ID_ALBUMBROWSER_ENQUEUE, _( "Enqueue" ), _( "Enqueue the album tracks to the playlist" ) );
         MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
         Menu.Append( MenuItem );
 
+        MenuItem = new wxMenuItem( &Menu, ID_ALBUMBROWSER_ENQUEUE_ASNEXT, _( "Enqueue Next" ), _( "Enqueue the album tracks to the playlist as Next Tracks" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+        Menu.Append( MenuItem );
+
         Menu.AppendSeparator();
 
         MenuItem = new wxMenuItem( &Menu, ID_ALBUMBROWSER_EDITLABELS, _( "Edit Labels" ), _( "Edit the labels assigned to the selected albums" ) );
@@ -325,7 +356,7 @@ void guAlbumBrowserItemPanel::OnContextMenu( wxContextMenuEvent &event )
 
         Menu.AppendSeparator();
 
-        MenuItem = new wxMenuItem( &Menu, ID_ALBUMBROWSER_COPYTOCLIPBOARD, wxT( "Copy to clipboard" ), _( "Copy the album info to clipboard" ) );
+        MenuItem = new wxMenuItem( &Menu, ID_ALBUMBROWSER_COPYTOCLIPBOARD, _( "Copy to clipboard" ), _( "Copy the album info to clipboard" ) );
         MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit_copy ) );
         Menu.Append( MenuItem );
 
@@ -388,6 +419,12 @@ void guAlbumBrowserItemPanel::OnEnqueueClicked( wxCommandEvent &event )
     m_AlbumBrowser->SelectAlbum( m_AlbumBrowserItem->m_AlbumId, true );
 }
 
+// -------------------------------------------------------------------------------- //
+void guAlbumBrowserItemPanel::OnEnqueueAsNextClicked( wxCommandEvent &event )
+{
+    m_AlbumBrowser->SelectAlbum( m_AlbumBrowserItem->m_AlbumId, true, true );
+}
+
 // -------------------------------------------------------------------------------- //
 void guAlbumBrowserItemPanel::OnAlbumDClicked( wxMouseEvent &event )
 {
@@ -422,18 +459,18 @@ void guAlbumBrowserItemPanel::OnCopyToClipboard( wxCommandEvent &event )
 // -------------------------------------------------------------------------------- //
 void guAlbumBrowserItemPanel::OnAlbumSelectName( wxCommandEvent &event )
 {
-    wxString * AlbumName = new wxString( m_AlbumBrowserItem->m_AlbumName );
-    wxCommandEvent evt( wxEVT_COMMAND_MENU_SELECTED, ID_ALBUM_SELECTNAME );
-    evt.SetClientData( ( void * ) AlbumName );
+    wxCommandEvent evt( wxEVT_COMMAND_MENU_SELECTED, ID_MAINFRAME_SELECT_ALBUM );
+    evt.SetInt( m_AlbumBrowserItem->m_AlbumId );
+    evt.SetExtraLong( guTRACK_TYPE_DB );
     wxPostEvent( wxTheApp->GetTopWindow(), evt );
 }
 
 // -------------------------------------------------------------------------------- //
 void guAlbumBrowserItemPanel::OnArtistSelectName( wxCommandEvent &event )
 {
-    wxString * ArtistName = new wxString( m_AlbumBrowserItem->m_ArtistName );
-    wxCommandEvent evt( wxEVT_COMMAND_MENU_SELECTED, ID_ARTIST_SELECTNAME );
-    evt.SetClientData( ( void * ) ArtistName );
+    wxCommandEvent evt( wxEVT_COMMAND_MENU_SELECTED, ID_MAINFRAME_SELECT_ARTIST );
+    evt.SetInt( m_AlbumBrowserItem->m_ArtistId );
+    evt.SetExtraLong( guTRACK_TYPE_DB );
     wxPostEvent( wxTheApp->GetTopWindow(), evt );
 }
 
@@ -464,7 +501,7 @@ void guAlbumBrowserItemPanel::OnAlbumCopyToClicked( wxCommandEvent &event )
 // -------------------------------------------------------------------------------- //
 void guAlbumBrowserItemPanel::OnAlbumEditLabelsClicked( wxCommandEvent &event )
 {
-    m_AlbumBrowser->OnAlbumEditLabelsClicked( m_AlbumBrowserItem->m_AlbumId );
+    m_AlbumBrowser->OnAlbumEditLabelsClicked( m_AlbumBrowserItem );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -570,7 +607,9 @@ void guAlbumBrowserItemPanel::SetAlbumCover( const wxString &cover )
 // guAlbumBrowser
 // -------------------------------------------------------------------------------- //
 guAlbumBrowser::guAlbumBrowser( wxWindow * parent, guDbLibrary * db, guPlayerPanel * playerpanel ) :
-    wxPanel( parent, wxID_ANY )
+    wxPanel( parent, wxID_ANY ),
+    m_RefreshTimer( this, guALBUMBROWSER_TIMER_ID_REFRESH ),
+    m_TextChangedTimer( this, guALBUMBROWSER_TIMER_ID_TEXTSEARCH )
 {
     m_Db = db;
     m_PlayerPanel = playerpanel;
@@ -578,9 +617,7 @@ guAlbumBrowser::guAlbumBrowser( wxWindow * parent, guDbLibrary * db, guPlayerPan
     m_ItemStart = 0;
     m_LastItemStart = wxNOT_FOUND;
     m_ItemCount = 1;
-    wxImage TmpImg = wxImage( guImage( guIMAGE_INDEX_blank_cd_cover ) );
-    TmpImg.Rescale( 100, 100 );
-    m_BlankCD = new wxBitmap( TmpImg );
+    m_BlankCD = new wxBitmap( guImage( guIMAGE_INDEX_no_cover ) );
 
     guConfig * Config = ( guConfig * ) guConfig::Get();
     m_DynFilter.FromString( Config->ReadStr( wxT( "Filter" ), wxEmptyString, wxT( "AlbumBrowser" ) ) );
@@ -606,7 +643,12 @@ guAlbumBrowser::guAlbumBrowser( wxWindow * parent, guDbLibrary * db, guPlayerPan
 	m_EditFilterBtn = new wxBitmapButton( this, wxID_ANY, guImage( guIMAGE_INDEX_tiny_search ), wxDefaultPosition, wxSize( 28, 28 ), wxBU_AUTODRAW );
 	FilterSizer->Add( m_EditFilterBtn, 0, wxTOP|wxRIGHT|wxLEFT, 5 );
 
-	FilterSizer->Add( 0, 0, 1, wxEXPAND, 5 );
+//	wxStaticText * SearchLabel = new wxStaticText( this, wxID_ANY, _( "Search:" ), wxDefaultPosition, wxDefaultSize, 0 );
+//	SearchLabel->Wrap( -1 );
+//	FilterSizer->Add( SearchLabel, 0, wxTOP|wxLEFT|wxALIGN_CENTER_VERTICAL, 5 );
+
+	m_SearchTextCtrl = new wxSearchCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER );
+	FilterSizer->Add( m_SearchTextCtrl, 1, wxTOP|wxLEFT| wxRIGHT|wxALIGN_CENTER_VERTICAL, 5 );
 
 	wxString m_OrderChoiceChoices[] = { _("Order By"), _("Name"), _("Year"), _("Year Desc."), _("Artist, Name"), _("Artist, Year"), _("Artist, Year Desc.") };
 	int m_OrderChoiceNChoices = sizeof( m_OrderChoiceChoices ) / sizeof( wxString );
@@ -657,15 +699,21 @@ guAlbumBrowser::guAlbumBrowser( wxWindow * parent, guDbLibrary * db, guPlayerPan
 //	m_NavSlider->Connect( wxEVT_SCROLL_PAGEUP, wxScrollEventHandler( guAlbumBrowser::OnChangingPosition ), NULL, this );
 //	m_NavSlider->Connect( wxEVT_SCROLL_PAGEDOWN, wxScrollEventHandler( guAlbumBrowser::OnChangingPosition ), NULL, this );
 
-	Connect( wxEVT_TIMER, wxTimerEventHandler( guAlbumBrowser::OnRefreshTimer ), NULL, this );
+	Connect( guALBUMBROWSER_TIMER_ID_REFRESH, wxEVT_TIMER, wxTimerEventHandler( guAlbumBrowser::OnRefreshTimer ), NULL, this );
 
 	Connect( ID_ALBUMBROWSER_UPDATEDETAILS, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlbumBrowser::OnUpdateDetails ), NULL, this );
 
 	Connect( wxEVT_MOUSEWHEEL, wxMouseEventHandler( guAlbumBrowser::OnMouseWheel ) );
 
+    m_SearchTextCtrl->Connect( wxEVT_COMMAND_TEXT_ENTER, wxCommandEventHandler( guAlbumBrowser::OnSearchSelected ), NULL, this );
+    m_SearchTextCtrl->Connect( wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( guAlbumBrowser::OnSearchTextChanged ), NULL, this );
+    m_SearchTextCtrl->Connect( wxEVT_COMMAND_SEARCHCTRL_CANCEL_BTN, wxCommandEventHandler( guAlbumBrowser::OnSearchCancelled ), NULL, this );
+
+	Connect( guALBUMBROWSER_TIMER_ID_TEXTSEARCH, wxEVT_TIMER, wxTimerEventHandler( guAlbumBrowser::OnTextChangedTimer ), NULL, this );
+
     RefreshCount();
     //ReloadItems();
-    m_RefreshTimer.SetOwner( this );
+    //m_RefreshTimer.SetOwner( this );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -708,8 +756,8 @@ void guAlbumBrowser::OnChangedSize( wxSizeEvent &event )
     wxSize Size = event.GetSize();
     if( Size != m_LastSize )
     {
-        size_t ColItems = Size.GetWidth() / 220;
-        size_t RowItems = Size.GetHeight() / 220;
+        size_t ColItems = Size.GetWidth() / guALBUMBROWSER_GRID_SIZE;
+        size_t RowItems = Size.GetHeight() / guALBUMBROWSER_GRID_SIZE;
         //guLogMessage( wxT( "Row: %i  Col:%i" ), RowItems, ColItems );
         if( ColItems * RowItems != m_ItemPanels.Count() )
         {
@@ -757,13 +805,91 @@ void guAlbumBrowser::OnChangedSize( wxSizeEvent &event )
     event.Skip();
 }
 
+// -------------------------------------------------------------------------------- //
+void guAlbumBrowser::OnSearchTextChanged( wxCommandEvent& event )
+{
+    if( m_TextChangedTimer.IsRunning() )
+        m_TextChangedTimer.Stop();
+    m_TextChangedTimer.Start( guALBUMBROWSER_TIMER_TEXTSEARCH_DELAY, wxTIMER_ONE_SHOT );
+}
+
+// -------------------------------------------------------------------------------- //
+void guAlbumBrowser::OnSearchCancelled( wxCommandEvent &event ) // CLEAN SEARCH STR
+{
+    m_SearchTextCtrl->Clear();
+}
+
+// -------------------------------------------------------------------------------- //
+void guAlbumBrowser::OnSearchSelected( wxCommandEvent& event )
+{
+    guTrackArray Tracks;
+    wxArrayInt Selections;
+    int Index;
+    int Count = m_AlbumItems.Count();
+    for( Index = 0; Index < Count; Index++ )
+    {
+        Selections.Add( m_AlbumItems[ Index ].m_AlbumId );
+    }
+
+    if( m_Db->GetAlbumsSongs( Selections, &Tracks ) && Tracks.Count() )
+    {
+        guConfig * Config = ( guConfig * ) guConfig::Get();
+        if( Config )
+        {
+            if( Config->ReadBool( wxT( "DefaultActionEnqueue" ), false, wxT( "General" ) ) )
+            {
+                m_PlayerPanel->AddToPlayList( Tracks, true );
+            }
+            else
+            {
+                m_PlayerPanel->SetPlayList( Tracks );
+            }
+        }
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+void guAlbumBrowser::OnTextChangedTimer( wxTimerEvent &event )
+{
+    wxString SearchString = m_SearchTextCtrl->GetLineText( 0 );
+    if( !SearchString.IsEmpty() )
+    {
+        if( SearchString.Length() > 1 )
+        {
+            m_TextSearchFilter = guSplitWords( SearchString );
+
+            RefreshCount();
+            ReloadItems();
+            m_LastItemStart = wxNOT_FOUND;
+            m_NavSlider->SetValue( 0 );
+            RefreshAll();
+
+            m_SearchTextCtrl->ShowCancelButton( true );
+        }
+    }
+    else
+    {
+        m_TextSearchFilter.Clear();
+
+        RefreshCount();
+        ReloadItems();
+        m_LastItemStart = wxNOT_FOUND;
+        m_NavSlider->SetValue( 0 );
+        RefreshAll();
+
+        m_SearchTextCtrl->ShowCancelButton( false );
+    }
+}
+
+
+
 // -------------------------------------------------------------------------------- //
 void guAlbumBrowser::ReloadItems( void )
 {
     m_AlbumItemsMutex.Lock();
     m_AlbumItems.Empty();
     m_Db->GetAlbums( &m_AlbumItems, m_FilterBtn->GetValue() ? &m_DynFilter : NULL,
-                     m_ItemStart, m_ItemCount, m_OrderChoice->GetSelection() - 1 );
+            m_TextSearchFilter, m_ItemStart, m_ItemCount, m_OrderChoice->GetSelection() - 1 );
     m_AlbumItemsMutex.Unlock();
 //    guLogMessage( wxT( "Read %i items from %i (%i)" ), m_AlbumItems.Count(), m_ItemStart, m_ItemCount );
 //    RefreshAll();
@@ -827,7 +953,7 @@ void guAlbumBrowser::OnRefreshTimer( wxTimerEvent &event )
 }
 
 // -------------------------------------------------------------------------------- //
-void guAlbumBrowser::SelectAlbum( const int albumid, const bool append )
+void guAlbumBrowser::SelectAlbum( const int albumid, const bool append, const bool asnext )
 {
     guTrackArray Tracks;
     wxArrayInt Selections;
@@ -835,7 +961,7 @@ void guAlbumBrowser::SelectAlbum( const int albumid, const bool append )
     if( m_Db->GetAlbumsSongs( Selections, &Tracks ) )
     {
         if( append )
-            m_PlayerPanel->AddToPlayList( Tracks );
+            m_PlayerPanel->AddToPlayList( Tracks, true, asnext );
         else
             m_PlayerPanel->SetPlayList( Tracks );
     }
@@ -936,7 +1062,9 @@ void guAlbumBrowser::OnCommandClicked( const int cmdid, const int albumid )
                 count = AlbumPaths.Count();
                 for( index = 0; index < count; index++ )
                 {
-                    Paths += wxT( " \"" ) + AlbumPaths[ index ] + wxT( "\"" );
+                    AlbumPaths[ index ].Replace( wxT( " " ), wxT( "\\ " ) );
+                    //Paths += wxT( " \"" ) + AlbumPaths[ index ] + wxT( "\"" );
+                    Paths += wxT( " " ) + AlbumPaths[ index ];
                 }
                 CurCmd.Replace( wxT( "{bp}" ), Paths.Trim( false ) );
             }
@@ -1001,8 +1129,11 @@ void guAlbumBrowser::OnAlbumDownloadCoverClicked( const int albumid )
                 wxImage * CoverImage = CoverEditor->GetSelectedCoverImage();
                 if( CoverImage )
                 {
-                    CoverImage->SaveFile( AlbumPath + wxT( "cover.jpg" ), wxBITMAP_TYPE_JPEG );
-                    m_Db->SetAlbumCover( Albums[ 0 ], AlbumPath + wxT( "cover.jpg" ) );
+                    guConfig * Config = ( guConfig * ) guConfig::Get();
+                    wxArrayString SearchCovers = Config->ReadAStr( wxT( "Word" ), wxEmptyString, wxT( "CoverSearch" ) );
+                    wxString CoverName = AlbumPath + ( SearchCovers.Count() ? SearchCovers[ 0 ] : wxT( "cover" ) ) + wxT( ".jpg" );
+                    CoverImage->SaveFile( CoverName, wxBITMAP_TYPE_JPEG );
+                    m_Db->SetAlbumCover( Albums[ 0 ], CoverName );
 
                     ReloadItems();
                     RefreshAll();
@@ -1027,16 +1158,17 @@ void guAlbumBrowser::OnAlbumSelectCoverClicked( const int albumid )
                 wxURI Uri( CoverFile );
                 if( Uri.IsReference() )
                 {
-                    //m_Db->SetAlbumCover( albumid, CoverFile );
-                    //ReloadItems();
-                    //RefreshAll();
                     SetAlbumCover( albumid, CoverFile );
                 }
                 else
                 {
-                    if( DownloadImage( CoverFile, SelCoverFile->GetAlbumPath() + wxT( "/cover.jpg" ) ) )
+                    guConfig * Config = ( guConfig * ) guConfig::Get();
+                    wxArrayString SearchCovers = Config->ReadAStr( wxT( "Word" ), wxEmptyString, wxT( "CoverSearch" ) );
+                    wxString CoverName = SelCoverFile->GetAlbumPath() + ( SearchCovers.Count() ? SearchCovers[ 0 ] : wxT( "cover" ) ) + wxT( ".jpg" );
+
+                    if( DownloadImage( CoverFile, CoverName ) )
                     {
-                        m_Db->SetAlbumCover( albumid, SelCoverFile->GetAlbumPath() + wxT( "/cover.jpg" ) );
+                        m_Db->SetAlbumCover( albumid, CoverName );
                         ReloadItems();
                         RefreshAll();
                     }
@@ -1094,24 +1226,34 @@ void guAlbumBrowser::OnAlbumCopyToClicked( const int albumid )
 }
 
 // -------------------------------------------------------------------------------- //
-void guAlbumBrowser::OnAlbumEditLabelsClicked( const int albumid )
+void guAlbumBrowser::OnAlbumEditLabelsClicked( const guAlbumBrowserItem * albumitem )
 {
-    guListItems Labels;
-    wxArrayInt  Albums;
+    guListItems Albums;
+    Albums.Add( new guListItem( albumitem->m_AlbumId, albumitem->m_AlbumName ) );
+//    if( Albums.Count() )
+//    {
+        wxArrayInt AlbumIds;
+        AlbumIds.Add( albumitem->m_AlbumId );
+        guArrayListItems LabelSets = m_Db->GetAlbumsLabels( AlbumIds );
 
-    m_Db->GetLabels( &Labels, true );
-
-    Albums.Add( albumid );
-    guLabelEditor * LabelEditor = new guLabelEditor( this, m_Db, _( "Albums Labels Editor" ), false,
-                                         Labels, m_Db->GetAlbumsLabels( Albums ) );
-    if( LabelEditor )
-    {
-        if( LabelEditor->ShowModal() == wxID_OK )
+        guLabelEditor * LabelEditor = new guLabelEditor( this, m_Db, _( "Albums Labels Editor" ),
+                            false, &Albums, &LabelSets );
+        if( LabelEditor )
         {
-            m_Db->UpdateAlbumsLabels( Albums, LabelEditor->GetCheckedIds() );
+            if( LabelEditor->ShowModal() == wxID_OK )
+            {
+                m_Db->UpdateAlbumsLabels( LabelSets );
+            }
+
+            LabelEditor->Destroy();
+
+            if( m_FilterBtn->GetValue() ) // If its filtered we may need to refresh as the filter can contains the label condition
+            {
+                ReloadItems();
+                RefreshAll();
+            }
         }
-        LabelEditor->Destroy();
-    }
+//    }
 }
 
 // -------------------------------------------------------------------------------- //
@@ -1134,8 +1276,10 @@ void guAlbumBrowser::OnAlbumEditTracksClicked( const int albumid )
             m_Db->UpdateSongs( &Songs );
             UpdateImages( Songs, Images );
             UpdateLyrics( Songs, Lyrics );
+
             m_PlayerPanel->UpdatedTracks( &Songs );
         }
+        guImagePtrArrayClean( &Images );
         TrackEditor->Destroy();
     }
 }
@@ -1177,7 +1321,16 @@ void guAlbumBrowser::OnMouseWheel( wxMouseEvent& event )
 // -------------------------------------------------------------------------------- //
 void guAlbumBrowser::LibraryUpdated( void )
 {
+    int CurPage = m_NavSlider->GetValue();
+
     RefreshCount();
+
+    if( CurPage > m_NavSlider->GetMax() )
+        CurPage = m_NavSlider->GetMax();
+
+    m_ItemStart = CurPage * m_ItemCount;
+    UpdateNavLabel( CurPage );
+
     ReloadItems();
     RefreshAll();
 }
@@ -1197,9 +1350,12 @@ void  guAlbumBrowser::SetAlbumCover( const int albumid, const wxString &cover )
         wxImage CoverImage( cover );
         if( CoverImage.IsOk() )
         {
-            if( CoverImage.SaveFile( AlbumPath + wxT( "/cover.jpg" ), wxBITMAP_TYPE_JPEG ) )
+            guConfig * Config = ( guConfig * ) guConfig::Get();
+            wxArrayString SearchCovers = Config->ReadAStr( wxT( "Word" ), wxEmptyString, wxT( "CoverSearch" ) );
+            wxString CoverName = AlbumPath + ( SearchCovers.Count() ? SearchCovers[ 0 ] : wxT( "cover" ) ) + wxT( ".jpg" );
+            if( CoverImage.SaveFile( CoverName, wxBITMAP_TYPE_JPEG ) )
             {
-                m_Db->SetAlbumCover( albumid, AlbumPath + wxT( "/cover.jpg" ) );
+                m_Db->SetAlbumCover( albumid, CoverName );
             }
         }
         else
