@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 #include "AuiNotebook.h"
 
 #include "Utils.h"
+#include "Images.h"
 
 #include <wx/dc.h>
 
@@ -41,11 +42,11 @@ guAuiTabArt::guAuiTabArt() : wxAuiDefaultTabArt()
 {
     m_BgColor = wxSystemSettings::GetColour( wxSYS_COLOUR_3DHILIGHT );
     m_SelBgColor = wxSystemSettings::GetColour( wxSYS_COLOUR_HIGHLIGHT );
-    //m_TextFgColor.Set( m_base_colour.Red() ^ 0xFF, m_base_colour.Green() ^ 0xFF, m_base_colour.Blue() ^ 0xFF );
     m_TextFgColor = wxSystemSettings::GetColour( wxSYS_COLOUR_INACTIVECAPTIONTEXT );
     m_SelTextFgColour = wxSystemSettings::GetColour( wxSYS_COLOUR_BTNTEXT );
-    //m_TextFgColor = * wxBLUE;
-//    guLogMessage( wxT( "FGColor:%s" ), m_TextFgColor.GetAsString( wxC2S_HTML_SYNTAX ).c_str() );
+
+    m_disabled_close_bmp = guImage( guIMAGE_INDEX_tiny_close_normal );
+    m_active_close_bmp = guImage( guIMAGE_INDEX_tiny_close_highlight );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -299,7 +300,7 @@ void guAuiTabArt::DrawTab(wxDC &dc, wxWindow * wnd, const wxAuiNotebookPage &pag
         }
 
         wxRect rect(tab_x + tab_width - close_button_width - 1,
-                    tab_y + (tab_height/2) - (bmp.GetHeight()/2),
+                    tab_y + (tab_height/2) - (bmp.GetHeight()/2) - 2,
                     close_button_width,
                     tab_height);
         IndentPressedBitmap(&rect, close_button_state);
