@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -27,6 +27,7 @@ WX_DEFINE_OBJARRAY(guFilterItemArray);
 
 wxArrayString m_FilterFieldChoices;
 wxArrayString m_FilterTextOptionChoices;
+wxArrayString m_FilterLabelOptionChoices;
 wxArrayString m_FilterNumberOptionChoices;
 wxArrayString m_FilterYearOptionChoices;
 wxArrayString m_FilterDateOptionChoices;
@@ -41,6 +42,7 @@ void inline InitArrayStrings( void )
     {
         m_FilterFieldChoices.Add( _( "Title" ) );
         m_FilterFieldChoices.Add( _( "Artist" ) );
+        m_FilterFieldChoices.Add( _( "Album Artist" ) );
         m_FilterFieldChoices.Add( _( "Album" ) );
         m_FilterFieldChoices.Add( _( "Genre" ) );
         m_FilterFieldChoices.Add( _( "Label" ) );
@@ -57,16 +59,29 @@ void inline InitArrayStrings( void )
 //
 //    if( !m_FilterTextOptionChoices.Count() )
 //    {
-        m_FilterTextOptionChoices.Add( _("Contains") );
-        m_FilterTextOptionChoices.Add( _("Does not contain") );
-        m_FilterTextOptionChoices.Add( _("Equals") );
-        m_FilterTextOptionChoices.Add( _("Begins with") );
-        m_FilterTextOptionChoices.Add( _("Ends with") );
+        m_FilterTextOptionChoices.Add( _("contains") );
+        m_FilterTextOptionChoices.Add( _("does not contain") );
+        m_FilterTextOptionChoices.Add( _("is") );
+        m_FilterTextOptionChoices.Add( _("is not") );
+        m_FilterTextOptionChoices.Add( _("begins with") );
+        m_FilterTextOptionChoices.Add( _("ends with") );
+//    }
+//
+//    if( !m_FilterTextOptionChoices.Count() )
+//    {
+        m_FilterLabelOptionChoices.Add( _("contains") );
+        m_FilterLabelOptionChoices.Add( _("does not contain") );
+        m_FilterLabelOptionChoices.Add( _("is") );
+        m_FilterLabelOptionChoices.Add( _("is not") );
+        m_FilterLabelOptionChoices.Add( _("begins with") );
+        m_FilterLabelOptionChoices.Add( _("ends with") );
+        m_FilterLabelOptionChoices.Add( _("not set") );
 //    }
 //
 //    if( !m_FilterNumberOptionChoices.Count() )
 //    {
-        m_FilterNumberOptionChoices.Add( _( "equals" ) );
+        m_FilterNumberOptionChoices.Add( _( "is" ) );
+        m_FilterNumberOptionChoices.Add( _( "is not" ) );
         m_FilterNumberOptionChoices.Add( _( "at least" ) );
         m_FilterNumberOptionChoices.Add( _( "at most" ) );
 //    }
@@ -74,6 +89,7 @@ void inline InitArrayStrings( void )
 //    if( !m_FilterYearOptionChoices.Count() )
 //    {
         m_FilterYearOptionChoices.Add( _( "in" ) );
+        m_FilterYearOptionChoices.Add( _( "not in" ) );
         m_FilterYearOptionChoices.Add( _( "after" ) );
         m_FilterYearOptionChoices.Add( _( "before" ) );
 //    }
@@ -96,6 +112,7 @@ void inline InitArrayStrings( void )
 //    {
         m_SortChoices.Add( _( "Title" ) );
         m_SortChoices.Add( _( "Artist" ) );
+        m_SortChoices.Add( _( "Album Artist" ) );
         m_SortChoices.Add( _( "Album" ) );
         m_SortChoices.Add( _( "Genre" ) );
         m_SortChoices.Add( _( "Label" ) );
@@ -151,9 +168,9 @@ void guFilterItem::SetFilterLabel( void )
     {
         case guDYNAMIC_FILTER_TYPE_TITLE : // String
         case guDYNAMIC_FILTER_TYPE_ARTIST :
+        case guDYNAMIC_FILTER_TYPE_ALBUMARTIST :
         case guDYNAMIC_FILTER_TYPE_ALBUM :
         case guDYNAMIC_FILTER_TYPE_GENRE :
-        case guDYNAMIC_FILTER_TYPE_LABEL :
         case guDYNAMIC_FILTER_TYPE_COMPOSER :
         case guDYNAMIC_FILTER_TYPE_COMMENT :
         case guDYNAMIC_FILTER_TYPE_PATH :
@@ -163,6 +180,13 @@ void guFilterItem::SetFilterLabel( void )
             break;
         }
 
+        case guDYNAMIC_FILTER_TYPE_LABEL :
+        {
+            m_Label += m_FilterLabelOptionChoices[ m_Option ] + wxT( " " );
+            m_Label += m_Text;
+            break;
+        }
+
         case guDYNAMIC_FILTER_TYPE_YEAR : // Year
         {
             m_Label += m_FilterYearOptionChoices[ m_Option ];
@@ -446,9 +470,14 @@ guDynPlayListEditor::guDynPlayListEditor( wxWindow * parent, guDynPlayList * pla
 	m_FilterFieldChoice->SetSelection( 0 );
 	m_FilterEditSizer->Add( m_FilterFieldChoice, 0, wxALIGN_CENTER_VERTICAL|wxBOTTOM|wxRIGHT|wxLEFT, 5 );
 
-	m_FilterOptionChoice = new wxChoice( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, m_FilterTextOptionChoices, 0 );
-	m_FilterOptionChoice->SetSelection( 0 );
-	m_FilterEditSizer->Add( m_FilterOptionChoice, 0, wxBOTTOM|wxRIGHT|wxALIGN_CENTER_VERTICAL, 5 );
+	m_FilterTextOptionChoice = new wxChoice( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, m_FilterTextOptionChoices, 0 );
+	m_FilterTextOptionChoice->SetSelection( 0 );
+	m_FilterEditSizer->Add( m_FilterTextOptionChoice, 0, wxBOTTOM|wxRIGHT|wxALIGN_CENTER_VERTICAL, 5 );
+
+	m_FilterLabelOptionChoice = new wxChoice( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, m_FilterLabelOptionChoices, 0 );
+	m_FilterLabelOptionChoice->SetSelection( 0 );
+    m_FilterLabelOptionChoice->Show( false );
+	m_FilterEditSizer->Add( m_FilterLabelOptionChoice, 0, wxBOTTOM|wxRIGHT|wxALIGN_CENTER_VERTICAL, 5 );
 
 	m_FilterText = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
 	m_FilterEditSizer->Add( m_FilterText, 1, wxALIGN_CENTER_VERTICAL|wxBOTTOM|wxRIGHT, 5 );
@@ -589,7 +618,8 @@ guDynPlayListEditor::guDynPlayListEditor( wxWindow * parent, guDynPlayList * pla
 	m_FiltersListBox->Connect( wxEVT_COMMAND_LISTBOX_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFiltersSelected ), NULL, this );
 
 	m_FilterFieldChoice->Connect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFilterFieldSelected ), NULL, this );
-	m_FilterOptionChoice->Connect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFilterOptionSelected ), NULL, this );
+	m_FilterTextOptionChoice->Connect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFilterOptionSelected ), NULL, this );
+	m_FilterLabelOptionChoice->Connect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFilterOptionSelected ), NULL, this );
 	m_FilterText->Connect( wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( guDynPlayListEditor::OnFilterTextChanged ), NULL, this );
 	m_FilterRating->Connect( guEVT_RATING_CHANGED, guRatingEventHandler( guDynPlayListEditor::OnRatingChanged ), NULL, this );
 	m_FilterDateOption2Choice->Connect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFilterDateOption2Selected ), NULL, this );
@@ -609,7 +639,8 @@ guDynPlayListEditor::~guDynPlayListEditor()
 	m_FiltersListBox->Disconnect( wxEVT_COMMAND_LISTBOX_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFiltersSelected ), NULL, this );
 
 	m_FilterFieldChoice->Disconnect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFilterFieldSelected ), NULL, this );
-	m_FilterOptionChoice->Disconnect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFilterOptionSelected ), NULL, this );
+	m_FilterTextOptionChoice->Disconnect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFilterOptionSelected ), NULL, this );
+	m_FilterLabelOptionChoice->Disconnect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFilterOptionSelected ), NULL, this );
 	m_FilterText->Disconnect( wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( guDynPlayListEditor::OnFilterTextChanged ), NULL, this );
 	m_FilterRating->Disconnect( guEVT_RATING_CHANGED, guRatingEventHandler( guDynPlayListEditor::OnRatingChanged ), NULL, this );
 	m_FilterDateOption2Choice->Disconnect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guDynPlayListEditor::OnFilterDateOption2Selected ), NULL, this );
@@ -653,16 +684,22 @@ guFilterItem guDynPlayListEditor::GetFilterItem( void )
 {
     guFilterItem FilterItem;
     int FilterType = m_FilterFieldChoice->GetSelection();
-    if( FilterType < guDYNAMIC_FILTER_TYPE_YEAR )
+    if( FilterType == guDYNAMIC_FILTER_TYPE_LABEL )
     {
         FilterItem.Set( FilterType,
-                    m_FilterOptionChoice->GetSelection(),
+                    m_FilterLabelOptionChoice->GetSelection(),
+                    FilterItem.m_Text = m_FilterText->GetValue() );
+    }
+    else if( FilterType < guDYNAMIC_FILTER_TYPE_YEAR )
+    {
+        FilterItem.Set( FilterType,
+                    m_FilterTextOptionChoice->GetSelection(),
                     FilterItem.m_Text = m_FilterText->GetValue() );
     }
     else if( FilterType == guDYNAMIC_FILTER_TYPE_RATING )
     {
         FilterItem.Set( FilterType,
-                        m_FilterOptionChoice->GetSelection(),
+                        m_FilterTextOptionChoice->GetSelection(),
                         m_FilterRating->GetRating(),
                         0 );
     }
@@ -672,7 +709,7 @@ guFilterItem guDynPlayListEditor::GetFilterItem( void )
         unsigned long min = m_LengthMinutes->GetValue();
         unsigned long sec = m_LengthSeconds->GetValue();
         FilterItem.Set( FilterType,
-                        m_FilterOptionChoice->GetSelection(),
+                        m_FilterTextOptionChoice->GetSelection(),
                         ( hour * 3600 ) + ( min * 60 ) + sec,
                         0 );
 
@@ -682,7 +719,7 @@ guFilterItem guDynPlayListEditor::GetFilterItem( void )
         unsigned long value = 0;
         m_FilterText->GetValue().ToULong( &value );
         FilterItem.Set( FilterType,
-                        m_FilterOptionChoice->GetSelection(),
+                        m_FilterTextOptionChoice->GetSelection(),
                         value,
                         m_FilterDateOption2Choice->GetSelection() );
     }
@@ -691,7 +728,7 @@ guFilterItem guDynPlayListEditor::GetFilterItem( void )
         unsigned long value = 0;
         m_FilterText->GetValue().ToULong( &value );
         FilterItem.Set( FilterType,
-                        m_FilterOptionChoice->GetSelection(),
+                        m_FilterTextOptionChoice->GetSelection(),
                         value,
                         0 );
     }
@@ -724,6 +761,8 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
     if( FilterType == guDYNAMIC_FILTER_TYPE_RATING )
     {
         m_FilterText->Show( false );
+        m_FilterTextOptionChoice->Show( true );
+        m_FilterLabelOptionChoice->Show( false );
         m_FilterRating->Show( true );
         m_FilterDateOption2Choice->Show( false );
         m_LengthHours->Show( false );
@@ -735,6 +774,8 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
     else if( FilterType == guDYNAMIC_FILTER_TYPE_LENGTH )
     {
         m_FilterText->Show( false );
+        m_FilterTextOptionChoice->Show( true );
+        m_FilterLabelOptionChoice->Show( false );
         m_FilterRating->Show( false );
         m_FilterDateOption2Choice->Show( false );
         m_LengthHours->Show( true );
@@ -747,6 +788,8 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
     else if( FilterType > guDYNAMIC_FILTER_TYPE_PLAYCOUNT )
     {
         m_FilterText->Show( true );
+        m_FilterTextOptionChoice->Show( true );
+        m_FilterLabelOptionChoice->Show( false );
         m_FilterRating->Show( false );
         m_FilterDateOption2Choice->Show( true );
         m_LengthHours->Show( false );
@@ -755,9 +798,24 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
         m_LengthSeparator2->Show( false );
         m_LengthSeconds->Show( false );
     }
+    else if( FilterType == guDYNAMIC_FILTER_TYPE_LABEL )
+    {
+        m_FilterText->Show( true );
+        m_FilterTextOptionChoice->Show( false );
+        m_FilterLabelOptionChoice->Show( true );
+        m_FilterRating->Show( false );
+        m_FilterDateOption2Choice->Show( false );
+        m_LengthHours->Show( false );
+        m_LengthSeparator1->Show( false );
+        m_LengthMinutes->Show( false );
+        m_LengthSeparator2->Show( false );
+        m_LengthSeconds->Show( false );
+    }
     else
     {
         m_FilterText->Show( true );
+        m_FilterTextOptionChoice->Show( true );
+        m_FilterLabelOptionChoice->Show( false );
         m_FilterRating->Show( false );
         m_FilterDateOption2Choice->Show( false );
         m_LengthHours->Show( false );
@@ -767,14 +825,14 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
         m_LengthSeconds->Show( false );
     }
 
-    m_FilterOptionChoice->Clear();
+    m_FilterTextOptionChoice->Clear();
 
     if( FilterType < guDYNAMIC_FILTER_TYPE_YEAR )
     {
         count = m_FilterTextOptionChoices.Count();
         for( index = 0; index < count; index++ )
         {
-            m_FilterOptionChoice->Append( m_FilterTextOptionChoices[ index ] );
+            m_FilterTextOptionChoice->Append( m_FilterTextOptionChoices[ index ] );
         }
     }
     else if( FilterType == guDYNAMIC_FILTER_TYPE_YEAR )
@@ -782,7 +840,7 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
         count = m_FilterYearOptionChoices.Count();
         for( index = 0; index < count; index++ )
         {
-            m_FilterOptionChoice->Append( m_FilterYearOptionChoices[ index ] );
+            m_FilterTextOptionChoice->Append( m_FilterYearOptionChoices[ index ] );
         }
     }
     else if( FilterType < guDYNAMIC_FILTER_TYPE_LASTPLAY )
@@ -790,7 +848,7 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
         count = m_FilterNumberOptionChoices.Count();
         for( index = 0; index < count; index++ )
         {
-            m_FilterOptionChoice->Append( m_FilterNumberOptionChoices[ index ] );
+            m_FilterTextOptionChoice->Append( m_FilterNumberOptionChoices[ index ] );
         }
     }
     else if( FilterType > guDYNAMIC_FILTER_TYPE_PLAYCOUNT )
@@ -798,10 +856,11 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
         count = m_FilterDateOptionChoices.Count();
         for( index = 0; index < count; index++ )
         {
-            m_FilterOptionChoice->Append( m_FilterDateOptionChoices[ index ] );
+            m_FilterTextOptionChoice->Append( m_FilterDateOptionChoices[ index ] );
         }
     }
-    m_FilterOptionChoice->SetSelection( 0 );
+    m_FilterTextOptionChoice->SetSelection( 0 );
+    m_FilterLabelOptionChoice->SetSelection( 0 );
 
     m_FilterEditSizer->Layout();
 }
@@ -816,7 +875,10 @@ void guDynPlayListEditor::OnFiltersSelected( wxCommandEvent &event )
         UpdateEditor( FilterItem->m_Type );
         m_FilterFieldChoice->SetSelection( FilterItem->m_Type );
         //guLogMessage( wxT( "Type : %u   Option: %u" ), FilterItem->m_Type, FilterItem->m_Option );
-        m_FilterOptionChoice->SetSelection( FilterItem->m_Option );
+        if( FilterItem->m_Type == guDYNAMIC_FILTER_TYPE_LABEL )
+            m_FilterLabelOptionChoice->SetSelection( FilterItem->m_Option );
+        else
+            m_FilterTextOptionChoice->SetSelection( FilterItem->m_Option );
         if( FilterItem->m_Type < guDYNAMIC_FILTER_TYPE_YEAR )
         {
             m_FilterText->SetValue( FilterItem->m_Text );
@@ -877,6 +939,14 @@ void guDynPlayListEditor::OnFilterFieldSelected( wxCommandEvent &event )
 // -------------------------------------------------------------------------------- //
 void guDynPlayListEditor::OnFilterOptionSelected( wxCommandEvent &event )
 {
+    bool IsNotSetLabel = m_FilterFieldChoice->GetSelection() == guDYNAMIC_FILTER_TYPE_LENGTH ||
+                          ( ( m_FilterFieldChoice->GetSelection() == guDYNAMIC_FILTER_TYPE_LABEL ) &&
+                          ( m_FilterLabelOptionChoice->GetSelection() == guDYNAMIC_FILTER_OPTION_LABELS_NOTSET ) );
+    m_FilterText->Enable( !IsNotSetLabel );
+    if( IsNotSetLabel )
+        m_FilterText->SetValue( wxEmptyString );
+    m_FilterAdd->Enable( IsNotSetLabel );
+
     if( m_CurFilter != wxNOT_FOUND )
     {
         m_HasChanged = true;
@@ -906,7 +976,10 @@ void guDynPlayListEditor::OnFilterTextChanged( wxCommandEvent& event )
     }
 
 
-    m_FilterAdd->Enable( !m_FilterText->GetValue().IsEmpty() );
+    m_FilterAdd->Enable( !m_FilterText->GetValue().IsEmpty() ||
+                         ( ( m_FilterFieldChoice->GetSelection() == guDYNAMIC_FILTER_TYPE_LABEL ) &&
+                         ( m_FilterLabelOptionChoice->GetSelection() == guDYNAMIC_FILTER_OPTION_LABELS_NOTSET ) ) );
+
     if( m_CurFilter != wxNOT_FOUND )
     {
         m_HasChanged = true;
