@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include <wx/xml/xml.h>
 
 WX_DEFINE_OBJARRAY(guAlbumInfoArray);
+WX_DEFINE_OBJARRAY(guTopTrackInfoArray);
 WX_DEFINE_OBJARRAY(guSimilarArtistInfoArray);
 WX_DEFINE_OBJARRAY(guSimilarTrackInfoArray);
 WX_DEFINE_OBJARRAY(guEventInfoArray);
@@ -129,12 +130,11 @@ wxString guLastFMRequest::DoRequest( const bool AddSign, const bool IsGetAction
         if( RetVal.IsEmpty() )
         {
             // Only with a UserAgent is accepted the Charset requested
-            http.AddHeader( wxT( "User-Agent: Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.5) Gecko/2008121622 Ubuntu/8.10 (intrepid) Firefox/3.0.5" ) );
+            http.AddHeader( wxT( "User-Agent: " ) guDEFAULT_BROWSER_USER_AGENT );
             http.AddHeader( wxT( "Accept: text/html" ) );
             http.AddHeader( wxT( "Accept-Charset: utf-8" ) );
-
             //guLogMessage( wxT( "LastFM.DoRequest %s\n" ), UrlStr.c_str() );
-
+            http.SetOpt( CURLOPT_FOLLOWLOCATION, 1 );
             http.Get( Buffer, UrlStr );
 
             if( Buffer )
@@ -158,6 +158,7 @@ wxString guLastFMRequest::DoRequest( const bool AddSign, const bool IsGetAction
     }
     else
     {
+        //guLogMessage( wxT( "LastFM.DoRequest POST %s\n" ), UrlStr.c_str() );
         http.Post( UrlStr.Mid( 1 ).char_str(), UrlStr.Length() - 1, LASTFM_API_ROOT );
     }
 
@@ -1102,12 +1103,100 @@ wxArrayString guLastFM::ArtistGetTopTags( const wxString &Artist )
 }
 
 // -------------------------------------------------------------------------------- //
-wxArrayString guLastFM::ArtistGetTopTracks( const wxString &Artist )
+void GetArtistTopTrackArtist( wxXmlNode * xmlnode, guTopTrackInfo * toptrack )
+{
+//                <name>Modern Talking</name>
+//                <mbid>98913495-8867-43b3-aa8d-db88ee4d4cdc</mbid>
+//                <url>http://www.last.fm/music/Modern+Talking</url>
+    while( xmlnode )
+    {
+        if( xmlnode->GetName() == wxT( "name" ) )
+        {
+            toptrack->m_ArtistName = xmlnode->GetNodeContent();
+        }
+        xmlnode = xmlnode->GetNext();
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+void GetArtistTopTrack( wxXmlNode * xmlnode, guTopTrackInfo * toptrack )
+{
+//              <name>Brother Louie</name>
+//              <playcount>34061</playcount>
+//              <listeners>11721</listeners>
+//              <mbid/>
+//              <url>http://www.last.fm/music/Modern+Talking/_/Brother+Louie</url>
+//              <streamable fulltrack="0">1</streamable>
+//              <artist>
+//                <name>Modern Talking</name>
+//                <mbid>98913495-8867-43b3-aa8d-db88ee4d4cdc</mbid>
+//                <url>http://www.last.fm/music/Modern+Talking</url>
+//              </artist>
+//              <image size="small">http://userserve-ak.last.fm/serve/34s/31763881.jpg</image>
+//              <image size="medium">http://userserve-ak.last.fm/serve/64s/31763881.jpg</image>
+//              <image size="large">http://userserve-ak.last.fm/serve/126/31763881.jpg</image>
+//              <image size="extralarge">http://userserve-ak.last.fm/serve/300x300/31763881.jpg</image>
+    while( xmlnode )
+    {
+        if( xmlnode->GetName() == wxT( "name" ) )
+        {
+            toptrack->m_TrackName = xmlnode->GetNodeContent();
+        }
+        else if( xmlnode->GetName() == wxT( "playcount" ) )
+        {
+            xmlnode->GetNodeContent().ToLong( &toptrack->m_PlayCount );
+        }
+        else if( xmlnode->GetName() == wxT( "listeners" ) )
+        {
+            xmlnode->GetNodeContent().ToLong( &toptrack->m_Listeners );
+        }
+        else if( xmlnode->GetName() == wxT( "artist" ) )
+        {
+            GetArtistTopTrackArtist( xmlnode->GetChildren(), toptrack );
+        }
+        else if( xmlnode->GetName() == wxT( "url" ) )
+        {
+            toptrack->m_Url = xmlnode->GetNodeContent();
+        }
+        else if( xmlnode->GetName() == wxT( "image" ) )
+        {
+            wxString ImgSize;
+            xmlnode->GetPropVal( wxT( "size" ), &ImgSize );
+            if( ImgSize == wxT( "large" ) && toptrack->m_ImageLink.IsEmpty() )
+            {
+                toptrack->m_ImageLink = xmlnode->GetNodeContent();
+            }
+            else if( ImgSize == wxT( "extralarge" ) && !xmlnode->GetNodeContent().IsEmpty() )
+            {
+                toptrack->m_ImageLink = xmlnode->GetNodeContent();
+            }
+        }
+        xmlnode = xmlnode->GetNext();
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+void GetArtistTopTracks( wxXmlNode * xmlnode, guTopTrackInfoArray * toptracks )
+{
+    while( xmlnode )
+    {
+        if( xmlnode->GetName() == wxT( "track" ) )
+        {
+            guTopTrackInfo TopTrackInfo;
+            GetArtistTopTrack( xmlnode->GetChildren(), &TopTrackInfo );
+            toptracks->Add( TopTrackInfo );
+        }
+        xmlnode = xmlnode->GetNext();
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+guTopTrackInfoArray guLastFM::ArtistGetTopTracks( const wxString &Artist )
 {
     guLastFMRequest Req; // = guLastFMRequest();
     wxString Res;
     wxString Status;
-    wxArrayString RetVal;
+    guTopTrackInfoArray RetVal;
 
     Req.SetMethod( wxT( "artist.gettoptracks" ) );
     Req.AddArgument( wxT( "api_key" ), LASTFM_API_KEY );
@@ -1122,6 +1211,27 @@ wxArrayString guLastFM::ArtistGetTopTracks( const wxString &Artist )
         wxXmlNode * XmlNode = XmlDoc.GetRoot();
         if( XmlNode )
         {
+//        <lfm status="ok">
+//          <toptracks artist="Modern Talking">
+//            <track rank="1">
+//              <name>Brother Louie</name>
+//              <playcount>34061</playcount>
+//              <listeners>11721</listeners>
+//              <mbid/>
+//              <url>http://www.last.fm/music/Modern+Talking/_/Brother+Louie</url>
+//              <streamable fulltrack="0">1</streamable>
+//              <artist>
+//                <name>Modern Talking</name>
+//                <mbid>98913495-8867-43b3-aa8d-db88ee4d4cdc</mbid>
+//                <url>http://www.last.fm/music/Modern+Talking</url>
+//              </artist>
+//              <image size="small">http://userserve-ak.last.fm/serve/34s/31763881.jpg</image>
+//              <image size="medium">http://userserve-ak.last.fm/serve/64s/31763881.jpg</image>
+//              <image size="large">http://userserve-ak.last.fm/serve/126/31763881.jpg</image>
+//              <image size="extralarge">http://userserve-ak.last.fm/serve/300x300/31763881.jpg</image>
+//            </track>
+//            <track rank="2">
+
             if( XmlNode->GetName() == wxT( "lfm" ) )
             {
                 //printf( XmlNode->GetName().char_str() ); printf( "\n" );
@@ -1132,21 +1242,7 @@ wxArrayString guLastFM::ArtistGetTopTracks( const wxString &Artist )
                     XmlNode = XmlNode->GetChildren();
                     if( XmlNode && XmlNode->GetName() == wxT( "toptracks" ) )
                     {
-                        wxXmlNode * XmlSubNode = XmlNode->GetChildren();
-                        while( XmlSubNode )
-                        {
-                            XmlNode = XmlSubNode->GetChildren();
-                            while( XmlNode )
-                            {
-                                if( XmlNode->GetName() == wxT( "name" ) )
-                                {
-                                    RetVal.Add( XmlNode->GetNodeContent() );
-                                    break;
-                                }
-                                XmlNode = XmlNode->GetNext();
-                            }
-                            XmlSubNode = XmlSubNode->GetNext();
-                        }
+                        GetArtistTopTracks( XmlNode->GetChildren(), &RetVal );
                     }
                 }
                 else if( Status == wxT( "failed" ) )
@@ -1746,3 +1842,91 @@ bool guLastFM::TrackRemoveTag( const wxString &Artist, const wxString &Track, co
 }
 
 // -------------------------------------------------------------------------------- //
+bool guLastFM::TrackLove( const wxString &artist, const wxString &title )
+{
+    guLastFMRequest     Req; // = guLastFMRequest();
+    wxString            Res;
+    wxString            Status;
+    wxString            ItemName;
+
+    Req.SetMethod( wxT( "track.love" ) );
+    Req.AddArgument( wxT( "api_key" ), LASTFM_API_KEY );
+    Req.AddArgument( wxT( "artist" ), artist, true );
+    Req.AddArgument( wxT( "track" ), title, true );
+    Res = Req.DoRequest( false );
+
+    m_ErrorCode = wxNOT_FOUND;
+    if( Res.Length() )
+    {
+        wxStringInputStream ins( Res );
+        wxXmlDocument XmlDoc( ins );
+        wxXmlNode * XmlNode = XmlDoc.GetRoot();
+        if( XmlNode )
+        {
+            if( XmlNode->GetName() == wxT( "lfm" ) )
+            {
+                XmlNode->GetPropVal( wxT( "status" ), &Status );
+                if( Status == wxT( "ok" ) )
+                {
+                    return true;
+                }
+                else if( Status == wxT( "failed" ) )
+                {
+                    XmlNode = XmlNode->GetChildren();
+                    if( XmlNode && XmlNode->GetName() == wxT( "error" ) )
+                    {
+                        XmlNode->GetPropVal( wxT( "code" ), &Status );
+                        Status.ToLong( ( long * ) &m_ErrorCode );
+                    }
+                }
+            }
+        }
+    }
+    return false;
+}
+
+// -------------------------------------------------------------------------------- //
+bool guLastFM::TrackBan( const wxString &artist, const wxString &title )
+{
+    guLastFMRequest     Req;
+    wxString            Res;
+    wxString            Status;
+    wxString            ItemName;
+
+    Req.SetMethod( wxT( "track.ban" ) );
+    Req.AddArgument( wxT( "api_key" ), LASTFM_API_KEY );
+    Req.AddArgument( wxT( "artist" ), artist, true );
+    Req.AddArgument( wxT( "track" ), title, true );
+    Res = Req.DoRequest( false );
+
+    m_ErrorCode = wxNOT_FOUND;
+    if( Res.Length() )
+    {
+        wxStringInputStream ins( Res );
+        wxXmlDocument XmlDoc( ins );
+        wxXmlNode * XmlNode = XmlDoc.GetRoot();
+        if( XmlNode )
+        {
+            if( XmlNode->GetName() == wxT( "lfm" ) )
+            {
+                XmlNode->GetPropVal( wxT( "status" ), &Status );
+                if( Status == wxT( "ok" ) )
+                {
+                    return true;
+                }
+                else if( Status == wxT( "failed" ) )
+                {
+                    XmlNode = XmlNode->GetChildren();
+                    if( XmlNode && XmlNode->GetName() == wxT( "error" ) )
+                    {
+                        XmlNode->GetPropVal( wxT( "code" ), &Status );
+                        Status.ToLong( ( long * ) &m_ErrorCode );
+                    }
+                }
+            }
+        }
+    }
+    return false;
+}
+
+// -------------------------------------------------------------------------------- //
