@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -34,6 +34,11 @@
 
 #define GULASTFMINFO_MAXITEMS  12
 
+// -------------------------------------------------------------------------------- //
+class guHtmlWindow  : public wxHtmlWindow
+{
+};
+
 // -------------------------------------------------------------------------------- //
 class guLastFMInfo
 {
@@ -136,6 +141,7 @@ class guLastFMTrackInfo : public guLastFMInfo
   public:
     guSimilarTrackInfo *   m_Track;
     int                    m_TrackId;
+    int                    m_ArtistId;
 
     guLastFMTrackInfo() { m_Track = NULL; m_TrackId = wxNOT_FOUND; };
 
@@ -145,6 +151,7 @@ class guLastFMTrackInfo : public guLastFMInfo
     {
         m_Track = track;
         m_TrackId = wxNOT_FOUND;
+        m_ArtistId = wxNOT_FOUND;
     };
 
     ~guLastFMTrackInfo()
@@ -155,6 +162,33 @@ class guLastFMTrackInfo : public guLastFMInfo
 };
 WX_DECLARE_OBJARRAY(guLastFMTrackInfo, guLastFMTrackInfoArray);
 
+// -------------------------------------------------------------------------------- //
+class guLastFMTopTrackInfo : public guLastFMInfo
+{
+  public:
+    guTopTrackInfo *       m_TopTrack;
+    int                    m_TrackId;
+    int                    m_ArtistId;
+
+    guLastFMTopTrackInfo() { m_TopTrack = NULL; m_TrackId = wxNOT_FOUND; };
+
+    guLastFMTopTrackInfo( int index, wxImage * image = NULL,
+                  guTopTrackInfo * track = NULL ) :
+            guLastFMInfo( index, image )
+    {
+        m_TopTrack = track;
+        m_TrackId = wxNOT_FOUND;
+        m_ArtistId = wxNOT_FOUND;
+    };
+
+    ~guLastFMTopTrackInfo()
+    {
+        if( m_TopTrack )
+            delete m_TopTrack;
+    };
+};
+WX_DECLARE_OBJARRAY(guLastFMTopTrackInfo, guLastFMTopTrackInfoArray);
+
 // -------------------------------------------------------------------------------- //
 class guLastFMEventInfo : public guLastFMInfo
 {
@@ -236,6 +270,23 @@ class guFetchAlbumInfoThread : public guFetchLastFMInfoThread
     friend class guDownloadAlbumImageThread;
 };
 
+// -------------------------------------------------------------------------------- //
+class guFetchTopTracksInfoThread : public guFetchLastFMInfoThread
+{
+  protected:
+    guDbCache *             m_DbCache;
+    wxString                m_ArtistName;
+    int                     m_ArtistId;
+
+  public:
+    guFetchTopTracksInfoThread( guLastFMPanel * lastfmpanel, guDbCache * dbcache, const wxChar * artistname );
+    ~guFetchTopTracksInfoThread();
+
+    virtual ExitCode Entry();
+
+    friend class guDownloadAlbumImageThread;
+};
+
 // -------------------------------------------------------------------------------- //
 class guFetchSimilarArtistInfoThread : public guFetchLastFMInfoThread
 {
@@ -292,13 +343,19 @@ class guLastFMInfoCtrl : public wxPanel
     virtual void        CreateControls( wxWindow * parent );
     virtual void        OnPlayClicked( wxCommandEvent &event );
     virtual void        OnEnqueueClicked( wxCommandEvent &event );
+    virtual void        OnEnqueueAsNextClicked( wxCommandEvent &event );
     virtual int         GetSelectedTracks( guTrackArray * tracks );
+    virtual void        OnSongSelectName( wxCommandEvent &event );
     virtual void        OnArtistSelectName( wxCommandEvent &event );
     virtual void        OnAlbumSelectName( wxCommandEvent &event );
 
     virtual void        OnBitmapMouseOver( wxCommandEvent &event );
     virtual wxString    GetBitmapImageUrl( void );
 
+    virtual void        OnMouse( wxMouseEvent &event );
+
+    virtual bool        ItemWasFound( void ) { return false; }
+
   public :
 	guLastFMInfoCtrl( wxWindow * parent, guDbLibrary * db, guDbCache * dbcache, guPlayerPanel * playerpanel, bool createcontrols = true );
 	~guLastFMInfoCtrl();
@@ -335,6 +392,8 @@ class guArtistInfoCtrl : public guLastFMInfoCtrl
 
     virtual wxString    GetBitmapImageUrl( void ) { return m_Info ? m_Info->m_ImageUrl : wxT( "" ); };
 
+    virtual bool        ItemWasFound( void ) { return m_Info && ( m_Info->m_ArtistId != wxNOT_FOUND ); };
+
   protected :
     virtual void        OnCopyToClipboard( wxCommandEvent &event );
 
@@ -365,6 +424,8 @@ class guAlbumInfoCtrl : public guLastFMInfoCtrl
 
     virtual wxString    GetBitmapImageUrl( void ) { return m_Info ? m_Info->m_ImageUrl : wxT( "" ); };
 
+    virtual bool        ItemWasFound( void ) { return m_Info && ( m_Info->m_AlbumId != wxNOT_FOUND ); };
+
   public :
     guAlbumInfoCtrl( wxWindow * parent, guDbLibrary * db, guDbCache * dbcache, guPlayerPanel * playerpanel );
     ~guAlbumInfoCtrl();
@@ -392,6 +453,8 @@ class guSimilarArtistInfoCtrl : public guLastFMInfoCtrl
 
     virtual wxString    GetBitmapImageUrl( void ) { return m_Info ? m_Info->m_ImageUrl : wxT( "" ); };
 
+    virtual bool        ItemWasFound( void ) { return m_Info && ( m_Info->m_ArtistId != wxNOT_FOUND ); };
+
   public :
     guSimilarArtistInfoCtrl( wxWindow * parent, guDbLibrary * db, guDbCache * dbcache, guPlayerPanel * playerpanel );
     ~guSimilarArtistInfoCtrl();
@@ -414,10 +477,15 @@ class guTrackInfoCtrl : public guLastFMInfoCtrl
     virtual wxString    GetItemUrl( void );
     //void                OnClick( wxMouseEvent &event );
     virtual int         GetSelectedTracks( guTrackArray * tracks );
+    virtual void        OnSongSelectName( wxCommandEvent &event );
     virtual void        OnArtistSelectName( wxCommandEvent &event );
 
+    void                OnSelectArtist( wxCommandEvent &event );
+
     virtual wxString    GetBitmapImageUrl( void ) { return m_Info ? m_Info->m_ImageUrl : wxT( "" ); };
 
+    virtual bool        ItemWasFound( void ) { return m_Info && ( m_Info->m_TrackId != wxNOT_FOUND ); };
+
   public :
     guTrackInfoCtrl( wxWindow * parent, guDbLibrary * db, guDbCache * dbcache, guPlayerPanel * playerpanel );
     ~guTrackInfoCtrl();
@@ -428,6 +496,37 @@ class guTrackInfoCtrl : public guLastFMInfoCtrl
 };
 WX_DEFINE_ARRAY_PTR( guTrackInfoCtrl *, guTrackInfoCtrlArray );
 
+// -------------------------------------------------------------------------------- //
+class guTopTrackInfoCtrl : public guLastFMInfoCtrl
+{
+  private :
+    guLastFMTopTrackInfo * m_Info;
+
+    virtual void        OnContextMenu( wxContextMenuEvent& event );
+    virtual void        CreateContextMenu( wxMenu * Menu );
+    virtual wxString    GetSearchText( void );
+    virtual wxString    GetItemUrl( void );
+    //void                OnClick( wxMouseEvent &event );
+    virtual int         GetSelectedTracks( guTrackArray * tracks );
+    virtual void        OnSongSelectName( wxCommandEvent &event );
+    virtual void        OnArtistSelectName( wxCommandEvent &event );
+
+    void                OnSelectArtist( wxCommandEvent &event );
+
+    virtual wxString    GetBitmapImageUrl( void ) { return m_Info ? m_Info->m_ImageUrl : wxT( "" ); };
+
+    virtual bool        ItemWasFound( void ) { return m_Info && ( m_Info->m_TrackId != wxNOT_FOUND ); };
+
+  public :
+    guTopTrackInfoCtrl( wxWindow * parent, guDbLibrary * db, guDbCache * dbcache, guPlayerPanel * playerpanel );
+    ~guTopTrackInfoCtrl();
+
+    void SetInfo( guLastFMTopTrackInfo * info );
+    virtual void Clear( void );
+
+};
+WX_DEFINE_ARRAY_PTR( guTopTrackInfoCtrl *, guTopTrackInfoCtrlArray );
+
 // -------------------------------------------------------------------------------- //
 class guEventInfoCtrl : public guLastFMInfoCtrl
 {
@@ -443,6 +542,8 @@ class guEventInfoCtrl : public guLastFMInfoCtrl
 
     virtual wxString    GetBitmapImageUrl( void ) { return m_Info ? m_Info->m_ImageUrl : wxT( "" ); };
 
+    virtual bool        ItemWasFound( void ) { return true; };
+
   public :
     guEventInfoCtrl( wxWindow * parent, guDbLibrary * db, guDbCache * dbcache, guPlayerPanel * playerpanel );
     ~guEventInfoCtrl();
@@ -473,6 +574,9 @@ class guLastFMPanel : public wxScrolledWindow
 	guFetchAlbumInfoThread *            m_AlbumsUpdateThread;
 	wxMutex                             m_AlbumsUpdateThreadMutex;
 
+	guFetchTopTracksInfoThread *        m_TopTracksUpdateThread;
+	wxMutex                             m_TopTracksUpdateThreadMutex;
+
 	guFetchSimilarArtistInfoThread *    m_ArtistsUpdateThread;
 	wxMutex                             m_ArtistsUpdateThreadMutex;
 
@@ -505,6 +609,11 @@ class guLastFMPanel : public wxScrolledWindow
 	wxGridSizer *                       m_AlbumsSizer;
     guAlbumInfoCtrlArray                m_AlbumInfoCtrls;
 
+	bool                                m_ShowTopTracks;
+    wxStaticText *                      m_TopTracksStaticText;
+	wxGridSizer *                       m_TopTracksSizer;
+	guTopTrackInfoCtrlArray             m_TopTrackInfoCtrls;
+
 	bool                                m_ShowArtists;
     wxStaticText *                      m_ArtistsStaticText;
 	wxGridSizer *                       m_ArtistsSizer;
@@ -526,6 +635,7 @@ class guLastFMPanel : public wxScrolledWindow
 
     void    OnUpdateArtistInfo( wxCommandEvent &event );
     void    OnUpdateAlbumItem( wxCommandEvent &event );
+    void    OnUpdateTopTrackItem( wxCommandEvent &event );
     void    OnUpdateArtistItem( wxCommandEvent &event );
     void    OnUpdateTrackItem( wxCommandEvent &event );
     void    OnUpdateEventItem( wxCommandEvent &event );
@@ -534,6 +644,7 @@ class guLastFMPanel : public wxScrolledWindow
 //	  void    OnHtmlLinkClicked( wxHtmlLinkEvent& event );
 	void    OnArInfoTitleDClicked( wxMouseEvent &event );
     void    OnTopAlbumsTitleDClick( wxMouseEvent &event );
+    void    OnTopTracksTitleDClick( wxMouseEvent &event );
 	void    OnSimArTitleDClick( wxMouseEvent &event );
 	void    OnSimTrTitleDClick( wxMouseEvent &event );
 	void    OnEventsTitleDClick( wxMouseEvent &event );
@@ -545,12 +656,12 @@ class guLastFMPanel : public wxScrolledWindow
     void    OnTextUpdated( wxCommandEvent& event );
     void    OnSearchBtnClick( wxCommandEvent &event );
 
-	void    OnAlbumTextClicked( wxMouseEvent &event );
-	void    OnArtistTextClicked( wxMouseEvent &event );
-	void    OnTrackTextClicked( wxMouseEvent &event );
-	void    OnAlbumTextRightClicked( wxMouseEvent &event );
-	void    OnArtistTextRightClicked( wxMouseEvent &event );
-	void    OnTrackTextRightClicked( wxMouseEvent &event );
+//	void    OnAlbumTextClicked( wxMouseEvent &event );
+//	void    OnArtistTextClicked( wxMouseEvent &event );
+//	void    OnTrackTextClicked( wxMouseEvent &event );
+//	void    OnAlbumTextRightClicked( wxMouseEvent &event );
+//	void    OnArtistTextRightClicked( wxMouseEvent &event );
+//	void    OnTrackTextRightClicked( wxMouseEvent &event );
 
     void    UpdateTrackChangeButtons( void );
 
@@ -568,6 +679,7 @@ class guLastFMPanel : public wxScrolledWindow
 
     friend class guFetchLastFMInfoThread;
     friend class guFetchAlbumInfoThread;
+    friend class guFetchTopTracksInfoThread;
     friend class guFetchSimilarArtistInfoThread;
     friend class guFetchTrackInfoThread;
     friend class guDownloadImageThread;
