@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -50,7 +50,7 @@
 
 #define guLISTVIEW_ITEM_MARGIN      2
 
-#define guLISTVIEW_TIMER_TIMEOUT    500
+#define guLISTVIEW_TIMER_TIMEOUT    1000
 #define guLISTVIEW_MIN_COL_SIZE     20
 
 DEFINE_EVENT_TYPE( guEVT_LISTBOX_ITEM_COL_CLICKED )
@@ -58,6 +58,7 @@ DEFINE_EVENT_TYPE( guEVT_LISTBOX_ITEM_COL_RCLICKED )
 
 WX_DEFINE_OBJARRAY(guListViewColumnArray);
 
+static wxString FindChars = wxT( "abcdefghijklmnñopqrstuvwxyzABCDEFGHIJKLMNÑOPQRSTUVWXYZ0123456789 @&*+-_=/\\ºª:.,;çÇ<>$%()'\"|" );
 
 // -------------------------------------------------------------------------------- //
 // guListViewColEdit
@@ -98,7 +99,10 @@ class guListViewClientTimer : public wxTimer
   public:
     guListViewClient * m_ListViewClient;
     //
-    guListViewClientTimer( guListViewClient * listviewclient ) { m_ListViewClient = listviewclient; }
+    guListViewClientTimer( guListViewClient * listviewclient )
+    {
+        m_ListViewClient = listviewclient;
+    }
 
     //Called each time the timer's timeout expires
     void Notify();
@@ -231,12 +235,6 @@ void guListView::OnChangedSize( wxSizeEvent &event )
     event.Skip();
 }
 
-// -------------------------------------------------------------------------------- //
-wxString guListView::OnGetItemText( const int row, const int col ) const
-{
-    return wxEmptyString;
-}
-
 // -------------------------------------------------------------------------------- //
 void guListView::DrawItem( wxDC &dc, const wxRect &rect, const int row, const int col ) const
 {
@@ -253,11 +251,6 @@ void guListView::DrawBackground( wxDC &dc, const wxRect &rect, const int row, co
     m_ListBox->DoDrawBackground( dc, rect, row, col );
 }
 
-// -------------------------------------------------------------------------------- //
-void guListView::CreateContextMenu( wxMenu * menu ) const
-{
-}
-
 // -------------------------------------------------------------------------------- //
 void guListView::OnContextMenu( wxContextMenuEvent& event )
 {
@@ -278,11 +271,6 @@ void guListView::OnContextMenu( wxContextMenuEvent& event )
     PopupMenu( &Menu, Point.x, Point.y );
 }
 
-// -------------------------------------------------------------------------------- //
-int guListView::GetItemCount( void ) const
-{
-    return m_ListBox->GetItemCount();
-}
 
 // -------------------------------------------------------------------------------- //
 void guListView::SetItemCount( const int count )
@@ -292,12 +280,6 @@ void guListView::SetItemCount( const int count )
         m_Header->RefreshWidth();
 }
 
-// -------------------------------------------------------------------------------- //
-void guListView::SetItemHeight( const int height )
-{
-    m_ListBox->SetItemHeigth( height );
-}
-
 // -------------------------------------------------------------------------------- //
 wxCoord guListView::OnMeasureItem( size_t n ) const
 {
@@ -314,30 +296,6 @@ wxCoord guListView::OnMeasureItem( size_t n ) const
     return y + 4;
 }
 
-// -------------------------------------------------------------------------------- //
-int guListView::GetFirstSelected( unsigned long &cookie ) const
-{
-    return m_ListBox->GetFirstSelected( cookie );
-}
-
-// -------------------------------------------------------------------------------- //
-int guListView::GetNextSelected( unsigned long &cookie ) const
-{
-    return m_ListBox->GetNextSelected( cookie );
-}
-
-// -------------------------------------------------------------------------------- //
-int guListView::GetSelection() const
-{
-    return m_ListBox->GetSelection();
-}
-
-// -------------------------------------------------------------------------------- //
-bool guListView::Select( size_t item, bool select )
-{
-    return m_ListBox->Select( item, select );
-}
-
 // -------------------------------------------------------------------------------- //
 void guListView::SetSelection( int selection )
 {
@@ -350,64 +308,18 @@ void guListView::SetSelection( int selection )
 }
 
 // -------------------------------------------------------------------------------- //
-size_t guListView::GetFirstVisibleLine() const
+void guListView::RefreshAll( int scrollto )
 {
-    return m_ListBox->GetFirstVisibleLine();
-}
-
-// -------------------------------------------------------------------------------- //
-size_t guListView::GetLastVisibleLine() const
-{
-    return m_ListBox->GetLastVisibleLine();
-}
-
-// -------------------------------------------------------------------------------- //
-bool guListView::ScrollLines( int lines )
-{
-    return m_ListBox->ScrollLines( lines );
-}
-
-// -------------------------------------------------------------------------------- //
-bool guListView::ScrollToLine( size_t line )
-{
-    return m_ListBox->ScrollToLine( line );
-}
-
-// -------------------------------------------------------------------------------- //
-void guListView::RefreshAll( int scroll )
-{
-    if( scroll != wxNOT_FOUND )
+    if( scrollto != wxNOT_FOUND )
     {
-        if( !m_ListBox->IsVisible( scroll ) )
-            m_ListBox->ScrollToLine( scroll );
+        if( !m_ListBox->IsVisible( scrollto ) )
+        {
+            m_ListBox->ScrollToLine( scrollto );
+        }
     }
     m_ListBox->RefreshAll();
 }
 
-// -------------------------------------------------------------------------------- //
-void guListView::RefreshLines( const int from, const int to )
-{
-    m_ListBox->RefreshLines( from, to );
-}
-
-// -------------------------------------------------------------------------------- //
-void guListView::RefreshLine( const int line )
-{
-    m_ListBox->RefreshLine( line );
-}
-
-// -------------------------------------------------------------------------------- //
-bool guListView::IsSelected( size_t row ) const
-{
-    return m_ListBox->IsSelected( row );
-}
-
-// -------------------------------------------------------------------------------- //
-int guListView::GetDragFiles( wxFileDataObject * files )
-{
-    return 0;
-}
-
 // -------------------------------------------------------------------------------- //
 void guListView::OnBeginDrag( wxMouseEvent &event )
 {
@@ -430,12 +342,6 @@ void guListView::OnBeginDrag( wxMouseEvent &event )
     }
 }
 
-// -------------------------------------------------------------------------------- //
-int guListView::GetSelectedSongs( guTrackArray * tracks ) const
-{
-    return 0;
-}
-
 // -------------------------------------------------------------------------------- //
 wxArrayInt guListView::GetSelectedItems( const bool convertall ) const
 {
@@ -483,12 +389,57 @@ wxArrayInt guListView::GetSelectedItems( const bool convertall ) const
     return RetVal;
 }
 
+// -------------------------------------------------------------------------------- //
+wxArrayInt guListView::GetSelectedIndexs( const bool convertall ) const
+{
+    wxArrayInt RetVal;
+    unsigned long cookie;
+    int item;
+    int index;
+    int count;
+    if( GetItemCount() )
+    {
+        if( m_ListBox->HasMultipleSelection() )
+        {
+            item = GetFirstSelected( cookie );
+            while( item != wxNOT_FOUND )
+            {
+                RetVal.Add( item );
+                if( convertall && ( item == 0 ) )
+                    break;
+                item = GetNextSelected( cookie );
+            }
+        }
+        else
+        {
+            item = m_ListBox->GetSelection();
+            if( item != wxNOT_FOUND )
+            {
+                RetVal.Add( item );
+            }
+        }
+
+        //
+        if( convertall && ( RetVal.Index( 0 ) != wxNOT_FOUND ) )
+        {
+            RetVal.Empty();
+            count = GetItemCount();
+            for( index = 0; index < count; index++ )
+            {
+                RetVal.Add( index );
+            }
+        }
+    }
+    return RetVal;
+}
+
 // -------------------------------------------------------------------------------- //
 void guListView::SetSelectedItems( const wxArrayInt &selection )
 {
     int index;
     int count = GetItemCount();
 
+    // TODO Need to speed up this
     ClearSelectedItems();
     if( selection.Count() )
     {
@@ -511,15 +462,72 @@ void guListView::SetSelectedItems( const wxArrayInt &selection )
 }
 
 // -------------------------------------------------------------------------------- //
-void guListView::ClearSelectedItems()
+void guListView::SetSelectedIndexs( const wxArrayInt &selection )
 {
-    SetSelection( wxNOT_FOUND );
+    int index;
+    int count;
+
+    // TODO Need to speed up this
+    ClearSelectedItems();
+    if( ( count = selection.Count() ) )
+    {
+        bool IsMultiple = m_ListBox->HasMultipleSelection();
+        for( index = 0; index < count; index++ )
+        {
+            if( IsMultiple )
+                Select( selection[ index ] );
+            else
+                SetSelection( selection[ index ] );
+        }
+        wxCommandEvent event( wxEVT_COMMAND_LISTBOX_SELECTED, m_ListBox->GetId() );
+        event.SetEventObject( m_ListBox );
+        event.SetInt( selection[ 0 ] );
+        (void) GetEventHandler()->ProcessEvent( event );
+    }
 }
 
 // -------------------------------------------------------------------------------- //
-long guListView::FindItem( long start, const wxString &str, bool partial )
+void guListView::GetSelectedItems( guListItems * items, const bool convertall ) const
 {
-    return m_ListBox->FindItem( start, str, partial );
+    unsigned long cookie;
+    int item;
+    int index;
+    int count;
+    bool AppendAll = false;
+    if( ( count = GetItemCount() ) )
+    {
+        if( m_ListBox->HasMultipleSelection() )
+        {
+            item = GetFirstSelected( cookie );
+            while( item != wxNOT_FOUND )
+            {
+                if( convertall && ( item == 0 ) )
+                {
+                    AppendAll = true;
+                    break;
+                }
+                items->Add( new guListItem( GetItemId( item ), GetItemName( item ) ) );
+                item = GetNextSelected( cookie );
+            }
+        }
+        else
+        {
+            item = m_ListBox->GetSelection();
+            if( item != wxNOT_FOUND )
+            {
+                items->Add( new guListItem( GetItemId( item ), GetItemName( item ) ) );
+            }
+        }
+
+        //
+        if( convertall && AppendAll )
+        {
+            for( index = 0; index < count; index++ )
+            {
+                items->Add( new guListItem( GetItemId( index ), GetItemName( index ) ) );
+            }
+        }
+    }
 }
 
 // -------------------------------------------------------------------------------- //
@@ -530,30 +538,6 @@ void guListView::SetColumnWidth( const int col, const int width )
         m_Header->RefreshWidth();
 }
 
-// -------------------------------------------------------------------------------- //
-int guListView::GetColumnWidth( const int col ) const
-{
-    return m_Columns->Item( col ).m_Width;
-}
-
-// -------------------------------------------------------------------------------- //
-bool guListView::IsAllowedColumnSelect( void ) const
-{
-    return m_ColSelect;
-}
-
-// -------------------------------------------------------------------------------- //
-wxString guListView::GetColumnLabel( const int col ) const
-{
-    return m_Columns->Item( col ).m_Label;
-}
-
-// -------------------------------------------------------------------------------- //
-int guListView::GetColumnId( const int col ) const
-{
-    return m_Columns->Item( col ).m_Id;
-}
-
 // -------------------------------------------------------------------------------- //
 void  guListView::SetColumnLabel( const int col, const wxString &label )
 {
@@ -570,17 +554,6 @@ void guListView::SetColumnImage( const int col, const int imageindex )
         m_Header->Refresh();
 }
 
-// -------------------------------------------------------------------------------- //
-void guListView::OnKeyDown( wxKeyEvent &event )
-{
-    event.Skip();
-}
-
-// -------------------------------------------------------------------------------- //
-int guListView::HitTest( wxCoord x, wxCoord y ) const
-{
-    return m_ListBox->HitTest( x, y );
-}
 
 // -------------------------------------------------------------------------------- //
 void guListView::SetImageList( wxImageList * imagelist )
@@ -593,21 +566,6 @@ void guListView::SetImageList( wxImageList * imagelist )
         m_Header->SetImageList( imagelist );
 }
 
-// -------------------------------------------------------------------------------- //
-void guListView::OnDropBegin( void )
-{
-}
-
-// -------------------------------------------------------------------------------- //
-void guListView::OnDropFile( const wxString &filename )
-{
-}
-
-// -------------------------------------------------------------------------------- //
-void guListView::OnDropEnd( void )
-{
-}
-
 // -------------------------------------------------------------------------------- //
 void guListView::OnDragOver( wxCoord x, wxCoord y )
 {
@@ -667,11 +625,6 @@ void guListView::OnDragOver( wxCoord x, wxCoord y )
     }
 }
 
-// -------------------------------------------------------------------------------- //
-void guListView::MoveSelection( void )
-{
-}
-
 // -------------------------------------------------------------------------------- //
 void guListView::OnMouse( wxMouseEvent &event )
 {
@@ -753,12 +706,14 @@ void guListViewClient::OnMouse( wxMouseEvent &event )
     int MouseX = event.m_x;
     int MouseY = event.m_y;
     int Item = HitTest( MouseX, MouseY );
+    bool ResetVals = false;
+
     // We want to get a better experience for dragging as before
     // when you click over selected items the items was unselected
     // even when you tried to drag then.
     // Here we check if the item was selected and if so wait for the button up
     // to unselecte the item
-    //guLogMessage( wxT( "WasLeftUp: %i  Selecting: %i" ), m_MouseWasLeftUp, m_MouseSelecting );
+    //guLogMessage( wxT( "ID: %u LD: %i LU: %i SD: %i CD: %i WasLeftUp: %i  Selecting: %i " ), event.GetId(), event.LeftDown(), event.LeftUp(), event.ShiftDown(), event.ControlDown(), m_MouseWasLeftUp, m_MouseSelecting );
     if( !m_MouseWasLeftUp && !event.ShiftDown() && !event.ControlDown() )
     {
         m_MouseWasLeftUp = event.LeftUp();
@@ -766,9 +721,9 @@ void guListViewClient::OnMouse( wxMouseEvent &event )
         {
             if( Item != wxNOT_FOUND )
             {
-                if( IsSelected( Item ) )
+                bool Selected = IsSelected( Item );
+                if( Selected )
                 {
-                    //guLogMessage( wxT( "Event Left Down/Up..." ) );
                     if( !m_MouseSelecting && event.LeftUp() )
                     {
                         // Its a LeftUp event
@@ -781,17 +736,20 @@ void guListViewClient::OnMouse( wxMouseEvent &event )
                 m_MouseSelecting = event.LeftDown();
             }
         }
+        else
+        {
+            ResetVals = true;
+        }
     }
     else
     {
-        m_MouseWasLeftUp = false;
-        m_MouseSelecting = false;
+        ResetVals = true;
     }
 
     // Only when the left or right is down and the click events are enabled
-    if( ( event.LeftDown() || event.RightDown() ) &&
-        m_ColumnClickEvents &&
-        ( Item != wxNOT_FOUND ) )
+    if( m_ColumnClickEvents &&
+        ( Item != wxNOT_FOUND ) &&
+        ( ( event.LeftDown() && !( ResetVals && m_MouseSelecting ) ) || event.RightDown() ) )
     {
         // We want get the left click events, calculate which column clicked on and
         // send a Column_Clicked event
@@ -836,6 +794,12 @@ void guListViewClient::OnMouse( wxMouseEvent &event )
     if( event.RightDown() && ( Item != wxNOT_FOUND ) && !IsSelected( Item ) )
         OnLeftDown( event );
 
+    if( ResetVals )
+    {
+        m_MouseWasLeftUp = false;
+        m_MouseSelecting = false;
+    }
+
     event.Skip();
 }
 
@@ -845,11 +809,9 @@ void guListViewClient::OnKeyDown( wxKeyEvent &event )
     wxChar KeyChar = event.GetKeyCode();
 
     int KeyMod = event.GetModifiers();
-    if( KeyMod == wxMOD_NONE )
+    if( KeyMod == wxMOD_NONE || KeyMod == wxMOD_SHIFT )
     {
-        if( ( KeyChar >= 'a' && KeyChar <= 'z' ) ||
-            ( KeyChar >= 'A' && KeyChar <= 'Z' ) ||
-            ( KeyChar >= '0' && KeyChar <= '9' ) )
+        if( FindChars.Find( KeyChar ) != wxNOT_FOUND )
         {
             if( m_SearchStrTimer->IsRunning() )
             {
@@ -906,9 +868,11 @@ void guListViewClient::OnPaint( wxPaintEvent &event )
     rectLine.width = clientSize.x;
 
     m_Owner->ItemsLock();
-    // iterate over all visible lines
     const size_t lineMax = GetVisibleEnd();
-    for( size_t line = GetFirstVisibleLine(); line < lineMax; line++ )
+    size_t line = GetFirstVisibleLine();
+    m_Owner->ItemsCheckRange( line, lineMax );
+    // iterate over all visible lines
+    for( ; line < lineMax; line++ )
     {
         const wxCoord hLine = OnGetLineHeight(line);
 
@@ -1079,11 +1043,11 @@ void guListViewClient::OnDrawBackground( wxDC &dc, const wxRect &rect, size_t n
 // -------------------------------------------------------------------------------- //
 wxString guListViewClient::GetItemSearchText( const int row )
 {
-    return m_Owner->GetItemName( row );
+    return m_Owner->GetItemSearchText( row );
 }
 
 // -------------------------------------------------------------------------------- //
-long guListViewClient::FindItem( long start, const wxString& str, bool partial )
+long guListViewClient::FindItem( long start, const wxString& str, bool partial, bool atstart )
 {
     if( str.empty() )
         return wxNOT_FOUND;
@@ -1103,9 +1067,14 @@ long guListViewClient::FindItem( long start, const wxString& str, bool partial )
             if( line_upper == str_upper )
                 return index;
         }
+        else if( atstart )
+        {
+            if( line_upper.Find( str_upper ) == 0 )
+                return index;
+        }
         else
         {
-            if( line_upper.find( str_upper ) == 0 )
+            if( line_upper.Find( str_upper ) != wxNOT_FOUND )
                 return index;
         }
     }
@@ -1478,7 +1447,7 @@ guListViewColEdit::guListViewColEdit( wxWindow * parent, guListViewColumnArray *
 	MainSizer = new wxBoxSizer( wxVERTICAL );
 
 	wxStaticBoxSizer * ColumnsSizer;
-	ColumnsSizer = new wxStaticBoxSizer( new wxStaticBox( this, wxID_ANY, wxT(" Columns ") ), wxHORIZONTAL );
+	ColumnsSizer = new wxStaticBoxSizer( new wxStaticBox( this, wxID_ANY, _(" Columns ") ), wxHORIZONTAL );
 
     int index;
     int count = columns->Count();
@@ -1742,10 +1711,12 @@ guListViewDropFilesThread::ExitCode guListViewDropFilesThread::Entry()
 {
     int index;
     int Count = m_Files.Count();
+    int Length = 0;
     for( index = 0; index < Count; ++index )
     {
         if( TestDestroy() )
             return 0;
+        Length += m_Files[ index ].Length();
         AddDropFiles( m_Files[ index ] );
     }
 
