@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #include "curl/http.h"
 #include "MusicBrainz.h"
 #include "Utils.h"
+#include "Version.h"
 
 #define guMUSICDNS_CLIENT_ID    "ca3d48c7383db1dcf6dccd1f0cab26e5"
 #define guMUSICDNS_BASEURL      "http://ofa.musicdns.org/ofa/1/track"
@@ -108,6 +109,7 @@ static gboolean gst_bus_async_callback( GstBus * bus, GstMessage * message, guMu
             {
                 //guLogMessage( wxT( "Gstreamer got fingerprint '%s'" ), wxString( fingerprint, wxConvUTF8 ).c_str() );
                 pobj->SetFingerprint( fingerprint );
+                g_free( fingerprint );
             }
 
             /* Free the tag list */
@@ -457,9 +459,10 @@ bool guMusicDns::DoGetMetadata( void )
 
     //guLogMessage( wxT( guMUSICDNS_BASEURL ) wxT( "%s" ), HtmlData.c_str() );
     wxCurlHTTP  http;
-    http.AddHeader( wxT( "User-Agent: Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.5) Gecko/2008121622 Ubuntu/8.10 (intrepid) Firefox/3.0.5" ) );
+    http.AddHeader( wxT( "User-Agent: " ) guDEFAULT_BROWSER_USER_AGENT );
     http.AddHeader( wxT( "Accept: text/html" ) );
     http.AddHeader( wxT( "Accept-Charset: utf-8" ) );
+    http.SetOpt( CURLOPT_FOLLOWLOCATION, 1 );
     if( http.Post( wxCURL_STRING2BUF( HtmlData ), HtmlData.Length(), wxT( guMUSICDNS_BASEURL ) ) )
     {
         SetXmlDoc( http.GetResponseBody() );
