@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -133,10 +133,6 @@ bool guPlayListFile::Save( const wxString &filename )
     {
         return WritePlsFile( filename );
     }
-    else if( filename.Lower().EndsWith( wxT( ".m3u" ) ) )
-    {
-        return WriteM3uFile( filename );
-    }
     else if( filename.Lower().EndsWith( wxT( ".xspf" ) ) )
     {
         return WriteXspfFile( filename );
@@ -145,6 +141,13 @@ bool guPlayListFile::Save( const wxString &filename )
     {
         return WriteAsxFile( filename );
     }
+    else
+    {
+        wxString FileName = filename;
+        if( !FileName.Lower().EndsWith( wxT( ".m3u" ) ) )
+            FileName += wxT( ".m3u" );
+        return WriteM3uFile( FileName );
+    }
     return false;
 }
 
