@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -70,25 +70,21 @@ guPodcastChannel::guPodcastChannel( const wxString &url )
 bool guPodcastChannel::ReadContent( void )
 {
     bool RetVal = false;
-    wxCurlHTTP  http;
-    //guLogMessage( wxT( "The address is %s" ), m_Url.c_str() );
 
-    http.AddHeader( wxT( "User-Agent: Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.5) Gecko/2008121622 Ubuntu/8.10 (intrepid) Firefox/3.0.5" ) );
-    http.AddHeader( wxT( "Accept: */*" ) );
-    http.AddHeader( wxT( "Accept-Charset: utf-8;iso-8859-1" ) );
-    char * Buffer = NULL;
-    http.Get( Buffer, m_Url );
-    if( Buffer )
+    wxString Content = GetUrlContent( m_Url );
+    if( !Content.IsEmpty() )
     {
-        wxMemoryInputStream ins( Buffer, Strlen( Buffer ) );
+        wxStringInputStream ins( Content );
         wxXmlDocument XmlDoc( ins );
-        //wxSt
         wxXmlNode * XmlNode = XmlDoc.GetRoot();
         if( XmlNode && XmlNode->GetName() == wxT( "rss" ) )
         {
             RetVal = ReadXml( XmlNode->GetChildren() );
         }
-        free( Buffer );
+        else
+        {
+            guLogMessage( wxT( "This url is not a valid podcast" ) );
+        }
     }
     else
     {
@@ -97,60 +93,83 @@ bool guPodcastChannel::ReadContent( void )
     return RetVal;
 }
 
+// -------------------------------------------------------------------------------- //
+bool guPodcastChannel::ReadXmlImage( wxXmlNode * XmlNode )
+{
+    while( XmlNode )
+    {
+        if( XmlNode->GetName() == wxT( "url" ) )
+        {
+            m_Image = XmlNode->GetNodeContent();
+            return true;
+        }
+        XmlNode = XmlNode->GetNext();
+    }
+    return false;
+}
+
 // -------------------------------------------------------------------------------- //
 bool guPodcastChannel::ReadXml( wxXmlNode * XmlNode )
 {
-    if( XmlNode && XmlNode->GetName() == wxT( "channel" ) )
+    while( XmlNode )
     {
-        XmlNode = XmlNode->GetChildren();
-        while( XmlNode )
+        if( XmlNode->GetName() == wxT( "channel" ) )
         {
-            if( XmlNode->GetName() == wxT( "title" ) )
+            XmlNode = XmlNode->GetChildren();
+            while( XmlNode )
             {
-                m_Title = XmlNode->GetNodeContent();
+                if( XmlNode->GetName() == wxT( "title" ) )
+                {
+                    m_Title = XmlNode->GetNodeContent();
+                }
+                else if( XmlNode->GetName() == wxT( "link" ) )
+                {
+                    m_Link = XmlNode->GetNodeContent();
+                }
+                else if( XmlNode->GetName() == wxT( "language" ) )
+                {
+                    m_Lang = XmlNode->GetNodeContent();
+                }
+                else if( XmlNode->GetName() == wxT( "description" ) )
+                {
+                    m_Description = XmlNode->GetNodeContent();
+                }
+                else if( XmlNode->GetName() == wxT( "itunes:author" ) )
+                {
+                    m_Author = XmlNode->GetNodeContent();
+                }
+                else if( XmlNode->GetName() == wxT( "itunes:owner" ) )
+                {
+                    ReadXmlOwner( XmlNode->GetChildren() );
+                }
+                else if( XmlNode->GetName() == wxT( "itunes:image" ) )
+                {
+                    XmlNode->GetPropVal( wxT( "href" ), &m_Image );
+                }
+                else if( XmlNode->GetName() == wxT( "image" ) )
+                {
+                    ReadXmlImage( XmlNode->GetChildren() );
+                }
+                else if( XmlNode->GetName() == wxT( "itunes:category" ) )
+                {
+                    XmlNode->GetPropVal( wxT( "text" ), &m_Category );
+                }
+                else if( XmlNode->GetName() == wxT( "itunes:summary" ) )
+                {
+                    m_Summary = XmlNode->GetNodeContent();
+                }
+                else if( XmlNode->GetName() == wxT( "item" ) )
+                {
+                    guPodcastItem * PodcastItem = new guPodcastItem( XmlNode->GetChildren() );
+                    //PodcastItem->ReadXml( XmlNode->GetChildren() );
+                    //guLogMessage( wxT( "Item Length: %i" ), PodcastItem->m_Length );
+                    m_Items.Add( PodcastItem );
+                }
+                XmlNode = XmlNode->GetNext();
             }
-            else if( XmlNode->GetName() == wxT( "link" ) )
-            {
-                m_Link = XmlNode->GetNodeContent();
-            }
-            else if( XmlNode->GetName() == wxT( "language" ) )
-            {
-                m_Lang = XmlNode->GetNodeContent();
-            }
-            else if( XmlNode->GetName() == wxT( "description" ) )
-            {
-                m_Description = XmlNode->GetNodeContent();
-            }
-            else if( XmlNode->GetName() == wxT( "itunes:author" ) )
-            {
-                m_Author = XmlNode->GetNodeContent();
-            }
-            else if( XmlNode->GetName() == wxT( "itunes:owner" ) )
-            {
-                ReadXmlOwner( XmlNode->GetChildren() );
-            }
-            else if( XmlNode->GetName() == wxT( "itunes:image" ) )
-            {
-                XmlNode->GetPropVal( wxT( "href" ), &m_Image );
-            }
-            else if( XmlNode->GetName() == wxT( "itunes:category" ) )
-            {
-                XmlNode->GetPropVal( wxT( "text" ), &m_Category );
-            }
-            else if( XmlNode->GetName() == wxT( "itunes:summary" ) )
-            {
-                m_Summary = XmlNode->GetNodeContent();
-            }
-            else if( XmlNode->GetName() == wxT( "item" ) )
-            {
-                guPodcastItem * PodcastItem = new guPodcastItem( XmlNode->GetChildren() );
-                //PodcastItem->ReadXml( XmlNode->GetChildren() );
-                //guLogMessage( wxT( "Item Length: %i" ), PodcastItem->m_Length );
-                m_Items.Add( PodcastItem );
-            }
-            XmlNode = XmlNode->GetNext();
+            return true;
         }
-        return true;
+        XmlNode = XmlNode->GetNext();
     }
     return false;
 }
@@ -282,6 +301,52 @@ int guPodcastChannel::GetUpdateItems( guDbLibrary * db, guPodcastItemArray * ite
   return items->Count();
 }
 
+// -------------------------------------------------------------------------------- //
+int guPodcastChannel::GetPendingChannelItems( guDbLibrary * db, int channelid, guPodcastItemArray * items )
+{
+  wxString query;
+  wxSQLite3ResultSet dbRes;
+
+  query = wxString::Format( wxT( "SELECT podcastitem_id, podcastitem_chid, podcastitem_title, "
+            "podcastitem_summary, podcastitem_author, podcastitem_enclosure, podcastitem_time, "
+            "podcastitem_file, podcastitem_filesize, podcastitem_length, "
+            "podcastitem_playcount, podcastitem_addeddate, podcastitem_lastplay, "
+            "podcastitem_status, "
+            "podcastch_title, podcastch_category "
+            "FROM podcastitems, podcastchs "
+            "WHERE podcastitem_chid = podcastch_id "
+            "AND podcastitem_chid = %u " ), channelid );
+
+  query += wxT( "AND podcastitem_status IN ( 1 ) " );
+
+  dbRes = db->ExecuteQuery( query );
+
+  while( dbRes.NextRow() )
+  {
+    guPodcastItem * Item = new guPodcastItem();
+    Item->m_Id = dbRes.GetInt( 0 );
+    Item->m_ChId = dbRes.GetInt( 1 );
+    Item->m_Title = dbRes.GetString( 2 );
+    Item->m_Summary = dbRes.GetString( 3 );
+    Item->m_Author = dbRes.GetString( 4 );
+    Item->m_Enclosure = dbRes.GetString( 5 );
+    Item->m_Time = dbRes.GetInt( 6 );
+    Item->m_FileName = dbRes.GetString( 7 );
+    Item->m_FileSize = dbRes.GetInt( 8 );
+    Item->m_Length = dbRes.GetInt( 9 );
+    Item->m_PlayCount = dbRes.GetInt( 10 );
+    Item->m_AddedDate = dbRes.GetInt( 11 );
+    Item->m_LastPlay = dbRes.GetInt( 12 );
+    Item->m_Status = dbRes.GetInt( 13 );
+
+    Item->m_Channel = dbRes.GetString( 14 );
+    Item->m_Category = dbRes.GetString( 15 );
+    items->Add( Item );
+  }
+  dbRes.Finalize();
+  return items->Count();
+}
+
 // -------------------------------------------------------------------------------- //
 int guPodcastChannel::CheckDownloadItems( guDbLibrary * db, guMainFrame * mainframe )
 {
@@ -297,6 +362,23 @@ int guPodcastChannel::CheckDownloadItems( guDbLibrary * db, guMainFrame * mainfr
             return Count;
         }
     }
+    else if( m_DownloadType == guPODCAST_DOWNLOAD_MANUALLY )
+    {
+        // Check if in the download thread this items are included and delete them
+        guPodcastItemArray Podcasts;
+        GetPendingChannelItems( db, m_Id, &Podcasts );
+
+        int Index;
+        int Count = Podcasts.Count();
+        if( Count )
+        {
+            mainframe->RemovePodcastDownloadItems( &Podcasts );
+            for( Index = 0; Index < Count; Index++ )
+            {
+                db->SetPodcastItemStatus( Podcasts[ Index ].m_Id, guPODCAST_STATUS_NORMAL );
+            }
+        }
+    }
     return 0;
 }
 
@@ -417,6 +499,7 @@ void guPodcastItem::ReadXml( wxXmlNode * XmlNode )
 {
     while( XmlNode )
     {
+        //guLogMessage( wxT( "Reading now : '%s'" ), XmlNode->GetName().c_str() );
         if( XmlNode->GetName() == wxT( "title" ) )
         {
             m_Title = XmlNode->GetNodeContent();
