@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 // -------------------------------------------------------------------------------- //
 #include "PodcastsPanel.h"
 
+#include "AuiDockArt.h"
 #include "ChannelEditor.h"
 #include "Commands.h"
 #include "Config.h"
@@ -72,6 +73,7 @@ guPodcastPanel::guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame
     }
 
     m_AuiManager.SetManagedWindow( this );
+    m_AuiManager.SetArtProvider( new guAuiDockArt() );
     m_AuiManager.SetFlags( wxAUI_MGR_ALLOW_FLOATING |
                            wxAUI_MGR_TRANSPARENT_DRAG |
                            wxAUI_MGR_TRANSPARENT_HINT );
@@ -82,16 +84,16 @@ guPodcastPanel::guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame
             wxSystemSettings::GetColour( wxSYS_COLOUR_CAPTIONTEXT ) );
 
     AuiDockArt->SetColour( wxAUI_DOCKART_ACTIVE_CAPTION_COLOUR,
-            wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
+            wxSystemSettings::GetColour( wxSYS_COLOUR_ACTIVEBORDER ) );
 
     AuiDockArt->SetColour( wxAUI_DOCKART_ACTIVE_CAPTION_GRADIENT_COLOUR,
-            wxAuiStepColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ), 130 ) );
+            wxSystemSettings::GetColour( wxSYS_COLOUR_3DSHADOW ) );
 
     AuiDockArt->SetColour( wxAUI_DOCKART_INACTIVE_CAPTION_COLOUR,
-            wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
+            wxSystemSettings::GetColour( wxSYS_COLOUR_INACTIVEBORDER ) );
 
     AuiDockArt->SetColour( wxAUI_DOCKART_INACTIVE_CAPTION_GRADIENT_COLOUR,
-            wxAuiStepColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ), 130 ) );
+            wxSystemSettings::GetColour( wxSYS_COLOUR_3DSHADOW ) );
 
     AuiDockArt->SetColour( wxAUI_DOCKART_GRADIENT_TYPE,
             wxAUI_GRADIENT_VERTICAL );
@@ -152,7 +154,7 @@ guPodcastPanel::guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame
 	m_DetailFlexGridSizer->SetFlexibleDirection( wxBOTH );
 	m_DetailFlexGridSizer->SetNonFlexibleGrowMode( wxFLEX_GROWMODE_SPECIFIED );
 
-	m_DetailImage = new wxStaticBitmap( m_DetailScrolledWindow, wxID_ANY, guImage( guIMAGE_INDEX_tiny_podcast_icon ), wxDefaultPosition, wxSize( 60,60 ), 0 );
+	m_DetailImage = new wxStaticBitmap( m_DetailScrolledWindow, wxID_ANY, guImage( guIMAGE_INDEX_mid_podcast ), wxDefaultPosition, wxSize( 60,60 ), 0 );
 	m_DetailFlexGridSizer->Add( m_DetailImage, 0, wxALL, 5 );
 
 	m_DetailChannelTitle = new wxStaticText( m_DetailScrolledWindow, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
@@ -291,6 +293,7 @@ guPodcastPanel::guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame
 
     Connect( ID_PODCASTS_ITEM_PLAY, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemPlay ), NULL, this );
     Connect( ID_PODCASTS_ITEM_ENQUEUE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemEnqueue ), NULL, this );
+    Connect( ID_PODCASTS_ITEM_ENQUEUE_ASNEXT, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemEnqueueAsNext ), NULL, this );
     Connect( ID_PODCASTS_ITEM_DEL, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemDelete ), NULL, this );
     Connect( ID_PODCASTS_ITEM_DOWNLOAD, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemDownload ), NULL, this );
     Connect( ID_PODCASTS_ITEM_COPYTO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemCopyTo ), NULL, this );
@@ -329,6 +332,7 @@ guPodcastPanel::~guPodcastPanel()
 
     Disconnect( ID_PODCASTS_ITEM_PLAY, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemPlay ), NULL, this );
     Disconnect( ID_PODCASTS_ITEM_ENQUEUE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemEnqueue ), NULL, this );
+    Disconnect( ID_PODCASTS_ITEM_ENQUEUE_ASNEXT, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemEnqueueAsNext ), NULL, this );
     Disconnect( ID_PODCASTS_ITEM_DEL, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemDelete ), NULL, this );
     Disconnect( ID_PODCASTS_ITEM_DOWNLOAD, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemDownload ), NULL, this );
     Disconnect( ID_PODCASTS_ITEM_COPYTO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemCopyTo ), NULL, this );
@@ -497,7 +501,7 @@ void guPodcastPanel::ChannelsCopyTo( wxCommandEvent &event )
     guPodcastItemArray PodcastItems;
     guTrackArray * Tracks = new guTrackArray();
 
-    m_Db->GetPodcastItems( &PodcastItems );
+    m_Db->GetPodcastItems( &PodcastItems, m_PodcastsListBox->GetFilters(), m_PodcastsListBox->GetOrder(), m_PodcastsListBox->GetOrderDesc() );
 
     int Index;
     int Count = PodcastItems.Count();
@@ -511,6 +515,7 @@ void guPodcastPanel::ChannelsCopyTo( wxCommandEvent &event )
             Track->m_FileName = PodcastItems[ Index ].m_FileName;
             Track->m_SongName = PodcastItems[ Index ].m_Title;
             Track->m_ArtistName = PodcastItems[ Index ].m_Author;
+            Track->m_AlbumId = PodcastItems[ Index ].m_ChId;
             Track->m_AlbumName = PodcastItems[ Index ].m_Channel;
             Track->m_Length = PodcastItems[ Index ].m_Length;
             Track->m_PlayCount = PodcastItems[ Index ].m_PlayCount;
@@ -553,7 +558,7 @@ void guPodcastPanel::UpdateChannels( wxCommandEvent &event )
 void guPodcastPanel::OnChannelsSelected( wxListEvent &event )
 {
     wxArrayInt SelectedItems = m_ChannelsListBox->GetSelectedItems();
-    m_Db->SetPodcastChannelFilters( SelectedItems );
+    m_PodcastsListBox->SetFilters( SelectedItems );
     m_PodcastsListBox->ReloadItems();
 
     if( SelectedItems.Count() == 1 && SelectedItems[ 0 ] != 0 )
@@ -669,7 +674,7 @@ void guPodcastPanel::UpdateChannelInfo( int itemid )
                 m_DetailImage->SetBitmap( PodcastImage );
             }
             else
-                m_DetailImage->SetBitmap( guBitmap( guIMAGE_INDEX_tiny_podcast_icon ) );
+                m_DetailImage->SetBitmap( guBitmap( guIMAGE_INDEX_mid_podcast ) );
         }
 
         m_DetailChannelTitle->SetLabel( PodcastChannel.m_Title );
@@ -682,7 +687,7 @@ void guPodcastPanel::UpdateChannelInfo( int itemid )
     }
     else
     {
-        m_DetailImage->SetBitmap( guBitmap( guIMAGE_INDEX_tiny_podcast_icon ) );
+        m_DetailImage->SetBitmap( guBitmap( guIMAGE_INDEX_mid_podcast ) );
         m_DetailChannelTitle->SetLabel( wxEmptyString );
         m_DetailDescText->SetLabel( wxEmptyString );
         m_DetailAuthorText->SetLabel( wxEmptyString );
@@ -708,7 +713,7 @@ void guPodcastPanel::OnPodcastItemUpdated( wxCommandEvent &event )
 }
 
 // -------------------------------------------------------------------------------- //
-void guPodcastPanel::OnSelectPodcasts( bool enqueue )
+void guPodcastPanel::OnSelectPodcasts( bool enqueue, const bool asnext )
 {
     int Index;
     int Count;
@@ -733,6 +738,7 @@ void guPodcastPanel::OnSelectPodcasts( bool enqueue )
                             Track->m_FileName = PodcastItem.m_FileName;
                             Track->m_SongName = PodcastItem.m_Title;
                             Track->m_ArtistName = PodcastItem.m_Author;
+                            Track->m_AlbumId = PodcastItem.m_ChId;
                             Track->m_AlbumName = PodcastItem.m_Channel;
                             Track->m_Length = PodcastItem.m_Length;
                             Track->m_PlayCount = PodcastItem.m_PlayCount;
@@ -770,7 +776,7 @@ void guPodcastPanel::OnSelectPodcasts( bool enqueue )
         {
             if( enqueue )
             {
-                m_PlayerPanel->AddToPlayList( Tracks );
+                m_PlayerPanel->AddToPlayList( Tracks, true, asnext );
             }
             else
             {
@@ -799,6 +805,12 @@ void guPodcastPanel::OnPodcastItemEnqueue( wxCommandEvent &event )
     OnSelectPodcasts( true );
 }
 
+// -------------------------------------------------------------------------------- //
+void guPodcastPanel::OnPodcastItemEnqueueAsNext( wxCommandEvent &event )
+{
+    OnSelectPodcasts( true, true );
+}
+
 // -------------------------------------------------------------------------------- //
 void guPodcastPanel::OnPodcastItemDelete( wxCommandEvent &event )
 {
@@ -812,7 +824,7 @@ void guPodcastPanel::OnPodcastItemDelete( wxCommandEvent &event )
 
         // Check if in the download thread this items are included and delete them
         guPodcastItemArray Podcasts;
-        m_Db->GetPodcastItems( Selection, &Podcasts );
+        m_Db->GetPodcastItems( Selection, &Podcasts, m_PodcastsListBox->GetOrder(), m_PodcastsListBox->GetOrderDesc() );
         m_MainFrame->RemovePodcastDownloadItems( &Podcasts );
 
         for( Index = 0; Index < Count; Index++ )
@@ -923,7 +935,19 @@ void guPodcastPanel::OnPaneClose( wxAuiManagerEvent &event )
     event.Veto();
 }
 
+// -------------------------------------------------------------------------------- //
+void guPodcastPanel::SelectPodcast( const int podcastid )
+{
+    m_ChannelsListBox->SetSelection( wxNOT_FOUND );
+    m_PodcastsListBox->SetSelection( m_PodcastsListBox->FindItem( podcastid ) );
+}
 
+// -------------------------------------------------------------------------------- //
+void guPodcastPanel::SelectChannel( const int channelid )
+{
+    m_ChannelsListBox->SetSelection( wxNOT_FOUND );
+    m_ChannelsListBox->SetSelection( m_ChannelsListBox->FindItem( channelid ) );
+}
 
 // -------------------------------------------------------------------------------- //
 // guChannelsListBox
@@ -940,6 +964,18 @@ void guChannelsListBox::GetItemsList( void )
     }
 }
 
+// -------------------------------------------------------------------------------- //
+void guChannelsListBox::OnKeyDown( wxKeyEvent &event )
+{
+    if( event.GetKeyCode() == WXK_DELETE )
+    {
+        wxCommandEvent CmdEvent( wxEVT_COMMAND_MENU_SELECTED, ID_PODCASTS_CHANNEL_DEL );
+        wxPostEvent( this, CmdEvent );
+        return;
+    }
+    event.Skip();
+}
+
 // -------------------------------------------------------------------------------- //
 int guChannelsListBox::GetSelectedSongs( guTrackArray * Songs ) const
 {
@@ -956,7 +992,7 @@ int guChannelsListBox::GetDragFiles( wxFileDataObject * files )
 void guChannelsListBox::CreateContextMenu( wxMenu * Menu ) const
 {
     wxMenuItem * MenuItem;
-    int SelCount = GetSelectedItems().Count();
+    int SelCount = GetSelectedCount();
 
     MenuItem = new wxMenuItem( Menu, ID_PODCASTS_CHANNEL_ADD, _( "New Channel" ), _( "Add a new podcast channel" ) );
     MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
@@ -992,21 +1028,22 @@ void guChannelsListBox::CreateContextMenu( wxMenu * Menu ) const
 }
 
 // -------------------------------------------------------------------------------- //
-// guPodcastListBox
-// -------------------------------------------------------------------------------- //
-wxString guPODCASTS_COLUMN_NAMES[] = {
-    _( "Status" ),
-    _( "Title" ),
-    _( "Channel" ),
-    _( "Category" ),
-    _( "Date" ),
-    _( "Length" ),
-    _( "Author" ),
-    _( "PlayCount" ),
-    _( "LastPlay" ),
-    _( "Added" )
-};
+int guChannelsListBox::FindItem( const int channelid )
+{
+    int Index;
+    int Count = m_Items->Count();
+    for( Index = 0; Index < Count; Index++ )
+    {
+        if( m_Items->Item( Index ).m_Id == channelid )
+        {
+            return Index;
+        }
+    }
+    return wxNOT_FOUND;
+}
 
+// -------------------------------------------------------------------------------- //
+// guPodcastListBox
 // -------------------------------------------------------------------------------- //
 guPodcastListBox::guPodcastListBox( wxWindow * parent, guDbLibrary * db ) :
     guListView( parent, wxLB_MULTIPLE | guLISTVIEW_COLUMN_SELECT | guLISTVIEW_COLUMN_SORTING | guLISTVIEW_ALLOWDRAG )
@@ -1015,6 +1052,17 @@ guPodcastListBox::guPodcastListBox( wxWindow * parent, guDbLibrary * db ) :
 
     guConfig * Config = ( guConfig * ) guConfig::Get();
 
+    m_ColumnNames.Add( _( "Status" ) );
+    m_ColumnNames.Add( _( "Title" ) );
+    m_ColumnNames.Add( _( "Channel" ) );
+    m_ColumnNames.Add( _( "Category" ) );
+    m_ColumnNames.Add( _( "Date" ) );
+    m_ColumnNames.Add( _( "Length" ) );
+    m_ColumnNames.Add( _( "Author" ) );
+    m_ColumnNames.Add( _( "PlayCount" ) );
+    m_ColumnNames.Add( _( "LastPlay" ) );
+    m_ColumnNames.Add( _( "Added" ) );
+
     m_Order = Config->ReadNum( wxT( "Order" ), 0, wxT( "Podcasts" ) );
     m_OrderDesc = Config->ReadNum( wxT( "OrderDesc" ), false, wxT( "Podcasts" ) );
 
@@ -1029,12 +1077,12 @@ guPodcastListBox::guPodcastListBox( wxWindow * parent, guDbLibrary * db ) :
     int ColId;
     wxString ColName;
     int index;
-    int count = sizeof( guPODCASTS_COLUMN_NAMES ) / sizeof( wxString );
+    int count = m_ColumnNames.Count();
     for( index = 0; index < count; index++ )
     {
         ColId = Config->ReadNum( wxString::Format( wxT( "PodcastsCol%u" ), index ), index, wxT( "PodcastsColumns" ) );
 
-        ColName = guPODCASTS_COLUMN_NAMES[ ColId ];
+        ColName = m_ColumnNames[ ColId ];
 
         ColName += ( ( ColId == m_Order ) ? ( m_OrderDesc ? wxT( " ▼" ) : wxT( " ▲" ) ) : wxEmptyString );
 
@@ -1056,7 +1104,7 @@ guPodcastListBox::~guPodcastListBox()
 {
     guConfig * Config = ( guConfig * ) guConfig::Get();
     int index;
-    int count = sizeof( guPODCASTS_COLUMN_NAMES ) / sizeof( wxString );
+    int count = m_ColumnNames.Count();
     for( index = 0; index < count; index++ )
     {
         Config->WriteNum( wxString::Format( wxT( "PodcastsCol%u" ), index ),
@@ -1153,7 +1201,7 @@ wxString guPodcastListBox::OnGetItemText( const int row, const int col ) const
 // -------------------------------------------------------------------------------- //
 void guPodcastListBox::GetItemsList( void )
 {
-    m_Db->GetPodcastItems( &m_PodItems );
+    m_Db->GetPodcastItems( &m_PodItems, m_PodChFilters, m_Order, m_OrderDesc );
 
     wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_MAINFRAME_UPDATE_SELINFO );
     AddPendingEvent( event );
@@ -1192,6 +1240,18 @@ void guPodcastListBox::ReloadItems( bool reset )
     RefreshAll();
 }
 
+// -------------------------------------------------------------------------------- //
+void guPodcastListBox::OnKeyDown( wxKeyEvent &event )
+{
+    if( event.GetKeyCode() == WXK_DELETE )
+    {
+        wxCommandEvent CmdEvent( wxEVT_COMMAND_MENU_SELECTED, ID_PODCASTS_ITEM_DEL );
+        wxPostEvent( this, CmdEvent );
+        return;
+    }
+    event.Skip();
+}
+
 // -------------------------------------------------------------------------------- //
 void guPodcastListBox::CreateContextMenu( wxMenu * Menu ) const
 {
@@ -1200,13 +1260,17 @@ void guPodcastListBox::CreateContextMenu( wxMenu * Menu ) const
     {
         wxMenuItem * MenuItem;
         MenuItem = new wxMenuItem( Menu, ID_PODCASTS_ITEM_PLAY, _( "Play" ), _( "Play current selected songs" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_playback_start ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_player_tiny_light_play ) );
         Menu->Append( MenuItem );
 
         MenuItem = new wxMenuItem( Menu, ID_PODCASTS_ITEM_ENQUEUE, _( "Enqueue" ), _( "Add current selected songs to playlist" ) );
         MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
         Menu->Append( MenuItem );
 
+        MenuItem = new wxMenuItem( Menu, ID_PODCASTS_ITEM_ENQUEUE_ASNEXT, _( "Enqueue Next" ), _( "Add current selected songs to playlist as Next Tracks" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
+        Menu->Append( MenuItem );
+
         Menu->AppendSeparator();
 
         MenuItem = new wxMenuItem( Menu, ID_PODCASTS_ITEM_DEL, _( "Delete" ), _( "Delete the current selected podcasts" ) );
@@ -1256,22 +1320,35 @@ void guPodcastListBox::SetOrder( int columnid )
     else
         m_OrderDesc = !m_OrderDesc;
 
-    m_Db->SetPodcastOrder( m_Order );
+    //m_Db->SetPodcastOrder( m_Order );
 
     int CurColId;
     int index;
-    int count = sizeof( guPODCASTS_COLUMN_NAMES ) / sizeof( wxString );
+    int count = m_ColumnNames.Count();
     for( index = 0; index < count; index++ )
     {
         CurColId = GetColumnId( index );
         SetColumnLabel( index,
-            guPODCASTS_COLUMN_NAMES[ CurColId ]  + ( ( CurColId == m_Order ) ?
+            m_ColumnNames[ CurColId ]  + ( ( CurColId == m_Order ) ?
                 ( m_OrderDesc ? wxT( " ▼" ) : wxT( " ▲" ) ) : wxEmptyString ) );
     }
 
     ReloadItems();
 }
 
+// -------------------------------------------------------------------------------- //
+void guPodcastListBox::SetFilters( const wxArrayInt &filters )
+{
+    if( filters.Index( 0 ) != wxNOT_FOUND )
+    {
+        m_PodChFilters.Empty();
+    }
+    else
+    {
+        m_PodChFilters = filters;
+    }
+}
+
 // -------------------------------------------------------------------------------- //
 int guPodcastListBox::GetSelectedSongs( guTrackArray * tracks ) const
 {
@@ -1293,6 +1370,7 @@ int guPodcastListBox::GetSelectedSongs( guTrackArray * tracks ) const
                 Track->m_FileName = PodcastItem.m_FileName;
                 Track->m_SongName = PodcastItem.m_Title;
                 Track->m_ArtistName = PodcastItem.m_Author;
+                Track->m_AlbumId = PodcastItem.m_ChId;
                 Track->m_AlbumName = PodcastItem.m_Channel;
                 Track->m_Length = PodcastItem.m_Length;
                 Track->m_PlayCount = PodcastItem.m_PlayCount;
@@ -1322,3 +1400,18 @@ int guPodcastListBox::GetDragFiles( wxFileDataObject * files )
 }
 
 // -------------------------------------------------------------------------------- //
+int guPodcastListBox::FindItem( const int podcastid )
+{
+    int Index;
+    int Count = m_PodItems.Count();
+    for( Index = 0; Index < Count; Index++ )
+    {
+        if( m_PodItems[ Index ].m_Id == podcastid )
+        {
+            return Index;
+        }
+    }
+    return wxNOT_FOUND;
+}
+
+// -------------------------------------------------------------------------------- //
