@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -41,6 +41,9 @@ guSoListBox::guSoListBox( wxWindow * parent, guDbLibrary * NewDb, wxString confn
 
     m_Db = NewDb;
     m_ConfName = confname;
+    m_ItemsFirst = wxNOT_FOUND;
+    m_ItemsLast = wxNOT_FOUND;
+    m_LastColumnRightClicked = wxNOT_FOUND;
 
     int ColOrder = Config->ReadNum( wxT( "TracksOrder" ), 0, wxT( "General" ) );
     bool ColOrderDesc = Config->ReadBool( wxT( "TracksOrderDesc" ), 0, wxT( "General" ) );
@@ -69,12 +72,17 @@ guSoListBox::guSoListBox( wxWindow * parent, guDbLibrary * NewDb, wxString confn
         InsertColumn( Column );
     }
 
-    m_GreyStar   = new wxBitmap( guImage( ( guIMAGE_INDEX ) ( guIMAGE_INDEX_grey_star_tiny + GURATING_STYLE_MID ) ) );
-    m_YellowStar = new wxBitmap( guImage( ( guIMAGE_INDEX ) ( guIMAGE_INDEX_yellow_star_tiny + GURATING_STYLE_MID ) ) );
+    m_NormalStar   = new wxBitmap( guImage( ( guIMAGE_INDEX ) ( guIMAGE_INDEX_star_normal_tiny + GURATING_STYLE_MID ) ) );
+    m_SelectStar = new wxBitmap( guImage( ( guIMAGE_INDEX ) ( guIMAGE_INDEX_star_highlight_tiny + GURATING_STYLE_MID ) ) );
 
-    Connect( ID_LASTFM_SEARCH_LINK, ID_LASTFM_SEARCH_LINK + 999, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnSearchLinkClicked ) );
-    Connect( ID_SONGS_COMMANDS, ID_SONGS_COMMANDS + 99, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnCommandClicked ) );
+    Connect( ID_LASTFM_SEARCH_LINK, ID_LASTFM_SEARCH_LINK + 999, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnSearchLinkClicked ), NULL, this );
+    Connect( ID_SONGS_COMMANDS, ID_SONGS_COMMANDS + 99, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnCommandClicked ), NULL, this );
     Connect( guEVT_LISTBOX_ITEM_COL_CLICKED, wxListEventHandler( guSoListBox::OnItemColumnClicked ), NULL, this );
+    Connect( guEVT_LISTBOX_ITEM_COL_RCLICKED, wxListEventHandler( guSoListBox::OnItemColumnRClicked ), NULL, this );
+
+//    Connect( ID_SONG_EDIT_COLUMN, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnColumEdit ), NULL, this );
+//    Connect( ID_SONG_SET_COLUMN, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnColumSet ), NULL, this );
+//    Connect( ID_SONG_SET_RATING_0, ID_SONG_SET_RATING_5, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnColumnSetRating ), NULL, this );
 
     ReloadItems();
 }
@@ -100,27 +108,32 @@ guSoListBox::~guSoListBox()
                            m_ConfName + wxT( "Columns" ) );
     }
 
-    if( m_GreyStar )
-        delete m_GreyStar;
-    if( m_YellowStar )
-        delete m_YellowStar;
+    if( m_NormalStar )
+        delete m_NormalStar;
+    if( m_SelectStar )
+        delete m_SelectStar;
 
     Disconnect( ID_LASTFM_SEARCH_LINK, ID_LASTFM_SEARCH_LINK + 999, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnSearchLinkClicked ) );
     Disconnect( ID_SONGS_COMMANDS, ID_SONGS_COMMANDS + 99, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnCommandClicked ) );
     Disconnect( guEVT_LISTBOX_ITEM_COL_CLICKED, wxListEventHandler( guSoListBox::OnItemColumnClicked ), NULL, this );
+    Disconnect( guEVT_LISTBOX_ITEM_COL_RCLICKED, wxListEventHandler( guSoListBox::OnItemColumnRClicked ), NULL, this );
 }
 
 // -------------------------------------------------------------------------------- //
 wxString guSoListBox::OnGetItemText( const int row, const int col ) const
 {
     guTrack * Song;
+    //guLogMessage( wxT( "GetItemText( %i, %i )    %i-%i  %i" ), row, col, m_ItemsFirst, m_ItemsLast, m_Items.Count() );
 
-    Song = &m_Items[ row ];
+    Song = &m_Items[ row - m_ItemsFirst ];
 
     switch( ( * m_Columns )[ col ].m_Id )
     {
         case guSONGS_COLUMN_NUMBER :
-          return wxString::Format( wxT( "%02u" ), Song->m_Number );
+          if( Song->m_Number )
+            return wxString::Format( wxT( "%u" ), Song->m_Number );
+          else
+            return wxEmptyString;
 
         case guSONGS_COLUMN_TITLE  :
           return Song->m_SongName;
@@ -128,6 +141,9 @@ wxString guSoListBox::OnGetItemText( const int row, const int col ) const
         case guSONGS_COLUMN_ARTIST :
           return Song->m_ArtistName;
 
+        case guSONGS_COLUMN_ALBUMARTIST :
+            return Song->m_AlbumArtist;
+
         case guSONGS_COLUMN_ALBUM :
           return Song->m_AlbumName;
 
@@ -156,7 +172,11 @@ wxString guSoListBox::OnGetItemText( const int row, const int col ) const
             return wxString::Format( wxT( "%i" ), Song->m_Rating );
 
         case guSONGS_COLUMN_PLAYCOUNT :
-            return wxString::Format( wxT( "%u" ), Song->m_PlayCount );
+        {
+            if( Song->m_PlayCount )
+                return wxString::Format( wxT( "%u" ), Song->m_PlayCount );
+            break;
+        }
 
         case guSONGS_COLUMN_LASTPLAY :
         {
@@ -176,6 +196,11 @@ wxString guSoListBox::OnGetItemText( const int row, const int col ) const
             AddedDate.Set( ( time_t ) Song->m_AddedTime );
             return AddedDate.FormatDate();
         }
+
+        case guSONGS_COLUMN_FORMAT :
+        {
+            return Song->m_Format;
+        }
     }
     return wxEmptyString;
 }
@@ -191,8 +216,8 @@ void guSoListBox::DrawItem( wxDC &dc, const wxRect &rect, const int row, const i
 
         for( x = 0; x < 5; x++ )
         {
-           dc.DrawBitmap( ( x >= m_Items[ row ].m_Rating ) ? * m_GreyStar : * m_YellowStar,
-                          rect.x + 3 + ( w * x ), rect.y + 6, true );
+           dc.DrawBitmap( ( x >= m_Items[ row - m_ItemsFirst ].m_Rating ) ? * m_NormalStar : * m_SelectStar,
+                          rect.x + 3 + ( w * x ), rect.y + 3, true );
         }
     }
     else
@@ -202,10 +227,25 @@ void guSoListBox::DrawItem( wxDC &dc, const wxRect &rect, const int row, const i
 }
 
 
+// -------------------------------------------------------------------------------- //
+void guSoListBox::ItemsCheckRange( const int start, const int end )
+{
+    //guLogMessage( wxT( "guSoListBox::ItemsCheckRange( %i, %i )" ), start, end );
+    if( m_ItemsFirst != start || m_ItemsLast != end )
+    {
+        m_Items.Empty();
+        m_Db->GetSongs( &m_Items, start, end );
+        m_ItemsFirst = start;
+        m_ItemsLast = end;
+    }
+    //guLogMessage( wxT( "Updated the items... %i" ), m_Items );
+}
+
 // -------------------------------------------------------------------------------- //
 void guSoListBox::GetItemsList( void )
 {
-    m_Db->GetSongs( &m_Items );
+    //m_Db->GetSongs( &m_Items, GetFirstVisibleLine(), GetLastVisibleLine() );
+    SetItemCount( m_Db->GetSongsCount() );
 
     wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_MAINFRAME_UPDATE_SELINFO );
     AddPendingEvent( event );
@@ -221,21 +261,23 @@ void guSoListBox::ReloadItems( bool reset )
     if( reset )
         SetSelection( -1 );
     else
-        Selection = GetSelectedItems( false );
+        Selection = GetSelectedIndexs( false );
 
     m_ItemsMutex.Lock();
 
     m_Items.Empty();
+    m_ItemsFirst = -1;
+    m_ItemsLast = -1;
 
     GetItemsList();
 
-    SetItemCount( m_Items.Count() );
+    //SetItemCount( m_Items.Count() );
 
     m_ItemsMutex.Unlock();
 
     if( !reset )
     {
-      SetSelectedItems( Selection );
+      SetSelectedIndexs( Selection );
       ScrollToLine( FirstVisible );
     }
     RefreshAll();
@@ -249,7 +291,15 @@ int guSoListBox::GetSelectedSongs( guTrackArray * tracks )
     int item = GetFirstSelected( cookie );
     while( item != wxNOT_FOUND )
     {
-        tracks->Add( new guTrack( m_Items[ item ] ) );
+        guTrack * Track = new guTrack();
+        if( m_Db->GetSong( GetItemId( item ), Track ) )
+        {
+            tracks->Add( Track );
+        }
+        else
+        {
+            delete Track;
+        }
         item = GetNextSelected( cookie );
     }
     m_ItemsMutex.Unlock();
@@ -264,7 +314,15 @@ void guSoListBox::GetAllSongs( guTrackArray * tracks )
     int count = m_Items.Count();
     for( index = 0; index < count; index++ )
     {
-        tracks->Add( new guTrack( m_Items[ index ] ) );
+        guTrack * Track = new guTrack();
+        if( m_Db->GetSong( GetItemId( index ), Track ) )
+        {
+            tracks->Add( Track );
+        }
+        else
+        {
+            delete Track;
+        }
     }
     m_ItemsMutex.Unlock();
 }
@@ -277,8 +335,8 @@ int guSoListBox::GetDragFiles( wxFileDataObject * files )
     int count = GetSelectedSongs( &Songs );
     for( index = 0; index < count; index++ )
     {
-       wxString FileName = Songs[ index ].m_FileName;
-       FileName.Replace( wxT( "#" ), wxT( "%23" ) );
+       wxString FileName = guFileDnDEncode( Songs[ index ].m_FileName );
+       //FileName.Replace( wxT( "#" ), wxT( "%23" ) );
        //FileName.Replace( wxT( "%" ), wxT( "%25" ) );
        //guLogMessage( wxT( "Adding song '%s'" ), Songs[ index ].m_FileName.c_str() );
        files->AddFile( FileName );
@@ -287,12 +345,13 @@ int guSoListBox::GetDragFiles( wxFileDataObject * files )
 }
 
 // -------------------------------------------------------------------------------- //
-wxArrayString guSoListBox::GetColumnNames( void )
+wxArrayString guSoListBox::GetColumnNames( void ) const
 {
     wxArrayString ColumnNames;
     ColumnNames.Add( wxT( "#" ) );
     ColumnNames.Add( _( "Title" ) );
     ColumnNames.Add( _( "Artist" ) );
+    ColumnNames.Add( _( "Album Artist" ) );
     ColumnNames.Add( _( "Album" ) );
     ColumnNames.Add( _( "Genre" ) );
     ColumnNames.Add( _( "Composer" ) );
@@ -304,6 +363,7 @@ wxArrayString guSoListBox::GetColumnNames( void )
     ColumnNames.Add( _( "PlayCount" ) );
     ColumnNames.Add( _( "Last Play" ) );
     ColumnNames.Add( _( "Added Date" ) );
+    ColumnNames.Add( _( "Format" ) );
     return ColumnNames;
 }
 
@@ -345,40 +405,135 @@ void AddSongsCommands( wxMenu * Menu, int SelCount )
     }
 }
 
+// -------------------------------------------------------------------------------- //
+void guSoListBox::AppendFastEditMenu( wxMenu * menu, const int selcount ) const
+{
+    wxMenuItem * MenuItem;
+    //guLogMessage( wxT( "guSoLisBox::AppendFastEditMenu %i - %i" ), m_LastColumnRightClicked, m_LastRowRightClicked );
+
+    int ColumnId = GetColumnId( m_LastColumnRightClicked );
+
+    // If its a column not editable
+    if( ColumnId == guSONGS_COLUMN_LENGTH ||
+        ColumnId == guSONGS_COLUMN_BITRATE ||
+        ColumnId == guSONGS_COLUMN_PLAYCOUNT ||
+        ColumnId == guSONGS_COLUMN_LASTPLAY ||
+        ColumnId == guSONGS_COLUMN_ADDEDDATE ||
+        ColumnId == guSONGS_COLUMN_FORMAT )
+        return;
+
+    wxArrayString ColumnNames = GetColumnNames();
+
+    wxString MenuText = _( "Edit" );
+    MenuText += wxT( " " ) + ColumnNames[ ColumnId ];
+
+    MenuItem = new wxMenuItem( menu, ID_SONG_EDIT_COLUMN,  MenuText, _( "Edit the clicked column for the selected tracks" ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit ) );
+    menu->Append( MenuItem );
+
+    if( selcount > 1 )
+    {
+        if( ColumnId == guSONGS_COLUMN_RATING )
+        {
+            wxMenu * RatingMenu = new wxMenu();
+
+            int Rating = m_Items[ m_LastRowRightClicked - m_ItemsFirst ].m_Rating;
+
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_0, wxT( "☆☆☆☆☆" ), _( "Set the rating to 0" ), wxITEM_CHECK );
+            RatingMenu->Append( MenuItem );
+            MenuItem->Check( Rating <= 0 );
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_1, wxT( "★☆☆☆☆" ), _( "Set the rating to 1" ), wxITEM_CHECK );
+            RatingMenu->Append( MenuItem );
+            MenuItem->Check( Rating == 1 );
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_2, wxT( "★★☆☆☆" ), _( "Set the rating to 2" ), wxITEM_CHECK );
+            RatingMenu->Append( MenuItem );
+            MenuItem->Check( Rating == 2 );
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_3, wxT( "★★★☆☆" ), _( "Set the rating to 3" ), wxITEM_CHECK );
+            RatingMenu->Append( MenuItem );
+            MenuItem->Check( Rating == 3 );
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_4, wxT( "★★★★☆" ), _( "Set the rating to 4" ), wxITEM_CHECK );
+            RatingMenu->Append( MenuItem );
+            MenuItem->Check( Rating == 4 );
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_5, wxT( "★★★★★" ), _( "Set the rating to 5" ), wxITEM_CHECK );
+            RatingMenu->Append( MenuItem );
+            MenuItem->Check( Rating == 5 );
+
+            menu->AppendSubMenu( RatingMenu, _( "Set Rating" ), _( "Set the current track rating" ) );
+        }
+        else
+        {
+            MenuText = _( "Set" );
+            MenuText += wxT( " " ) + ColumnNames[ ColumnId ] + wxT( " " );
+            MenuText += _( "to" );
+            MenuText += wxT( " '" ) + OnGetItemText( m_LastRowRightClicked, m_LastColumnRightClicked ) + wxT( "'" );
+
+            MenuItem = new wxMenuItem( menu, ID_SONG_SET_COLUMN,  MenuText, _( "Set the clicked column for the selected tracks" ) );
+            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit ) );
+            menu->Append( MenuItem );
+        }
+    }
+
+    menu->AppendSeparator();
+}
+
 // -------------------------------------------------------------------------------- //
 void guSoListBox::CreateContextMenu( wxMenu * Menu ) const
 {
     wxMenuItem * MenuItem;
-    int SelCount = GetSelectedItems().Count();
+    int SelCount = GetSelectedCount();
 
-    MenuItem = new wxMenuItem( Menu, ID_SONG_PLAY, _( "Play" ), _( "Play current selected songs" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_playback_start ) );
-    Menu->Append( MenuItem );
+    if( SelCount )
+    {
+        MenuItem = new wxMenuItem( Menu, ID_SONG_PLAY, _( "Play" ), _( "Play current selected songs" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_player_tiny_light_play ) );
+        Menu->Append( MenuItem );
+    }
 
     MenuItem = new wxMenuItem( Menu, ID_SONG_PLAYALL, _( "Play All" ), _( "Play all songs" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_playback_start ) );
-    Menu->Append( MenuItem );
-
-    MenuItem = new wxMenuItem( Menu, ID_SONG_ENQUEUE, _( "Enqueue" ), _( "Add current selected songs to the playlist" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
-    Menu->Append( MenuItem );
-
-    MenuItem = new wxMenuItem( Menu, ID_SONG_ENQUEUEALL, _( "Enqueue All" ), _( "Add all songs to the playlist" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
-    Menu->Append( MenuItem );
-
-    Menu->AppendSeparator();
-
-    MenuItem = new wxMenuItem( Menu, ID_SONG_SAVEPLAYLIST, _( "Save Playlist" ), _( "Save all selected tracks as a playlist" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_doc_save ) );
-    Menu->Append( MenuItem );
-
-    MenuItem = new wxMenuItem( Menu, ID_SONG_COPYTO, _( "Copy to..." ), _( "Copy the current selected songs to a directory or device" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit_copy ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_player_tiny_light_play ) );
     Menu->Append( MenuItem );
 
     if( SelCount )
     {
+        MenuItem = new wxMenuItem( Menu, ID_SONG_ENQUEUE, _( "Enqueue" ), _( "Add current selected songs to the playlist" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+        Menu->Append( MenuItem );
+
+        MenuItem = new wxMenuItem( Menu, ID_SONG_ENQUEUE_ASNEXT, _( "Enqueue Next" ), _( "Add current selected songs to the playlist as Next Tracks" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+        Menu->Append( MenuItem );
+    }
+
+    MenuItem = new wxMenuItem( Menu, ID_SONG_ENQUEUEALL, _( "Enqueue All" ), _( "Add all songs to the playlist" ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+    Menu->Append( MenuItem );
+
+    MenuItem = new wxMenuItem( Menu, ID_SONG_ENQUEUEALL_ASNEXT, _( "Enqueue All Next" ), _( "Add all songs to the playlist as Next Tracks" ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+    Menu->Append( MenuItem );
+
+    if( SelCount )
+    {
+        Menu->AppendSeparator();
+
+        MenuItem = new wxMenuItem( Menu, ID_SONG_DELETE_LIBRARY, _( "Remove from Library" ), _( "Remove the current selected tracks from library" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit_clear ) );
+        Menu->Append( MenuItem );
+
+        MenuItem = new wxMenuItem( Menu, ID_SONG_DELETE_DRIVE, _( "Delete from Drive" ), _( "Remove the current selected tracks from drive" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit_clear ) );
+        Menu->Append( MenuItem );
+
+        Menu->AppendSeparator();
+
+        MenuItem = new wxMenuItem( Menu, ID_SONG_SAVEPLAYLIST, _( "Save to Playlist" ), _( "Save all selected tracks as a playlist" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_doc_save ) );
+        Menu->Append( MenuItem );
+
+        MenuItem = new wxMenuItem( Menu, ID_SONG_COPYTO, _( "Copy to..." ), _( "Copy the current selected songs to a directory or device" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit_copy ) );
+        Menu->Append( MenuItem );
+
         Menu->AppendSeparator();
 
         MenuItem = new wxMenuItem( Menu, ID_SONG_EDITLABELS, _( "Edit Labels" ), _( "Edit the labels assigned to the selected songs" ) );
@@ -391,17 +546,24 @@ void guSoListBox::CreateContextMenu( wxMenu * Menu ) const
 
         Menu->AppendSeparator();
 
-        MenuItem = new wxMenuItem( Menu, ID_SONG_BROWSE_GENRE, _( "Select Genre" ), _( "Selects the genre of the current song" ) );
-        //MenuItem->SetBitmap( guImage( guIMAGE_INDEX_search ) );
-        Menu->Append( MenuItem );
+        AppendFastEditMenu( Menu, SelCount );
 
-        MenuItem = new wxMenuItem( Menu, ID_SONG_BROWSE_ARTIST, _( "Select Artist" ), _( "Selects the artist of the current song" ) );
-        //MenuItem->SetBitmap( guImage( guIMAGE_INDEX_search ) );
-        Menu->Append( MenuItem );
+        wxMenu *     SubMenu;
+        SubMenu = new wxMenu();
 
-        MenuItem = new wxMenuItem( Menu, ID_SONG_BROWSE_ALBUM, _( "Select Album" ), _( "Select the album of the current song" ) );
-        //MenuItem->SetBitmap( guImage( guIMAGE_INDEX_search ) );
-        Menu->Append( MenuItem );
+        MenuItem = new wxMenuItem( Menu, ID_SONG_BROWSE_GENRE, _( "Genre" ), _( "Selects the genre of the current song" ) );
+        SubMenu->Append( MenuItem );
+
+        MenuItem = new wxMenuItem( Menu, ID_SONG_BROWSE_ARTIST, _( "Artist" ), _( "Selects the artist of the current song" ) );
+        SubMenu->Append( MenuItem );
+
+        MenuItem = new wxMenuItem( Menu, ID_SONG_BROWSE_ALBUMARTIST, _( "Album Artist" ), _( "Selects the album artist of the current song" ) );
+        SubMenu->Append( MenuItem );
+
+        MenuItem = new wxMenuItem( Menu, ID_SONG_BROWSE_ALBUM, _( "Album" ), _( "Select the album of the current song" ) );
+        SubMenu->Append( MenuItem );
+
+        Menu->AppendSubMenu( SubMenu, _( "Select" ), _( "Search in the library" ) );
 
         Menu->AppendSeparator();
 
@@ -474,7 +636,8 @@ void guSoListBox::OnCommandClicked( wxCommandEvent &event )
                 count = AlbumPaths.Count();
                 for( index = 0; index < count; index++ )
                 {
-                    Paths += wxT( " \"" ) + AlbumPaths[ index ] + wxT( "\"" );
+                    AlbumPaths[ index ].Replace( wxT( " " ), wxT( "\\ " ) );
+                    Paths += wxT( " " ) + AlbumPaths[ index ];
                 }
                 CurCmd.Replace( wxT( "{bp}" ), Paths.Trim( false ) );
             }
@@ -519,21 +682,39 @@ void guSoListBox::OnCommandClicked( wxCommandEvent &event )
 // -------------------------------------------------------------------------------- //
 wxString guSoListBox::GetSearchText( int item ) const
 {
-    return wxString::Format( wxT( "\"%s\" \"%s\"" ),
-        m_Items[ item ].m_ArtistName.c_str(),
-        m_Items[ item ].m_SongName.c_str() );
+    if( item >= m_ItemsFirst && item < m_ItemsLast )
+    {
+        return wxString::Format( wxT( "\"%s\" \"%s\"" ),
+            m_Items[ item - m_ItemsFirst ].m_ArtistName.c_str(),
+            m_Items[ item - m_ItemsFirst ].m_SongName.c_str() );
+    }
+    return wxEmptyString;
 }
 
 // -------------------------------------------------------------------------------- //
 wxString guSoListBox::GetItemName( const int row ) const
 {
-    return m_Items[ row ].m_SongName;
+    if( row >= m_ItemsFirst && row < m_ItemsLast )
+    {
+        return m_Items[ row - m_ItemsFirst ].m_SongName;
+    }
+    else
+    {
+        return m_Db->GetSongsName( row );
+    }
 }
 
 // -------------------------------------------------------------------------------- //
 int guSoListBox::GetItemId( const int row ) const
 {
-    return m_Items[ row ].m_SongId;
+    if( row >= m_ItemsFirst && row < m_ItemsLast )
+    {
+        return m_Items[ row - m_ItemsFirst ].m_SongId;
+    }
+    else
+    {
+        return m_Db->GetSongsId( row );
+    }
 }
 
 // -------------------------------------------------------------------------------- //
@@ -553,17 +734,67 @@ void guSoListBox::OnItemColumnClicked( wxListEvent &event )
             Rating = wxMin( 5, ( wxMax( 0, MouseX - 3 ) / w ) + 1 );
 
         int Row = event.GetInt();
-        if( m_Items[ Row ].m_Rating == Rating )
+        if( m_Items[ Row - m_ItemsFirst ].m_Rating == Rating )
             Rating = 0;
-        m_Items[ Row ].m_Rating = Rating;
-        m_Db->SetTrackRating( m_Items[ Row ].m_SongId, Rating );
-        //RefreshLine( Row );
 
-        // Update the track in database, playlist, etc
-        ( ( guMainFrame * ) wxTheApp->GetTopWindow() )->UpdatedTrack( guUPDATED_TRACKS_NONE, &m_Items[ Row ] );
+//        m_Items[ Row - m_ItemsFirst ].m_Rating = Rating;
+//        m_Db->SetTrackRating( m_Items[ Row - m_ItemsFirst ].m_SongId, Rating );
+//        //RefreshLine( Row );
+//
+//        // Update the track in database, playlist, etc
+//        ( ( guMainFrame * ) wxTheApp->GetTopWindow() )->UpdatedTrack( guUPDATED_TRACKS_NONE, &m_Items[ Row - m_ItemsFirst ] );
+        wxCommandEvent RatingEvent( wxEVT_COMMAND_MENU_SELECTED, ID_SONG_SET_RATING_0 + Rating );
+        AddPendingEvent( RatingEvent );
     }
 }
 
+// -------------------------------------------------------------------------------- //
+void guSoListBox::OnItemColumnRClicked( wxListEvent &event )
+{
+    m_LastColumnRightClicked = event.m_col;
+    m_LastRowRightClicked = event.GetInt();
+    //guLogMessage( wxT( "Column %i Row %i Right Clicked..." ), m_LastColumnRightClicked, event.GetInt() );
+}
+
+// -------------------------------------------------------------------------------- //
+wxVariant guSoListBox::GetLastDataClicked( void )
+{
+    guTrack * Track = &m_Items[ m_LastRowRightClicked - m_ItemsFirst ];
+
+    int ColId = GetColumnId( m_LastColumnRightClicked );
+    switch( ColId )
+    {
+        case guSONGS_COLUMN_NUMBER :
+            return wxVariant( ( long ) Track->m_Number );
+
+        case guSONGS_COLUMN_TITLE :
+            return wxVariant( Track->m_SongName );
+
+        case guSONGS_COLUMN_ARTIST :
+            return wxVariant( Track->m_ArtistName );
+
+        case guSONGS_COLUMN_ALBUMARTIST :
+            return wxVariant( Track->m_AlbumArtist );
+
+        case guSONGS_COLUMN_ALBUM :
+            return wxVariant( Track->m_AlbumName );
+
+        case guSONGS_COLUMN_GENRE :
+            return wxVariant( Track->m_GenreName );
+
+        case guSONGS_COLUMN_COMPOSER :
+            return wxVariant( Track->m_Composer );
+
+        case guSONGS_COLUMN_DISK :
+            return wxVariant( Track->m_Disk );
+
+        case guSONGS_COLUMN_YEAR :
+            return wxVariant( ( long ) Track->m_Year );
+    }
+
+    return wxVariant();
+}
+
 // -------------------------------------------------------------------------------- //
 void guSoListBox::UpdatedTracks( const guTrackArray * tracks )
 {
@@ -584,7 +815,7 @@ void guSoListBox::UpdatedTracks( const guTrackArray * tracks )
             if( CurTrack.m_FileName == m_Items[ ItemIndex ].m_FileName )
             {
                 m_Items[ ItemIndex ] = CurTrack;
-                RefreshLine( ItemIndex );
+                RefreshLine( ItemIndex + m_ItemsFirst );
             }
         }
     }
@@ -610,3 +841,18 @@ void guSoListBox::UpdatedTrack( const guTrack * track )
 }
 
 // -------------------------------------------------------------------------------- //
+int guSoListBox::FindItem( const int trackid )
+{
+    int Index;
+    int Count = m_Items.Count();
+    for( Index = 0; Index < Count; Index++ )
+    {
+        if( m_Items[ Index ].m_SongId == trackid )
+        {
+            return Index;
+        }
+    }
+    return wxNOT_FOUND;
+}
+
+// -------------------------------------------------------------------------------- //
