@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -145,7 +145,7 @@ wxImage * GetID3v2Image( ID3v2::Tag * tagv2 )
 {
     TagLib::ID3v2::FrameList FrameList = tagv2->frameListMap()["APIC"];
 
-    wxImage * CoverImage = GetID3v2ImageType( FrameList );
+    wxImage * CoverImage = GetID3v2ImageType( FrameList, TagLib::ID3v2::AttachedPictureFrame::FrontCover );
 
     if( !CoverImage )
     {
@@ -325,17 +325,78 @@ bool SetXiphCommentLyrics( Ogg::XiphComment * xiphcomment, const wxString &lyric
     return false;
 }
 
-//// -------------------------------------------------------------------------------- //
-//wxImage * GetMp4Image( TagLib::MP4::Tag * mp4tag )
-//{
-//    return NULL;
-//}
-//
-//// -------------------------------------------------------------------------------- //
-//bool SetMp4Image( TagLib::MP4::Tag * mp4tag, const wxImage * image )
-//{
-//    return false;
-//}
+#ifdef TAGLIB_WITH_MP4_COVERS
+// -------------------------------------------------------------------------------- //
+wxImage * GetMp4Image( TagLib::MP4::Tag * mp4tag )
+{
+    if( mp4tag && mp4tag->itemListMap().contains( "covr" ) )
+    {
+        TagLib::MP4::CoverArtList Covers = mp4tag->itemListMap()[ "covr" ].toCoverArtList();
+
+        for( TagLib::MP4::CoverArtList::Iterator it = Covers.begin(); it != Covers.end(); it++ )
+        {
+            int ImgType = wxBITMAP_TYPE_INVALID;
+            if( it->format() == TagLib::MP4::CoverArt::PNG )
+            {
+                ImgType = wxBITMAP_TYPE_PNG;
+            }
+            else if( it->format() == TagLib::MP4::CoverArt::JPEG )
+            {
+                ImgType = wxBITMAP_TYPE_JPEG;
+            }
+
+            wxMemoryOutputStream ImgOutStream;
+            ImgOutStream.Write( it->data().data(), it->data().size() );
+            wxMemoryInputStream ImgInputStream( ImgOutStream );
+            wxImage * CoverImage = new wxImage( ImgInputStream, ImgType );
+            if( CoverImage )
+            {
+                if( CoverImage->IsOk() )
+                {
+                    return CoverImage;
+                }
+                else
+                {
+                    delete CoverImage;
+                }
+            }
+        }
+    }
+    return NULL;
+}
+
+// -------------------------------------------------------------------------------- //
+bool SetMp4Image( TagLib::MP4::Tag * mp4tag, const wxImage * image )
+{
+    if( mp4tag )
+    {
+        if( mp4tag->itemListMap().contains( "covr" ) )
+        {
+            mp4tag->itemListMap().erase( "covr" );
+        }
+
+        if( image )
+        {
+            wxMemoryOutputStream ImgOutputStream;
+            if( image && image->SaveFile( ImgOutputStream, wxBITMAP_TYPE_JPEG ) )
+            {
+                ByteVector ImgData( ( TagLib::uint ) ImgOutputStream.GetSize() );
+                ImgOutputStream.CopyTo( ImgData.data(), ImgOutputStream.GetSize() );
+
+                TagLib::MP4::CoverArtList CoverList;
+                TagLib::MP4::CoverArt Cover( TagLib::MP4::CoverArt::JPEG, ImgData );
+                CoverList.append( Cover );
+                mp4tag->itemListMap()[ "covr" ] = CoverList;
+
+                return true;
+            }
+            return false;
+        }
+        return true;
+    }
+    return false;
+}
+#endif
 
 // -------------------------------------------------------------------------------- //
 wxString GetMp4Lyrics( TagLib::MP4::Tag * mp4tag )
@@ -367,7 +428,57 @@ bool SetMp4Lyrics( TagLib::MP4::Tag * mp4tag, const wxString &lyrics )
     return false;
 }
 
+// -------------------------------------------------------------------------------- //
+wxImage * GetApeItemImage( const TagLib::APE::Item &item )
+{
+    if( item.type() == TagLib::APE::Item::Binary )
+    {
+        TagLib::ByteVector CoverData = item.value();
 
+        if( CoverData.size() )
+        {
+            wxMemoryOutputStream ImgOutStream;
+            ImgOutStream.Write( CoverData.data(), CoverData.size() );
+            wxMemoryInputStream ImgInputStream( ImgOutStream );
+            wxImage * CoverImage = new wxImage( ImgInputStream, wxBITMAP_TYPE_JPEG );
+            if( CoverImage )
+            {
+                if( CoverImage->IsOk() )
+                {
+                    return CoverImage;
+                }
+                else
+                {
+                    delete CoverImage;
+                }
+            }
+        }
+    }
+    return NULL;
+}
+
+// -------------------------------------------------------------------------------- //
+wxImage * GetApeImage( TagLib::APE::Tag * apetag )
+{
+    if( apetag )
+    {
+        if( apetag->itemListMap().contains( "Cover Art (front)" ) )
+        {
+            return GetApeItemImage( apetag->itemListMap()[ "Cover Art (front)" ] );
+        }
+        else if( apetag->itemListMap().contains( "Cover Art (other)" ) )
+        {
+            return GetApeItemImage( apetag->itemListMap()[ "Cover Art (other)" ] );
+        }
+    }
+    return NULL;
+}
+
+// -------------------------------------------------------------------------------- //
+bool SetApeImage( TagLib::APE::Tag * apetag, const wxImage * image )
+{
+    return false;
+}
 
 
 // -------------------------------------------------------------------------------- //
@@ -381,6 +492,7 @@ guTagInfo::guTagInfo( const wxString &filename )
     m_Year = 0;
     m_Length = 0;
     m_Bitrate = 0;
+    m_Compilation = false;
 };
 
 // -------------------------------------------------------------------------------- //
@@ -396,7 +508,8 @@ void guTagInfo::SetFileName( const wxString &filename )
     m_FileName = filename;
     if( !filename.IsEmpty() )
     {
-        m_TagFile = new TagLib::FileRef( filename.ToUTF8(), true, TagLib::AudioProperties::Fast );
+        m_TagFile = new TagLib::FileRef( filename.mb_str( wxConvFile ), true, TagLib::AudioProperties::Fast );
+        //m_TagFile = new TagLib::FileRef( filename.ToUTF8(), true, TagLib::AudioProperties::Fast );
     }
     else
     {
@@ -424,9 +537,17 @@ bool guTagInfo::Read( void )
     if( m_Tag )
     {
         m_TrackName = TStringTowxString( m_Tag->title() );
+        if( m_TrackName.IsEmpty() )
+            m_TrackName = m_FileName.AfterLast( wxT( '/' ) );
         m_ArtistName = TStringTowxString( m_Tag->artist() );
+        if( m_ArtistName.IsEmpty() )
+            m_ArtistName = _( "Unknown" );
         m_AlbumName = TStringTowxString( m_Tag->album() );
+        if( m_AlbumName.IsEmpty() )
+            m_AlbumName = m_FileName.BeforeLast( wxT( '/' ) ).AfterLast( wxT( '/' ) );
         m_GenreName = TStringTowxString( m_Tag->genre() );
+        if( m_GenreName.IsEmpty() )
+            m_GenreName = _( "Unknown" );
         m_Comments = TStringTowxString( m_Tag->comment() );
         m_Track = m_Tag->track();
         m_Year = m_Tag->year();
@@ -541,6 +662,16 @@ bool guMp3TagInfo::Read( void )
                 m_Composer = TStringTowxString( m_TagId3v2->frameListMap()[ "TCOM" ].front()->toString() );
             }
 
+            if( m_TagId3v2->frameListMap().contains( "TPE2" ) )
+            {
+                m_AlbumArtist = TStringTowxString( m_TagId3v2->frameListMap()[ "TPE2" ].front()->toString() );
+            }
+
+            if( m_TagId3v2->frameListMap().contains( "TCMP" ) )
+            {
+                m_Compilation = TStringTowxString( m_TagId3v2->frameListMap()[ "TCMP" ].front()->toString() ) == wxT( "1" );
+            }
+
             if( m_TrackLabels.Count() == 0 )
             {
                 ID3v2::UserTextIdentificationFrame * Frame = ID3v2::UserTextIdentificationFrame::find( m_TagId3v2, "guTRLABELS" );
@@ -624,6 +755,16 @@ bool guMp3TagInfo::Write( void )
         frame->setText( wxStringToTString( m_Composer ) );
         m_TagId3v2->addFrame( frame );
 
+        m_TagId3v2->removeFrames( "TPE2" );
+        frame = new TagLib::ID3v2::TextIdentificationFrame( "TPE2" );
+        frame->setText( wxStringToTString( m_AlbumArtist ) );
+        m_TagId3v2->addFrame( frame );
+
+        m_TagId3v2->removeFrames( "TCMP" );
+        frame = new TagLib::ID3v2::TextIdentificationFrame( "TCMP" );
+        frame->setText( wxStringToTString( wxString::Format( wxT( "%u" ), m_Compilation ) ) );
+        m_TagId3v2->addFrame( frame );
+
         // I have found several TRCK fields in the mp3s
         m_TagId3v2->removeFrames( "TRCK" );
         m_TagId3v2->setTrack( m_Track );
@@ -722,12 +863,29 @@ bool guFlacTagInfo::Read( void )
         if( m_XiphComment )
         {
             if( m_XiphComment->fieldListMap().contains( "COMPOSER" ) )
+            {
                 m_Composer = TStringTowxString( m_XiphComment->fieldListMap()["COMPOSER"].front() );
+            }
 
             if( m_XiphComment->fieldListMap().contains( "DISCNUMBER" ) )
             {
                 m_Disk = TStringTowxString( m_XiphComment->fieldListMap()["DISCNUMBER"].front() );
             }
+
+            if( m_XiphComment->fieldListMap().contains( "COMPILATION" ) )
+            {
+                m_Compilation = TStringTowxString( m_XiphComment->fieldListMap()["COMPILATION"].front() ) == wxT( "1" );
+            }
+
+            if( m_XiphComment->fieldListMap().contains( "ALBUMARTIST" ) )
+            {
+                m_AlbumArtist = TStringTowxString( m_XiphComment->fieldListMap()["ALBUMARTIST"].front() );
+            }
+            else if( m_XiphComment->fieldListMap().contains( "ALBUM ARTIST" ) )
+            {
+                m_AlbumArtist = TStringTowxString( m_XiphComment->fieldListMap()["ALBUM ARTIST"].front() );
+            }
+
             return true;
         }
     }
@@ -741,6 +899,8 @@ bool guFlacTagInfo::Write( void )
     {
         m_XiphComment->addField( "DISCNUMBER", wxStringToTString( m_Disk ) );
         m_XiphComment->addField( "COMPOSER", wxStringToTString( m_Composer ) );
+        m_XiphComment->addField( "COMPILATION", wxStringToTString( wxString::Format( wxT( "%u" ), m_Compilation ) ) );
+        m_XiphComment->addField( "ALBUMARTIST", wxStringToTString(  m_AlbumArtist ) );
     }
     return guTagInfo::Write();
 }
@@ -759,7 +919,7 @@ wxImage * guFlacTagInfo::GetImage( void )
     FLAC__Metadata_SimpleIterator * iter = FLAC__metadata_simple_iterator_new();
     if( iter )
     {
-        if( FLAC__metadata_simple_iterator_init( iter, m_FileName.ToUTF8(), true, false ) )
+        if( FLAC__metadata_simple_iterator_init( iter, m_FileName.mb_str( wxConvFile ), true, false ) )
         {
             while( FLAC__metadata_simple_iterator_next( iter ) )
             {
@@ -804,7 +964,7 @@ bool guFlacTagInfo::SetImage( const wxImage * image )
     Chain = FLAC__metadata_chain_new();
     if( Chain )
     {
-        if( FLAC__metadata_chain_read( Chain, m_FileName.ToUTF8() ) )
+        if( FLAC__metadata_chain_read( Chain, m_FileName.mb_str( wxConvFile ) ) )
         {
             Iter = FLAC__metadata_iterator_new();
             if( Iter )
@@ -931,12 +1091,29 @@ bool guOggTagInfo::Read( void )
         if( m_XiphComment )
         {
             if( m_XiphComment->fieldListMap().contains( "COMPOSER" ) )
+            {
                 m_Composer = TStringTowxString( m_XiphComment->fieldListMap()["COMPOSER"].front() );
+            }
 
             if( m_XiphComment->fieldListMap().contains( "DISCNUMBER" ) )
             {
                 m_Disk = TStringTowxString( m_XiphComment->fieldListMap()["DISCNUMBER"].front() );
             }
+
+            if( m_XiphComment->fieldListMap().contains( "COMPILATION" ) )
+            {
+                m_Compilation = TStringTowxString( m_XiphComment->fieldListMap()["COMPILATION"].front() ) == wxT( "1" );
+            }
+
+            if( m_XiphComment->fieldListMap().contains( "ALBUMARTIST" ) )
+            {
+                m_AlbumArtist = TStringTowxString( m_XiphComment->fieldListMap()["ALBUMARTIST"].front() );
+            }
+            else if( m_XiphComment->fieldListMap().contains( "ALBUM ARTIST" ) )
+            {
+                m_AlbumArtist = TStringTowxString( m_XiphComment->fieldListMap()["ALBUM ARTIST"].front() );
+            }
+
             return true;
         }
     }
@@ -950,6 +1127,8 @@ bool guOggTagInfo::Write( void )
     {
         m_XiphComment->addField( "DISCNUMBER", wxStringToTString( m_Disk ) );
         m_XiphComment->addField( "COMPOSER", wxStringToTString( m_Composer ) );
+        m_XiphComment->addField( "COMPILATION", wxStringToTString( wxString::Format( wxT( "%u" ), m_Compilation ) ) );
+        m_XiphComment->addField( "ALBUMARTIST", wxStringToTString(  m_AlbumArtist ) );
     }
     return guTagInfo::Write();
 }
@@ -1018,8 +1197,15 @@ bool guMp4TagInfo::Read( void )
     {
         if( m_Mp4Tag )
         {
+            if( m_Mp4Tag->itemListMap().contains( "aArt" ) )
+            {
+                m_AlbumArtist = TStringTowxString( m_Mp4Tag->itemListMap()["aArt"].toStringList().front() );
+            }
+
             if( m_Mp4Tag->itemListMap().contains( "\xA9wrt" ) )
+            {
                 m_Composer = TStringTowxString( m_Mp4Tag->itemListMap()["\xa9wrt"].toStringList().front() );
+            }
 
             if( m_Mp4Tag->itemListMap().contains( "disk" ) )
             {
@@ -1028,6 +1214,12 @@ bool guMp4TagInfo::Read( void )
                     m_Mp4Tag->itemListMap()["disk"].toIntPair().second );
 
             }
+
+            if( m_Mp4Tag->itemListMap().contains( "cpil" ) )
+            {
+                m_Compilation = m_Mp4Tag->itemListMap()["cpil"].toBool();
+            }
+
         }
         return true;
     }
@@ -1065,123 +1257,36 @@ bool guMp4TagInfo::Write( void )
 {
     if( m_Mp4Tag )
     {
+        m_Mp4Tag->itemListMap()["aArt"] = TagLib::StringList( wxStringToTString( m_AlbumArtist ) );
         m_Mp4Tag->itemListMap()["\xA9wrt"] = TagLib::StringList( wxStringToTString( m_Composer ) );
         int first;
         int second;
         guStrDiskToDiskNum( m_Disk, first, second );
         m_Mp4Tag->itemListMap()["disk"] = TagLib::MP4::Item( first, second );
-            //mp4tag->itemListMap()["disk"] = TagLib::MP4::Item( discnumber, 0 );
+        m_Mp4Tag->itemListMap()["cpil"] = TagLib::MP4::Item( m_Compilation );
     }
     return guTagInfo::Write();
 }
 
-//// -------------------------------------------------------------------------------- //
-//bool guMp4TagInfo::CanHandleImages( void )
-//{
-//    return true;
-//}
-//
-//// -------------------------------------------------------------------------------- //
-//wxImage * guMp4TagInfo::GetImage( void )
-//{
-//    MP4FileHandle mp4_file = MP4Read( m_FileName.ToUTF8() );
-//    if( mp4_file != MP4_INVALID_FILE_HANDLE )
-//    {
-//        if( MP4GetMetadataCoverArtCount( mp4_file ) )
-//        {
-//            uint8_t *   CoverData;
-//            uint32_t    CoverSize;
-//            if( MP4GetMetadataCoverArt( mp4_file, &CoverData, &CoverSize ) )
-//            {
-//                wxMemoryOutputStream ImgOutStream;
-//                ImgOutStream.Write( CoverData, CoverSize );
-//                wxMemoryInputStream ImgInputStream( ImgOutStream );
-//                // TODO : Determine image type from data stream
-//                wxImage * CoverImage = new wxImage( ImgInputStream, wxBITMAP_TYPE_JPEG );
-//                if( !CoverImage || !CoverImage->IsOk() )
-//                {
-//                    if( CoverImage )
-//                        delete CoverImage;
-//                    CoverImage = new wxImage( ImgInputStream, wxBITMAP_TYPE_PNG );
-//                }
-//
-//                if( CoverImage )
-//                {
-//                    if( CoverImage->IsOk() )
-//                    {
-//                        return CoverImage;
-//                    }
-//                    else
-//                    {
-//                        delete CoverImage;
-//                    }
-//                }
-//            }
-//        }
-//        MP4Close( mp4_file );
-//    }
-//    else
-//    {
-//        guLogError( wxT( "GetImage : could not open the file %s" ), m_FileName.c_str() );
-//    }
-//    return NULL;
-//}
-//
-//// -------------------------------------------------------------------------------- //
-//bool guMp4TagInfo::SetImage( const wxImage * image )
-//{
-//    bool RetVal = false;
-//    MP4FileHandle mp4_file = MP4Modify( m_FileName.ToUTF8() );
-//    if( mp4_file != MP4_INVALID_FILE_HANDLE )
-//    {
-//        if( image )
-//        {
-//            wxMemoryOutputStream ImgOutputStream;
-//            if( image->SaveFile( ImgOutputStream, wxBITMAP_TYPE_JPEG ) )
-//            {
-//                uint8_t * CoverData = ( uint8_t * ) malloc( ImgOutputStream.GetSize() );
-//                if( CoverData )
-//                {
-//                    ImgOutputStream.CopyTo( CoverData, ImgOutputStream.GetSize() );
-//                    RetVal = MP4SetMetadataCoverArt( mp4_file, CoverData, ImgOutputStream.GetSize() );
-//                    free( CoverData );
-//                }
-//                else
-//                {
-//                    guLogError( wxT( "could not allocate memory for image processing %s" ), m_FileName.c_str() );
-//                }
-//            }
-//        }
-//        else
-//        {
-//            RetVal = MP4DeleteMetadataCoverArt( mp4_file );
-//        }
-//        MP4Close( mp4_file );
-//    }
-//    else
-//    {
-//        guLogError( wxT( "SetImage : could not open the file %s" ), m_FileName.c_str() );
-//    }
-//    return RetVal;
-//}
+#ifdef TAGLIB_WITH_MP4_COVERS
+// -------------------------------------------------------------------------------- //
+bool guMp4TagInfo::CanHandleImages( void )
+{
+    return true;
+}
 
-//// -------------------------------------------------------------------------------- //
-//bool guMp4TagInfo::CanHandleImages( void )
-//{
-//    return true;
-//}
-//
-//// -------------------------------------------------------------------------------- //
-//wxImage * guMp4TagInfo::GetImage( void )
-//{
-//    return GetMp4Image( m_Mp4Tag );
-//}
-//
-//// -------------------------------------------------------------------------------- //
-//bool guMp4TagInfo::SetImage( const wxImage * image )
-//{
-//    return SetMp4Image( m_Mp4Tag, image ) && m_TagFile->save();
-//}
+// -------------------------------------------------------------------------------- //
+wxImage * guMp4TagInfo::GetImage( void )
+{
+    return GetMp4Image( m_Mp4Tag );
+}
+
+// -------------------------------------------------------------------------------- //
+bool guMp4TagInfo::SetImage( const wxImage * image )
+{
+    return SetMp4Image( m_Mp4Tag, image ) && m_Mp4Tag->save();
+}
+#endif
 
 // -------------------------------------------------------------------------------- //
 bool guMp4TagInfo::CanHandleLyrics( void )
@@ -1192,16 +1297,14 @@ bool guMp4TagInfo::CanHandleLyrics( void )
 // -------------------------------------------------------------------------------- //
 wxString guMp4TagInfo::GetLyrics( void )
 {
-    TagLib::MP4::File tagfile( m_FileName.ToUTF8() );
-    return GetMp4Lyrics( tagfile.tag() );
+    //TagLib::MP4::File tagfile( m_FileName.mb_str( wxConvFile ) );
+    return GetMp4Lyrics( ( ( TagLib::MP4::File * ) m_TagFile->file() )->tag() );
 }
 
 // -------------------------------------------------------------------------------- //
 bool guMp4TagInfo::SetLyrics( const wxString &lyrics )
 {
-    TagLib::MP4::File tagfile( m_FileName.ToUTF8() );
-
-    return SetMp4Lyrics( tagfile.tag(), lyrics ) && tagfile.save();
+    return SetMp4Lyrics( ( ( TagLib::MP4::File * ) m_TagFile->file() )->tag(), lyrics ) && m_TagFile->save();
 }
 
 
@@ -1212,6 +1315,12 @@ bool guMp4TagInfo::SetLyrics( const wxString &lyrics )
 // -------------------------------------------------------------------------------- //
 guMpcTagInfo::guMpcTagInfo( const wxString &filename ) : guTagInfo( filename )
 {
+    if( m_TagFile && !m_TagFile->isNull() )
+    {
+        m_ApeTag = ( ( TagLib::MPC::File * ) m_TagFile->file() )->APETag();
+    }
+    else
+        m_ApeTag = NULL;
 }
 
 // -------------------------------------------------------------------------------- //
@@ -1219,10 +1328,80 @@ guMpcTagInfo::~guMpcTagInfo()
 {
 }
 
+// -------------------------------------------------------------------------------- //
+bool guMpcTagInfo::Read( void )
+{
+    if( guTagInfo::Read() )
+    {
+        if( m_ApeTag )
+        {
+            if( m_ApeTag->itemListMap().contains( "COMPOSER" ) )
+            {
+                m_Composer = TStringTowxString( m_ApeTag->itemListMap()["COMPOSER"].toStringList().front() );
+            }
+
+            if( m_ApeTag->itemListMap().contains( "DISCNUMBER" ) )
+            {
+                m_Disk = TStringTowxString( m_ApeTag->itemListMap()["DISCNUMBER"].toStringList().front() );
+            }
+
+            if( m_ApeTag->itemListMap().contains( "COMPILATION" ) )
+            {
+                m_Compilation = TStringTowxString( m_ApeTag->itemListMap()["COMPILATION"].toStringList().front() ) == wxT( "1" );
+            }
+
+            if( m_ApeTag->itemListMap().contains( "ALBUM ARTIST" ) )
+            {
+                m_AlbumArtist = TStringTowxString( m_ApeTag->itemListMap()["ALBUM ARTIST"].toStringList().front() );
+            }
+            else if( m_ApeTag->itemListMap().contains( "ALBUMARTIST" ) )
+            {
+                m_AlbumArtist = TStringTowxString( m_ApeTag->itemListMap()["ALBUMARTIST"].toStringList().front() );
+            }
+        }
+        return true;
+    }
+    return false;
+}
+
+// -------------------------------------------------------------------------------- //
+bool guMpcTagInfo::Write( void )
+{
+    if( m_ApeTag )
+    {
+        m_ApeTag->addValue( "COMPOSER", wxStringToTString( m_Composer ) );
+        m_ApeTag->addValue( "DISCNUMBER", wxStringToTString( m_Disk ) );
+        m_ApeTag->addValue( "COMPILATION", wxStringToTString( wxString::Format( wxT( "%u" ), m_Compilation ) ) );
+        m_ApeTag->addValue( "ALBUM ARTIST", wxStringToTString( m_AlbumArtist ) );
+    }
+    return guTagInfo::Write();
+}
+
+// -------------------------------------------------------------------------------- //
+bool guMpcTagInfo::CanHandleImages( void )
+{
+    return true;
+}
+
+// -------------------------------------------------------------------------------- //
+wxImage * guMpcTagInfo::GetImage( void )
+{
+    return GetApeImage( m_ApeTag );
+}
+
+// -------------------------------------------------------------------------------- //
+bool guMpcTagInfo::SetImage( const wxImage * image )
+{
+    return m_ApeTag && SetApeImage( m_ApeTag, image ) && Write();
+}
+
+
+
+
 // -------------------------------------------------------------------------------- //
 // guApeTagInfo
 // -------------------------------------------------------------------------------- //
-guApeTagInfo::guApeTagInfo( const wxString &filename ) : guTagInfo( filename )
+guApeTagInfo::guApeTagInfo( const wxString &filename ) : guTagInfo(), m_ApeFile( filename )
 {
     //wxASSERT( !m_FileName.Lower().EndsWith( wxT( ".ape" ) ) );
 }
@@ -1235,28 +1414,46 @@ guApeTagInfo::~guApeTagInfo()
 // -------------------------------------------------------------------------------- //
 bool guApeTagInfo::Read( void )
 {
-    guApeFile File( m_FileName );
-    guApeTag * Tag = File.GetApeTag();
+    guApeTag * Tag = m_ApeFile.GetApeTag();
     if( Tag )
     {
         m_TrackName = Tag->GetTitle();
+        if( m_TrackName.IsEmpty() )
+            m_TrackName = m_FileName.AfterLast( wxT( '/' ) );
         m_ArtistName = Tag->GetArtist();
+        if( m_ArtistName.IsEmpty() )
+            m_ArtistName = _( "Unknown" );
         m_AlbumName = Tag->GetAlbum();
+        if( m_AlbumName.IsEmpty() )
+            m_AlbumName = m_FileName.BeforeLast( wxT( '/' ) ).AfterLast( wxT( '/' ) );
         m_GenreName = Tag->GetGenre();
+        if( m_GenreName.IsEmpty() )
+            m_GenreName = _( "Unknown" );
         m_Track = Tag->GetTrack();
         m_Year = Tag->GetYear();
-        m_Length = File.GetTrackLength();
-        m_Bitrate = File.GetBitRate();
+        m_Length = m_ApeFile.GetTrackLength();
+        m_Bitrate = m_ApeFile.GetBitRate();
+
+        m_Comments = Tag->GetItemValue( APE_TAG_KEY_COMMENT );
+        m_Composer = Tag->GetItemValue( APE_TAG_KEY_COMPOSER );
+        m_Disk = Tag->GetItemValue( APE_TAG_KEY_MEDIA );
+        m_AlbumArtist = Tag->GetItemValue( APE_TAG_KEY_ALBUMARTIST );
+        if( m_AlbumArtist.IsEmpty() )
+            m_AlbumArtist = Tag->GetItemValue( wxT( "AlbumArtist" ) );
+
         return true;
     }
+    else
+    {
+        guLogError( wxT( "Ape file with no tags found" ) );
+    }
     return false;
 }
 
 // -------------------------------------------------------------------------------- //
 bool guApeTagInfo::Write( void )
 {
-    guApeFile File( m_FileName );
-    guApeTag * Tag = File.GetApeTag();
+    guApeTag * Tag = m_ApeFile.GetApeTag();
     if( Tag )
     {
         Tag->SetTitle( m_TrackName );
@@ -1265,12 +1462,43 @@ bool guApeTagInfo::Write( void )
         Tag->SetGenre( m_GenreName );
         Tag->SetTrack( m_Track );
         Tag->SetYear( m_Year );
-        File.WriteApeTag();
+        Tag->SetItem( APE_TAG_KEY_COMMENT, m_Comments );
+        Tag->SetItem( APE_TAG_KEY_COMPOSER, m_Composer );
+        Tag->SetItem( APE_TAG_KEY_MEDIA, m_Disk );
+        Tag->SetItem( APE_TAG_KEY_ALBUMARTIST, m_AlbumArtist );
+
+        m_ApeFile.WriteApeTag();
         return true;
     }
     return false;
 }
 
+// -------------------------------------------------------------------------------- //
+bool guApeTagInfo::CanHandleLyrics( void )
+{
+    return true;
+}
+
+// -------------------------------------------------------------------------------- //
+wxString guApeTagInfo::GetLyrics( void )
+{
+    guApeTag * Tag = m_ApeFile.GetApeTag();
+    if( Tag )
+        return Tag->GetItemValue( APE_TAG_KEY_LYRICS );
+    return wxEmptyString;
+}
+
+// -------------------------------------------------------------------------------- //
+bool guApeTagInfo::SetLyrics( const wxString &lyrics )
+{
+    guApeTag * Tag = m_ApeFile.GetApeTag();
+    if( Tag )
+    {
+        Tag->SetItem( APE_TAG_KEY_LYRICS, lyrics );
+        return m_ApeFile.WriteApeTag();
+    }
+    return false;
+}
 
 
 
