@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -34,6 +34,24 @@
 
 const wxEventType guTrackEditEvent = wxNewEventType();
 
+// -------------------------------------------------------------------------------- //
+void guImagePtrArrayClean( guImagePtrArray * images )
+{
+    int Index;
+    int Count;
+    if( images && ( Count = images->Count() ) )
+    {
+        for( Index = 0; Index < Count; Index++ )
+        {
+            if( ( * images )[ Index ] )
+            {
+                delete ( * images )[ Index ];
+                ( * images )[ Index ] = NULL;
+            }
+        }
+    }
+}
+
 // -------------------------------------------------------------------------------- //
 guTrackEditor::guTrackEditor( wxWindow * parent, guDbLibrary * db, guTrackArray * songs,
     guImagePtrArray * images, wxArrayString * lyrics )
@@ -138,6 +156,18 @@ guTrackEditor::guTrackEditor( wxWindow * parent, guDbLibrary * db, guTrackArray
 	m_ArtistTextCtrl = new wxTextCtrl( DetailPanel, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
 	DataFlexSizer->Add( m_ArtistTextCtrl, 1, wxALIGN_CENTER_VERTICAL|wxEXPAND|wxTOP|wxRIGHT, 5 );
 
+	m_AACopyButton = new wxBitmapButton( DetailPanel, wxID_ANY, guImage( guIMAGE_INDEX_tiny_edit_copy ), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW );
+	m_AACopyButton->SetToolTip( _( "Copy the Album Artist name to all the tracks you are editing" ) );
+	DataFlexSizer->Add( m_AACopyButton, 0, wxALIGN_CENTER_VERTICAL|wxTOP|wxRIGHT|wxLEFT, 5 );
+
+	wxStaticText * AAStaticText = new wxStaticText( DetailPanel, wxID_ANY, _( "A. Artist:" ), wxDefaultPosition, wxDefaultSize, 0 );
+	AAStaticText->SetToolTip( _( "shows the Album Artist of the track" ) );
+	AAStaticText->Wrap( -1 );
+	DataFlexSizer->Add( AAStaticText, 0, wxALIGN_CENTER_VERTICAL|wxALIGN_RIGHT|wxTOP|wxRIGHT, 5 );
+
+	m_AlbumArtistTextCtrl = new wxTextCtrl( DetailPanel, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
+	DataFlexSizer->Add( m_AlbumArtistTextCtrl, 1, wxALIGN_CENTER_VERTICAL|wxEXPAND|wxTOP|wxRIGHT, 5 );
+
 	m_AlCopyButton = new wxBitmapButton( DetailPanel, wxID_ANY, guImage( guIMAGE_INDEX_tiny_edit_copy ), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW );
 	m_AlCopyButton->SetToolTip( _( "Copy the Album name to all the tracks you are editing" ) );
 	DataFlexSizer->Add( m_AlCopyButton, 0, wxALIGN_CENTER_VERTICAL|wxTOP|wxRIGHT|wxLEFT, 5 );
@@ -260,7 +290,7 @@ guTrackEditor::guTrackEditor( wxWindow * parent, guDbLibrary * db, guTrackArray
 
 	MainDetailSizer->Add( DataFlexSizer, 0, wxEXPAND, 5 );
 
-	m_DetailInfoStaticText = new wxStaticText( DetailPanel, wxID_ANY, wxT("Type\t: mp3\nBitRate\t: 160 kbps\nLength\t: 04:09\nFileSize\t: 4.9 Mbytes"), wxDefaultPosition, wxDefaultSize, 0 );
+	m_DetailInfoStaticText = new wxStaticText( DetailPanel, wxID_ANY, wxT("Type\t:\nBitRate\t:\nLength\t:\nFileSize\t:"), wxDefaultPosition, wxDefaultSize, 0 );
 	m_DetailInfoStaticText->Wrap( -1 );
 	MainDetailSizer->Add( m_DetailInfoStaticText, 1, wxALL|wxEXPAND, 5 );
 
@@ -286,7 +316,7 @@ guTrackEditor::guTrackEditor( wxWindow * parent, guDbLibrary * db, guTrackArray
 	CommentPanel->SetSizer( CommentSizer );
 	CommentPanel->Layout();
 	CommentSizer->Fit( CommentPanel );
-	MainNoteBook->AddPage( CommentPanel, wxT("Comments"), false );
+	MainNoteBook->AddPage( CommentPanel, _("Comments"), false );
 
 	//
 	// Pictures
@@ -359,6 +389,9 @@ guTrackEditor::guTrackEditor( wxWindow * parent, guDbLibrary * db, guTrackArray
 	    wxT( "http://leoslyrics.com" ),
 	    wxT( "http://lyrc.com.ar" ),
 	    wxT( "http://cduniverse.com" ),
+//	    wxT( "http://lyricsfly.com" ),
+        wxT( "http://chartlyrics.com" ),
+        wxT( "http://lyricsplugin.com" ),
 	    wxT( "http://ultimate-guitar.com" )
 	    };
 	int LyricsChoiceNChoices = sizeof( LyricsChoiceChoices ) / sizeof( wxString );
@@ -411,7 +444,7 @@ guTrackEditor::guTrackEditor( wxWindow * parent, guDbLibrary * db, guTrackArray
 
     wxStaticText * MBQueryArtistStaticText;
 	wxStaticText * MBQueryTitleStaticText;
-	MBQueryArtistStaticText = new wxStaticText( MBrainzPanel, wxID_ANY, wxT("Artist:"), wxDefaultPosition, wxDefaultSize, 0 );
+	MBQueryArtistStaticText = new wxStaticText( MBrainzPanel, wxID_ANY, _("Artist:"), wxDefaultPosition, wxDefaultSize, 0 );
 	MBQueryArtistStaticText->Wrap( -1 );
 	MBQuerySizer->Add( MBQueryArtistStaticText, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
 
@@ -419,7 +452,7 @@ guTrackEditor::guTrackEditor( wxWindow * parent, guDbLibrary * db, guTrackArray
 	m_MBQueryArtistTextCtrl->SetToolTip( _( "Type the artist name to search in musicbrainz" ) );
 	MBQuerySizer->Add( m_MBQueryArtistTextCtrl, 1, wxALIGN_CENTER_VERTICAL|wxTOP|wxBOTTOM|wxRIGHT, 5 );
 
-	MBQueryTitleStaticText = new wxStaticText( MBrainzPanel, wxID_ANY, wxT("Title:"), wxDefaultPosition, wxDefaultSize, 0 );
+	MBQueryTitleStaticText = new wxStaticText( MBrainzPanel, wxID_ANY, _("Title:"), wxDefaultPosition, wxDefaultSize, 0 );
 	MBQueryTitleStaticText->Wrap( -1 );
 	MBQuerySizer->Add( MBQueryTitleStaticText, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
 
@@ -516,7 +549,7 @@ guTrackEditor::guTrackEditor( wxWindow * parent, guDbLibrary * db, guTrackArray
 	m_MBrainzTiCopyButton->SetToolTip( _( "Copy the song names to the edited tracks" ) );
 	MBDetailFlexGridSizer->Add( m_MBrainzTiCopyButton, 0, wxBOTTOM|wxRIGHT|wxALIGN_CENTER_VERTICAL, 5 );
 
-	m_MBrainzLengthStaticText = new wxStaticText( MBrainzPanel, wxID_ANY, wxT("Length:"), wxDefaultPosition, wxDefaultSize, 0 );
+	m_MBrainzLengthStaticText = new wxStaticText( MBrainzPanel, wxID_ANY, _("Length:"), wxDefaultPosition, wxDefaultSize, 0 );
 	m_MBrainzLengthStaticText->Wrap( -1 );
 	MBDetailFlexGridSizer->Add( m_MBrainzLengthStaticText, 0, wxBOTTOM|wxRIGHT|wxLEFT|wxALIGN_CENTER_VERTICAL, 5 );
 
@@ -528,7 +561,7 @@ guTrackEditor::guTrackEditor( wxWindow * parent, guDbLibrary * db, guTrackArray
 
 	MBNumberSizer->Add( 0, 0, 1, wxEXPAND, 5 );
 
-	m_MBrainzNumberStaticText = new wxStaticText( MBrainzPanel, wxID_ANY, wxT("Number:"), wxDefaultPosition, wxDefaultSize, 0 );
+	m_MBrainzNumberStaticText = new wxStaticText( MBrainzPanel, wxID_ANY, _("Number:"), wxDefaultPosition, wxDefaultSize, 0 );
 	m_MBrainzNumberStaticText->Wrap( -1 );
 	MBNumberSizer->Add( m_MBrainzNumberStaticText, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxBOTTOM|wxRIGHT|wxLEFT, 5 );
 
@@ -616,6 +649,7 @@ guTrackEditor::guTrackEditor( wxWindow * parent, guDbLibrary * db, guTrackArray
 	m_MoveUpButton->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnMoveUpBtnClick ), NULL, this );
 	m_MoveDownButton->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnMoveDownBtnClick ), NULL, this );
 	m_ArCopyButton->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnArCopyButtonClicked ), NULL, this );
+	m_AACopyButton->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnAACopyButtonClicked ), NULL, this );
 	m_AlCopyButton->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnAlCopyButtonClicked ), NULL, this );
 	m_TiCopyButton->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnTiCopyButtonClicked ), NULL, this );
 	m_CoCopyButton->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnCoCopyButtonClicked ), NULL, this );
@@ -689,6 +723,7 @@ guTrackEditor::~guTrackEditor()
 	m_MoveUpButton->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnMoveUpBtnClick ), NULL, this );
 	m_MoveDownButton->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnMoveDownBtnClick ), NULL, this );
 	m_ArCopyButton->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnArCopyButtonClicked ), NULL, this );
+	m_AACopyButton->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnAACopyButtonClicked ), NULL, this );
 	m_AlCopyButton->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnAlCopyButtonClicked ), NULL, this );
 	m_TiCopyButton->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnTiCopyButtonClicked ), NULL, this );
 	m_CoCopyButton->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guTrackEditor::OnCoCopyButtonClicked ), NULL, this );
@@ -739,7 +774,6 @@ guTrackEditor::~guTrackEditor()
         delete m_MBrainzReleases;
     if( m_MBrainzAlbums )
         delete m_MBrainzAlbums;
-
 }
 
 // -------------------------------------------------------------------------------- //
@@ -777,6 +811,8 @@ void guTrackEditor::OnMoveUpBtnClick( wxCommandEvent &event )
 
     event.SetInt( m_CurItem );
     OnSongListBoxSelected( event );
+
+    MBCheckCountAndLengths();
 }
 
 // -------------------------------------------------------------------------------- //
@@ -787,6 +823,7 @@ void guTrackEditor::OnMoveDownBtnClick( wxCommandEvent &event )
     wxImage * MovedImage = ( * m_Images )[ m_CurItem ];
     wxString MovedLyric = ( * m_Lyrics )[ m_CurItem ];
     m_Images->RemoveAt( m_CurItem );
+    m_Lyrics->RemoveAt( m_CurItem );
     m_SongListBox->SetString( m_CurItem, m_SongListBox->GetString( m_CurItem + 1 ) );
     m_CurItem++;
     m_Items->Insert( MovedTrack, m_CurItem );
@@ -798,6 +835,8 @@ void guTrackEditor::OnMoveDownBtnClick( wxCommandEvent &event )
 
     event.SetInt( m_CurItem );
     OnSongListBoxSelected( event );
+
+    MBCheckCountAndLengths();
 }
 
 // -------------------------------------------------------------------------------- //
@@ -807,17 +846,24 @@ void guTrackEditor::ReadItemData( void )
     if( m_CurItem >= 0 )
     {
         guTrack * Track = &( * m_Items )[ m_CurItem ];
+        m_AlbumArtistTextCtrl->SetValue( Track->m_AlbumArtist );
         m_ArtistTextCtrl->SetValue( Track->m_ArtistName );
         m_AlbumTextCtrl->SetValue( Track->m_AlbumName );
         m_TitleTextCtrl->SetValue( Track->m_SongName );
         m_CompTextCtrl->SetValue( Track->m_Composer );
-        m_NumberTextCtrl->SetValue( wxString::Format( wxT( "%u" ), Track->m_Number ) );
+        if( Track->m_Number )
+            m_NumberTextCtrl->SetValue( wxString::Format( wxT( "%u" ), Track->m_Number ) );
+        else
+            m_NumberTextCtrl->SetValue( wxEmptyString );
         m_DiskTextCtrl->SetValue( Track->m_Disk );
         m_GenreComboBox->SetValue( Track->m_GenreName );
-        m_YearTextCtrl->SetValue( wxString::Format( wxT( "%u" ), Track->m_Year ) );
+        if( Track->m_Year )
+            m_YearTextCtrl->SetValue( wxString::Format( wxT( "%u" ), Track->m_Year ) );
+        else
+            m_YearTextCtrl->SetValue( wxEmptyString );
         m_Rating->SetRating( Track->m_Rating );
         m_CommentText->SetValue( Track->m_Comments );
-        m_DetailInfoStaticText->SetLabel( wxString::Format( wxT( "File Type\t: %s\n"
+        m_DetailInfoStaticText->SetLabel( wxString::Format( _( "File Type\t: %s\n"
                                                "BitRate\t: %u Kbps\n"
                                                "Length\t: %s\n"
                                                "File Size\t: %s" ),
@@ -840,6 +886,7 @@ void guTrackEditor::ReadItemData( void )
     }
     else
     {
+        m_AlbumArtistTextCtrl->SetValue( wxEmptyString );
         m_ArtistTextCtrl->SetValue( wxEmptyString );
         m_AlbumTextCtrl->SetValue( wxEmptyString );
         m_TitleTextCtrl->SetValue( wxEmptyString );
@@ -871,6 +918,8 @@ void guTrackEditor::WriteItemData( void )
     //guLogMessage( wxT( "WriteItemData: %i" ), m_CurItem );
     if( m_CurItem >= 0 )
     {
+        if( m_AlbumArtistTextCtrl->IsModified() )
+          ( * m_Items )[ m_CurItem ].m_AlbumArtist = m_AlbumArtistTextCtrl->GetLineText( 0 );
         if( m_ArtistTextCtrl->IsModified() )
           ( * m_Items )[ m_CurItem ].m_ArtistName = m_ArtistTextCtrl->GetLineText( 0 );
         if( m_AlbumTextCtrl->IsModified() )
@@ -897,6 +946,16 @@ void guTrackEditor::WriteItemData( void )
 }
 
 
+// -------------------------------------------------------------------------------- //
+void guTrackEditor::OnAACopyButtonClicked( wxCommandEvent& event )
+{
+    int index;
+    wxString CurData = m_AlbumArtistTextCtrl->GetLineText( 0 );
+    int count = m_Items->Count();
+    for( index = 0; index < count; index++ )
+        ( * m_Items )[ index ].m_AlbumArtist = CurData;
+}
+
 // -------------------------------------------------------------------------------- //
 void guTrackEditor::OnArCopyButtonClicked( wxCommandEvent& event )
 {
@@ -1100,10 +1159,14 @@ void guTrackEditor::OnSaveImageClicked( wxCommandEvent &event )
     wxImage * pCurImage = ( * m_Images )[ m_CurItem ];
     wxASSERT( pCurImage );
 
+    guConfig * Config = ( guConfig * ) guConfig::Get();
+    wxArrayString SearchCovers = Config->ReadAStr( wxT( "Word" ), wxEmptyString, wxT( "CoverSearch" ) );
+    wxString CoverName = ( SearchCovers.Count() ? SearchCovers[ 0 ] : wxT( "cover" ) ) + wxT( ".jpg" );
+
     wxFileDialog * FileDialog = new wxFileDialog( this,
         wxT( "Select the filename to save" ),
         wxPathOnly( ( * m_Items )[ m_CurItem ].m_FileName ),
-        wxT( "cover.jpg" ),
+        CoverName,
         wxT( "*.jpg;*.png" ),
         wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
 
@@ -1142,8 +1205,12 @@ void guTrackEditor::OnSearchImageClicked( wxCommandEvent &event )
     {
         if( CoverEditor->ShowModal() == wxID_OK )
         {
-            ( * m_Images )[ m_CurItem ] = new wxImage( * CoverEditor->GetSelectedCoverImage() );
-            RefreshImage();
+            wxImage * SelectedCover = CoverEditor->GetSelectedCoverImage();
+            if( SelectedCover )
+            {
+                ( * m_Images )[ m_CurItem ] = new wxImage( * SelectedCover );
+                RefreshImage();
+            }
         }
         CoverEditor->Destroy();
     }
@@ -1237,18 +1304,25 @@ void guTrackEditor::OnMBrainzAddButtonClicked( wxCommandEvent &event )
     }
     else if( m_MBrainzCurTrack < ( int ) m_Items->Count() )
     {
-        wxSetCursor( * wxHOURGLASS_CURSOR );
-        m_MBrainzAddButton->Enable( false );
-        m_MBrainzThreadMutex.Lock();
-        if( m_MBrainzThread )
+        if( wxFileExists( ( * m_Items )[ m_MBrainzCurTrack ].m_FileName ) )
         {
-            m_MBrainzThread->Pause();
-            m_MBrainzThread->Delete();
-        }
-        m_MBrainzThread = new guMusicBrainzMetadataThread( this, m_MBrainzCurTrack );
-        m_MBrainzThreadMutex.Unlock();
-        m_MBrainzCurTrack++;
+            wxSetCursor( * wxHOURGLASS_CURSOR );
+            m_MBrainzAddButton->Enable( false );
+            m_MBrainzThreadMutex.Lock();
+            if( m_MBrainzThread )
+            {
+                m_MBrainzThread->Pause();
+                m_MBrainzThread->Delete();
+            }
+            m_MBrainzThread = new guMusicBrainzMetadataThread( this, m_MBrainzCurTrack );
+            m_MBrainzThreadMutex.Unlock();
         //guLogMessage( wxT( "Albums search thread created" ) );
+        }
+        else
+        {
+            guLogError( wxT( "Could not find the track '%s'" ), ( * m_Items )[ m_CurItem ].m_FileName.c_str() );
+        }
+        m_MBrainzCurTrack++;
     }
 }
 
@@ -1343,6 +1417,7 @@ void guTrackEditor::UpdateMBrainzTrackInfo( void )
             {
                 m_MBrainzArtistStaticText->SetForegroundColour( Track->m_ArtistName == m_MBrainzReleases->Item( m_MBrainzCurAlbum ).m_ArtistName ?
                                         m_NormalColor : m_ErrorColor );
+                m_MBrainzArtistTextCtrl->SetValue( m_MBrainzReleases->Item( m_MBrainzCurAlbum ).m_ArtistName );
             }
 
             // ALbum
@@ -1400,6 +1475,28 @@ int guTrackEditor::CheckTracksLengths( guMBTrackArray * mbtracks, guTrackArray *
     return RetVal;
 }
 
+// -------------------------------------------------------------------------------- //
+void guTrackEditor::MBCheckCountAndLengths( void )
+{
+    if( m_MBrainzReleases && m_MBrainzReleases->Count() )
+    {
+        guMBRelease * MBRelease = &m_MBrainzReleases->Item( m_MBrainzCurAlbum );
+        // Check the number of tracks
+        wxString InfoText;
+        if( MBRelease->m_Tracks.Count() != m_Items->Count() )
+        {
+            InfoText = wxString::Format( _( "Error: The album have %u tracks and you are editing %u" ),
+                            MBRelease->m_Tracks.Count(), m_Items->Count() );
+        }
+        if( CheckTracksLengths( &MBRelease->m_Tracks, m_Items ) )
+        {
+            InfoText += _( "\n"
+                             "Warning: The length of some edited tracks don't match" );
+        }
+        m_MBrainzInfoStaticText->SetLabel( InfoText );
+    }
+}
+
 // -------------------------------------------------------------------------------- //
 void guTrackEditor::OnMBrainzAlbumChoiceSelected( wxCommandEvent &event )
 {
@@ -1416,19 +1513,7 @@ void guTrackEditor::OnMBrainzAlbumChoiceSelected( wxCommandEvent &event )
         m_MBrainzCopyButton->Enable( true );
         m_MBrainzAlbumTextCtrl->SetValue( MBRelease->m_Title );
 
-        // Check the number of tracks
-        wxString InfoText;
-        if( MBRelease->m_Tracks.Count() != m_Items->Count() )
-        {
-            InfoText = wxString::Format( _( "Error: The album have %u tracks and you are editing %u" ),
-                            MBRelease->m_Tracks.Count(), m_Items->Count() );
-        }
-        if( CheckTracksLengths( &MBRelease->m_Tracks, m_Items ) )
-        {
-            InfoText += _( "\n"
-                             "Warning: The length of some edited tracks don't match" );
-        }
-        m_MBrainzInfoStaticText->SetLabel( InfoText );
+        MBCheckCountAndLengths();
     }
     else
     {
@@ -1614,36 +1699,9 @@ void guTrackEditor::OnSearchLyrics( wxCommandEvent &event )
 
     int Engine = m_LyricSrvChoice->GetSelection();
 
-    if( Engine == guLYRIC_ENGINE_LYRICWIKI )
-    {
-        m_LyricThread = new guLyricWikiEngine( this, &m_LyricThread,
-                                m_LyricArtistTextCtrl->GetValue().c_str(),
-                                m_LyricTrackTextCtrl->GetValue().c_str() );
-    }
-    else if( Engine == guLYRIC_ENGINE_LEOSLYRICS )
-    {
-        m_LyricThread = new guLeosLyricsEngine( this, &m_LyricThread,
-                                m_LyricArtistTextCtrl->GetValue().c_str(),
-                                m_LyricTrackTextCtrl->GetValue().c_str() );
-    }
-    else if( Engine == guLYRIC_ENGINE_LYRC_COM_AR )
-    {
-        m_LyricThread = new guLyrcComArEngine( this, &m_LyricThread,
-                                m_LyricArtistTextCtrl->GetValue().c_str(),
-                                m_LyricTrackTextCtrl->GetValue().c_str() );
-    }
-    else if( Engine == guLYRIC_ENGINE_CDUNIVERSE )
-    {
-        m_LyricThread = new guCDUEngine( this, &m_LyricThread,
-                                m_LyricArtistTextCtrl->GetValue().c_str(),
-                                m_LyricTrackTextCtrl->GetValue().c_str() );
-    }
-    else //if( Engine == guLYRIC_ENGINE_ULTGUITAR )
-    {
-        m_LyricThread = new guUltGuitarEngine( this, &m_LyricThread,
-                                m_LyricArtistTextCtrl->GetValue().c_str(),
-                                m_LyricTrackTextCtrl->GetValue().c_str() );
-    }
+    m_LyricThread = guGetSearchLyricEngine( Engine, ( wxEvtHandler * ) this, &m_LyricThread,
+        m_LyricArtistTextCtrl->GetValue().c_str(), m_LyricTrackTextCtrl->GetValue().c_str() );
+
     m_LyricReloadButton->Enable( false );
 }
 
@@ -1653,8 +1711,11 @@ void guTrackEditor::OnDownloadedLyric( wxCommandEvent &event )
     wxString * Content = ( wxString * ) event.GetClientData();
     if( Content )
     {
-        m_LyricsTextCtrl->SetValue( * Content );
-        ( * m_Lyrics )[ m_CurItem ] = * Content;
+        wxHtmlEntitiesParser EntitiesParser;
+        wxString NewLyric = EntitiesParser.Parse( * Content );
+        NewLyric.Trim().Trim( false );
+        m_LyricsTextCtrl->SetValue( NewLyric );
+        ( * m_Lyrics )[ m_CurItem ] = NewLyric;
         delete Content;
     }
     m_LyricReloadButton->Enable( true );
