@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -39,13 +39,16 @@ DBusHandlerResult Handle_Messages( DBusConnection * conn, DBusMessage * msg, voi
 // -------------------------------------------------------------------------------- //
 void Handle_Response( DBusPendingCall * PCall, void * udata )
 {
-	guDBusServer * DBusObj = ( guDBusServer * ) udata;
+	guDBusClient * DBusObj = ( guDBusClient * ) udata;
 	DBusMessage * reply;
 	reply = dbus_pending_call_steal_reply( PCall );
     guDBusMessage * Msg = new guDBusMessage( reply );
 
     DBusObj->HandleMessages( Msg, NULL );
 
+    delete Msg;
+
+    //delete Msg;
 	dbus_message_unref( reply );
 	dbus_pending_call_unref( PCall );
 }
@@ -147,11 +150,10 @@ DBusHandlerResult guDBusServer::HandleMessages( guDBusMessage * msg, guDBusMessa
         m_ClientsMutex.Unlock();
     }
 
-    // release the dbus objects
-    dbus_message_unref( msg->GetMessage() );
+    delete msg;
 
     if( reply )
-        dbus_message_unref( reply->GetMessage() );
+        delete reply;
 
     return RetVal;
 }
@@ -181,7 +183,9 @@ bool guDBusServer::AddMatch( const char * rule )
 
     if( dbus_error_is_set( &error ) )
     {
-        printf( "Could not add the match %s : %s\n", rule, error.message );
+        guLogError( wxT( "Could not add the match %s : %s" ),
+            wxString::FromAscii( rule ).c_str(),
+            wxString::FromAscii( error.message ).c_str() );
         dbus_error_free( &error );
         return false;
     }
@@ -191,15 +195,15 @@ bool guDBusServer::AddMatch( const char * rule )
 // -------------------------------------------------------------------------------- //
 bool guDBusServer::Send( guDBusMessage * msg )
 {
-    return dbus_connection_send( m_DBusConn, msg->GetMessage(), NULL );
+    return msg && dbus_connection_send( m_DBusConn, msg->GetMessage(), NULL );
 }
 
 // -------------------------------------------------------------------------------- //
-bool guDBusServer::SendWithReply( guDBusMessage * msg, int timeout )
+bool guDBusServer::SendWithReply( guDBusMessage * msg, guDBusClient * client, int timeout )
 {
     DBusPendingCall * PCall = NULL;
     bool RetVal = dbus_connection_send_with_reply( m_DBusConn, msg->GetMessage(), &PCall, timeout );
-	dbus_pending_call_set_notify( PCall, Handle_Response, (void *) this, NULL );
+	dbus_pending_call_set_notify( PCall, Handle_Response, (void *) client, NULL );
     return RetVal;
 }
 
@@ -208,9 +212,13 @@ guDBusMessage * guDBusServer::SendWithReplyAndBlock( guDBusMessage * msg, int ti
 {
     DBusError error;
     dbus_error_init( &error );
+    guDBusMessage * Reply;
 
     DBusMessage * Result = dbus_connection_send_with_reply_and_block( m_DBusConn, msg->GetMessage(), timeout, &error );
-    return Result ? new guDBusMessage( Result ) : NULL;
+    Reply =  Result ? new guDBusMessage( Result ) : NULL;
+    if( Result )
+        dbus_message_unref( Result );
+    return Reply;
 }
 
 // -------------------------------------------------------------------------------- //
@@ -321,7 +329,7 @@ bool guDBusClient::Send( guDBusMessage * msg )
 // -------------------------------------------------------------------------------- //
 bool guDBusClient::SendWithReply( guDBusMessage * msg, int timeout )
 {
-    return m_DBusServer->SendWithReply( msg, timeout );
+    return m_DBusServer->SendWithReply( msg, this, timeout );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -373,115 +381,121 @@ guDBusMessage::~guDBusMessage()
 }
 
 // -------------------------------------------------------------------------------- //
- DBusMessage * guDBusMessage::GetMessage()
+DBusMessage * guDBusMessage::GetMessage()
 {
     return m_DBusMsg;
 }
 
 // -------------------------------------------------------------------------------- //
- const char * guDBusMessage::GetErrorName()
+const char * guDBusMessage::GetErrorName()
 {
     return dbus_message_get_error_name( m_DBusMsg );
 }
 
 // -------------------------------------------------------------------------------- //
- const char * guDBusMessage::GetInterface()
+const char * guDBusMessage::GetInterface()
 {
     return dbus_message_get_interface( m_DBusMsg );
 }
 
 // -------------------------------------------------------------------------------- //
- const char * guDBusMessage::GetMember()
+const char * guDBusMessage::GetMember()
 {
     return dbus_message_get_member( m_DBusMsg );
 }
 
 // -------------------------------------------------------------------------------- //
- bool guDBusMessage::NeedReply()
+bool guDBusMessage::NeedReply()
 {
     return !dbus_message_get_no_reply( m_DBusMsg );
 }
 
 // -------------------------------------------------------------------------------- //
- const char * guDBusMessage::GetPath()
+void guDBusMessage::SetNoReply( const bool needreply )
+{
+    dbus_message_set_no_reply( m_DBusMsg, needreply );
+}
+
+// -------------------------------------------------------------------------------- //
+const char * guDBusMessage::GetPath()
 {
     return dbus_message_get_path( m_DBusMsg );
 }
 
 // -------------------------------------------------------------------------------- //
- unsigned int guDBusMessage::GetReplySerial()
+unsigned int guDBusMessage::GetReplySerial()
 {
     return dbus_message_get_reply_serial( m_DBusMsg );
 }
 
 // -------------------------------------------------------------------------------- //
- const char * guDBusMessage::GetSender()
+const char * guDBusMessage::GetSender()
 {
     return dbus_message_get_sender( m_DBusMsg );
 }
 
 // -------------------------------------------------------------------------------- //
- const char * guDBusMessage::GetDestination()
+const char * guDBusMessage::GetDestination()
 {
     return dbus_message_get_destination( m_DBusMsg );
 }
 
 // -------------------------------------------------------------------------------- //
- unsigned int guDBusMessage::GetSerial()
+unsigned int guDBusMessage::GetSerial()
 {
     return dbus_message_get_serial( m_DBusMsg );
 }
 
 // -------------------------------------------------------------------------------- //
- int guDBusMessage::GetType()
+int guDBusMessage::GetType()
 {
     return dbus_message_get_type( m_DBusMsg );
 }
 
 // -------------------------------------------------------------------------------- //
- bool guDBusMessage::HasDestination( const char * dest )
+bool guDBusMessage::HasDestination( const char * dest )
 {
     return dbus_message_has_destination( m_DBusMsg, dest );
 }
 
 // -------------------------------------------------------------------------------- //
- bool guDBusMessage::HasInterface( const char * iface )
+bool guDBusMessage::HasInterface( const char * iface )
 {
     return dbus_message_has_interface( m_DBusMsg, iface );
 }
 
 // -------------------------------------------------------------------------------- //
- bool guDBusMessage::HasMember( const char * member )
+bool guDBusMessage::HasMember( const char * member )
 {
     return dbus_message_has_member( m_DBusMsg, member );
 }
 
 // -------------------------------------------------------------------------------- //
- bool guDBusMessage::HasPath( const char * path )
+bool guDBusMessage::HasPath( const char * path )
 {
     return dbus_message_has_path( m_DBusMsg, path );
 }
 
 // -------------------------------------------------------------------------------- //
- bool guDBusMessage::HasSender( const char * sender )
+bool guDBusMessage::HasSender( const char * sender )
 {
     return dbus_message_has_sender( m_DBusMsg, sender );
 }
 
 // -------------------------------------------------------------------------------- //
- bool guDBusMessage::IsError( const char * errname )
+bool guDBusMessage::IsError( const char * errname )
 {
     return dbus_message_is_error( m_DBusMsg, errname );
 }
 
 // -------------------------------------------------------------------------------- //
- bool guDBusMessage::IsMethodCall( const char * iface, const char * method )
+bool guDBusMessage::IsMethodCall( const char * iface, const char * method )
 {
     return dbus_message_is_method_call( m_DBusMsg, iface, method );
 }
 
 // -------------------------------------------------------------------------------- //
- bool guDBusMessage::IsSignal( const char * iface, const char * signal_name )
+bool guDBusMessage::IsSignal( const char * iface, const char * signal_name )
 {
     return dbus_message_is_signal( m_DBusMsg, iface, signal_name );
 }
@@ -506,15 +520,15 @@ guDBusMethodCall::guDBusMethodCall( const char * dest, const char * path, const
 // -------------------------------------------------------------------------------- //
 // guDBusMethodReturn
 // -------------------------------------------------------------------------------- //
-guDBusMethodReturn::guDBusMethodReturn( guDBusMessage * msg )  :
-  guDBusMessage( dbus_message_new_method_return( msg->GetMessage() ) )
+guDBusMethodReturn::guDBusMethodReturn( guDBusMessage * msg )
 {
+    m_DBusMsg = dbus_message_new_method_return( msg->GetMessage() );
 }
 
 // -------------------------------------------------------------------------------- //
-guDBusMethodReturn::guDBusMethodReturn( DBusMessage * msg )  :
-  guDBusMessage( dbus_message_new_method_return( msg ) )
+guDBusMethodReturn::guDBusMethodReturn( DBusMessage * msg )
 {
+    m_DBusMsg = dbus_message_new_method_return( msg );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -523,6 +537,12 @@ guDBusSignal::guDBusSignal( const char * path, const char * iface, const char *
     m_DBusMsg = dbus_message_new_signal( path, iface, name );
 }
 
+// -------------------------------------------------------------------------------- //
+guDBusSignal::~guDBusSignal()
+{
+    //m_DBusMsg = NULL;
+}
+
 // -------------------------------------------------------------------------------- //
 // guDBusThread
 // -------------------------------------------------------------------------------- //
