@@ -1,5 +1,5 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2010 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
@@ -204,22 +204,22 @@ void FillMetadataArgs( guDBusMessage * reply, const guTrack * CurTrack )
 
     const char * metadata_names[] = {
         "location", "title", "artist", "album", "tracknumber",
-        "time", "mtime", "genre", "rating", "year", "arturl"
+        "time", "mtime", "genre", "rating", "year", "arturl", "bitrate"
     };
 
     dbus_message_iter_init_append( reply->GetMessage(), &args );
 
     dbus_message_iter_open_container( &args, DBUS_TYPE_ARRAY, "{sv}", &dict );
 
-    FillMetadataDetails( &dict, metadata_names[ 0 ], ( const char * ) wxString::Format( wxT( "file://%s" ), CurTrack->m_FileName.c_str() ).char_str() );
-    FillMetadataDetails( &dict, metadata_names[ 1 ], ( const char * ) CurTrack->m_SongName.char_str() );
-    FillMetadataDetails( &dict, metadata_names[ 2 ], ( const char * ) CurTrack->m_ArtistName.char_str() );
-    FillMetadataDetails( &dict, metadata_names[ 3 ], ( const char * ) CurTrack->m_AlbumName.char_str() );
+    FillMetadataDetails( &dict, metadata_names[ 0 ], ( const char * ) ( wxT( "file://" ) + CurTrack->m_FileName ).mb_str( wxConvUTF8 ) );
+    FillMetadataDetails( &dict, metadata_names[ 1 ], ( const char * ) CurTrack->m_SongName.mb_str( wxConvUTF8 ) );
+    FillMetadataDetails( &dict, metadata_names[ 2 ], ( const char * ) CurTrack->m_ArtistName.mb_str( wxConvUTF8 ) );
+    FillMetadataDetails( &dict, metadata_names[ 3 ], ( const char * ) CurTrack->m_AlbumName.mb_str( wxConvUTF8 ) );
     if( CurTrack->m_Number )
         FillMetadataDetails( &dict, metadata_names[ 4 ], ( const int ) CurTrack->m_Number );
     FillMetadataDetails( &dict, metadata_names[ 5 ], ( const int ) CurTrack->m_Length );
     FillMetadataDetails( &dict, metadata_names[ 6 ], ( const int ) CurTrack->m_Length * 1000 );
-    FillMetadataDetails( &dict, metadata_names[ 7 ], ( const char * ) CurTrack->m_GenreName.char_str() );
+    FillMetadataDetails( &dict, metadata_names[ 7 ], ( const char * ) CurTrack->m_GenreName.mb_str( wxConvUTF8 ) );
 
     if( CurTrack->m_Rating >= 0 )
         FillMetadataDetails( &dict, metadata_names[ 8 ], ( const int ) CurTrack->m_Rating );
@@ -228,7 +228,10 @@ void FillMetadataArgs( guDBusMessage * reply, const guTrack * CurTrack )
         FillMetadataDetails( &dict, metadata_names[ 9 ], ( const int ) CurTrack->m_Year );
 
 //    if( !CurTrack->m_CoverPath.IsEmpty() )
-//        FillMetadataDetails( &dict, metadata_names[ 10 ], ( const char * ) wxString::Format( wxT( "file://%s" ), CurTrack->m_CoverPath.c_str() ).char_str() );
+//        FillMetadataDetails( &dict, metadata_names[ 10 ], ( const char * ) ( wxT( "file://" ) + CurTrack->m_CoverPath ).mb_str( wxConvUTF8 ) );
+
+    if( CurTrack->m_Rating )
+        FillMetadataDetails( &dict, metadata_names[ 11 ], ( const int ) CurTrack->m_Bitrate );
 
     dbus_message_iter_close_container( &args, &dict );
 
@@ -244,22 +247,22 @@ void FillMetadataArgs( guDBusMessage * reply, const guCurrentTrack * CurTrack )
 
     const char * metadata_names[] = {
         "location", "title", "artist", "album", "tracknumber",
-        "time", "mtime", "genre", "rating", "year", "arturl"
+        "time", "mtime", "genre", "rating", "year", "arturl", "bitrate"
     };
 
     dbus_message_iter_init_append( reply->GetMessage(), &args );
 
     dbus_message_iter_open_container( &args, DBUS_TYPE_ARRAY, "{sv}", &dict );
 
-    FillMetadataDetails( &dict, metadata_names[ 0 ], ( const char * ) wxString::Format( wxT( "file://%s" ), CurTrack->m_FileName.c_str() ).char_str() );
-    FillMetadataDetails( &dict, metadata_names[ 1 ], ( const char * ) CurTrack->m_SongName.char_str() );
-    FillMetadataDetails( &dict, metadata_names[ 2 ], ( const char * ) CurTrack->m_ArtistName.char_str() );
-    FillMetadataDetails( &dict, metadata_names[ 3 ], ( const char * ) CurTrack->m_AlbumName.char_str() );
+    FillMetadataDetails( &dict, metadata_names[ 0 ], ( const char * ) ( wxT( "file://" ) + CurTrack->m_FileName ).mb_str( wxConvUTF8 ) );
+    FillMetadataDetails( &dict, metadata_names[ 1 ], ( const char * ) CurTrack->m_SongName.mb_str( wxConvUTF8 ) );
+    FillMetadataDetails( &dict, metadata_names[ 2 ], ( const char * ) CurTrack->m_ArtistName.mb_str( wxConvUTF8 ) );
+    FillMetadataDetails( &dict, metadata_names[ 3 ], ( const char * ) CurTrack->m_AlbumName.mb_str( wxConvUTF8 ) );
     if( CurTrack->m_Number )
         FillMetadataDetails( &dict, metadata_names[ 4 ], ( const int ) CurTrack->m_Number );
     FillMetadataDetails( &dict, metadata_names[ 5 ], ( const int ) CurTrack->m_Length );
     FillMetadataDetails( &dict, metadata_names[ 6 ], ( const int ) CurTrack->m_Length * 1000 );
-    FillMetadataDetails( &dict, metadata_names[ 7 ], ( const char * ) CurTrack->m_GenreName.char_str() );
+    FillMetadataDetails( &dict, metadata_names[ 7 ], ( const char * ) CurTrack->m_GenreName.mb_str( wxConvUTF8 ) );
     if( CurTrack->m_Rating >= 0 )
         FillMetadataDetails( &dict, metadata_names[ 8 ], ( const int ) CurTrack->m_Rating );
 
@@ -267,7 +270,10 @@ void FillMetadataArgs( guDBusMessage * reply, const guCurrentTrack * CurTrack )
         FillMetadataDetails( &dict, metadata_names[ 9 ], ( const int ) CurTrack->m_Year );
 
     if( !CurTrack->m_CoverPath.IsEmpty() )
-        FillMetadataDetails( &dict, metadata_names[ 10 ], ( const char * ) wxString::Format( wxT( "file://%s" ), CurTrack->m_CoverPath.c_str() ).char_str() );
+        FillMetadataDetails( &dict, metadata_names[ 10 ], ( const char * ) ( wxT( "file://" ) + CurTrack->m_CoverPath ).mb_str( wxConvUTF8 ) );
+
+    if( CurTrack->m_Rating )
+        FillMetadataDetails( &dict, metadata_names[ 11 ], ( const int ) CurTrack->m_Bitrate );
 
     dbus_message_iter_close_container( &args, &dict );
 
@@ -333,7 +339,7 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                 Flush();
                 RetVal = DBUS_HANDLER_RESULT_HANDLED;
             }
-            else if( !strcmp( Path, "/Tracklist" ) )
+            else if( !strcmp( Path, "/TrackList" ) )
             {
                 DBusMessageIter args;
                 dbus_message_iter_init_append( reply->GetMessage(), &args );
@@ -419,11 +425,11 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                 }
                 else if( !strcmp( Member, "Pause" ) )
                 {
-                    if( m_PlayerPanel->GetState() == wxMEDIASTATE_PLAYING )
-                    {
+                    //if( m_PlayerPanel->GetState() == guMediaState_PLAYING )
+                    //{
                         wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_PLAY );
                         wxPostEvent( m_PlayerPanel, event );
-                    }
+                    //}
                     Send( reply );
                     Flush();
                     RetVal = DBUS_HANDLER_RESULT_HANDLED;
@@ -438,8 +444,18 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                 }
                 else if( !strcmp( Member, "Play" ) )
                 {
-                    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_PLAY );
-                    wxPostEvent( m_PlayerPanel, event );
+                    if( m_PlayerPanel->GetState() == guMEDIASTATE_PAUSED )
+                    {
+                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_PLAY );
+                        wxPostEvent( m_PlayerPanel, event );
+                    }
+                    else
+                    {
+                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_STOP );
+                        wxPostEvent( m_PlayerPanel, event );
+                        event.SetId( ID_PLAYERPANEL_PLAY );
+                        wxPostEvent( m_PlayerPanel, event );
+                    }
                     Send( reply );
                     Flush();
                     RetVal = DBUS_HANDLER_RESULT_HANDLED;
@@ -456,14 +472,15 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                 {
                     int PlayStatus;
                     int Dummy = 0;
-                    wxMediaState State = m_PlayerPanel->GetState();
-                    if( State == wxMEDIASTATE_STOPPED )
+                    guMediaState State = m_PlayerPanel->GetState();
+                    if( State == guMEDIASTATE_STOPPED )
                         PlayStatus = 2;
-                    else if( State == wxMEDIASTATE_PAUSED )
+                    else if( State == guMEDIASTATE_PAUSED )
                         PlayStatus = 1;
-                    else if( State == wxMEDIASTATE_PLAYING )
+                    else if( State == guMEDIASTATE_PLAYING )
                         PlayStatus = 0;
-                    int PlayLoop = m_PlayerPanel->GetPlayLoop();
+                    int PlaySingle = ( m_PlayerPanel->GetPlayLoop() == 2 );
+                    int PlayLoop = ( m_PlayerPanel->GetPlayLoop() == 1 );
 
                     DBusMessageIter status;
                     DBusMessageIter args;
@@ -472,7 +489,7 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                     dbus_message_iter_open_container( &args, DBUS_TYPE_STRUCT, NULL, &status );
                     dbus_message_iter_append_basic( &status, DBUS_TYPE_INT32, &PlayStatus );
                     dbus_message_iter_append_basic( &status, DBUS_TYPE_INT32, &Dummy );
-                    dbus_message_iter_append_basic( &status, DBUS_TYPE_INT32, &Dummy );
+                    dbus_message_iter_append_basic( &status, DBUS_TYPE_INT32, &PlaySingle );
                     dbus_message_iter_append_basic( &status, DBUS_TYPE_INT32, &PlayLoop );
                     dbus_message_iter_close_container( &args, &status );
                     Send( reply );
@@ -671,8 +688,25 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
 
                         //printf( "AddTrack: %s\n", TrackPath );
 
+                        if( PlayTrack )
+                        {
+                            m_PlayerPanel->ClearPlayList();
+
+                            if( m_PlayerPanel->GetState() != guMEDIASTATE_STOPPED )
+                            {
+                                wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_STOP );
+                                wxPostEvent( m_PlayerPanel, event );
+                            }
+                        }
+
                         m_PlayerPanel->AddToPlayList( wxString( TrackPath, wxConvUTF8 ) );
-                        // For now this player dont allow to play current loaded track
+
+                        if( PlayTrack )
+                        {
+                            wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_PLAY );
+                            m_PlayerPanel->OnNextTrackButtonClick( event );
+                            m_PlayerPanel->OnPlayButtonClick( event );
+                        }
 
                         int TrackAdded = 1;
                         if( !dbus_message_iter_append_basic( &args, DBUS_TYPE_INT32, &TrackAdded ) )
@@ -703,7 +737,7 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                     else
                     {
                         m_PlayerPanel->RemoveItem( TrackNum );
-                        Send( reply );
+                        //Send( reply );
                         Flush();
                         RetVal = DBUS_HANDLER_RESULT_HANDLED;
                     }
@@ -729,7 +763,6 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                     {
                         m_PlayerPanel->SetPlayLoop( PlayLoop );
 
-                        Send( reply );
                         Flush();
                         RetVal = DBUS_HANDLER_RESULT_HANDLED;
                     }
@@ -756,7 +789,6 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                         wxCommandEvent event;
                         m_PlayerPanel->OnRandomPlayButtonClick( event );
 
-                        Send( reply );
                         Flush();
                         RetVal = DBUS_HANDLER_RESULT_HANDLED;
                     }
@@ -765,71 +797,6 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
         }
     }
 
-//    // MULTIMEDIA KEYS
-//    else if( Type == DBUS_MESSAGE_TYPE_SIGNAL )  // If its a Signal message
-//    {
-//        if( !strcmp( Path, "/org/gnome/SettingsDaemon" ) ||
-//            !strcmp( Path, "/org/gnome/SettingsDaemon/MediaKeys" ) )
-//        {
-//            if( !strcmp( Member, "MediaPlayerKeyPressed" ) )
-//            {
-//                DBusError error;
-//                dbus_error_init( &error );
-//
-//                const char * s = NULL;
-//                const char * KeyName = NULL;
-//
-//                dbus_message_get_args( msg->GetMessage(), &error,
-//                      DBUS_TYPE_STRING, &s,
-//                      DBUS_TYPE_STRING, &KeyName,
-//                      DBUS_TYPE_INVALID );
-//
-//                if( dbus_error_is_set( &error ) )
-//                {
-//                    printf( "Could not read the MediaPlayerKeyPressed parameters : %s\n", error.message );
-//                    dbus_error_free( &error );
-//                    //RetVal =  DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-//                }
-//                else
-//                {
-//                    if( !strcmp( KeyName, "Play" ) )
-//                    {
-//                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_PLAY );
-//                        wxPostEvent( m_PlayerPanel, event );
-//                        RetVal = DBUS_HANDLER_RESULT_HANDLED;
-//                    }
-//                    else if( !strcmp( KeyName, "Stop" ) )
-//                    {
-//                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_STOP );
-//                        wxPostEvent( m_PlayerPanel, event );
-//                        RetVal = DBUS_HANDLER_RESULT_HANDLED;
-//                    }
-//                    else if( !strcmp( KeyName, "Previous" ) )
-//                    {
-//                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_PREVTRACK );
-//                        wxPostEvent( m_PlayerPanel, event );
-//                        RetVal = DBUS_HANDLER_RESULT_HANDLED;
-//                    }
-//                    else if( !strcmp( KeyName, "Next" ) )
-//                    {
-//                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_NEXTTRACK );
-//                        wxPostEvent( m_PlayerPanel, event );
-//                        RetVal = DBUS_HANDLER_RESULT_HANDLED;
-//                    }
-//                    else if( !strcmp( KeyName, "Pause" ) )
-//                    {
-//                        if( m_PlayerPanel->GetState() == wxMEDIASTATE_PLAYING )
-//                        {
-//                            wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_PLAY );
-//                            wxPostEvent( m_PlayerPanel, event );
-//                        }
-//                        RetVal = DBUS_HANDLER_RESULT_HANDLED;
-//                    }
-//                }
-//            }
-//        }
-//    }
-
     return RetVal;
 }
 
@@ -843,6 +810,7 @@ void guMPRIS::OnPlayerTrackChange()
         FillMetadataArgs( signal, CurTrack );
         Send( signal );
         Flush();
+        delete signal;
     }
     else
     {
@@ -858,14 +826,17 @@ void guMPRIS::OnPlayerStatusChange()
     {
         int PlayStatus;
         int Dummy = 0;
-        wxMediaState State = m_PlayerPanel->GetState();
-        if( State == wxMEDIASTATE_STOPPED )
+        guMediaState State = m_PlayerPanel->GetState();
+        if( State == guMEDIASTATE_STOPPED )
             PlayStatus = 2;
-        else if( State == wxMEDIASTATE_PAUSED )
+        else if( State == guMEDIASTATE_PAUSED )
             PlayStatus = 1;
-        else if( State == wxMEDIASTATE_PLAYING )
+        else if( State == guMEDIASTATE_PLAYING )
             PlayStatus = 0;
-        int PlayLoop = m_PlayerPanel->GetPlayLoop();
+        int PlaySingle = ( m_PlayerPanel->GetPlayLoop() == 2 );
+        int PlayLoop = ( m_PlayerPanel->GetPlayLoop() == 1 );
+
+        guLogMessage( wxT( "StatusChanged( %i, %i, %i, %i )" ), PlayStatus, Dummy, Dummy, PlayLoop );
 
         DBusMessageIter status;
         DBusMessageIter args;
@@ -874,12 +845,13 @@ void guMPRIS::OnPlayerStatusChange()
         dbus_message_iter_open_container( &args, DBUS_TYPE_STRUCT, NULL, &status );
         dbus_message_iter_append_basic( &status, DBUS_TYPE_INT32, &PlayStatus );
         dbus_message_iter_append_basic( &status, DBUS_TYPE_INT32, &Dummy );
-        dbus_message_iter_append_basic( &status, DBUS_TYPE_INT32, &Dummy );
+        dbus_message_iter_append_basic( &status, DBUS_TYPE_INT32, &PlaySingle );
         dbus_message_iter_append_basic( &status, DBUS_TYPE_INT32, &PlayLoop );
         dbus_message_iter_close_container( &args, &status );
 
         Send( signal );
         Flush();
+        delete signal;
     }
     else
     {
@@ -927,6 +899,11 @@ void guMPRIS::OnTrackListChange()
         }
         Send( signal );
         Flush();
+        delete signal;
+    }
+    else
+    {
+        guLogError( wxT( "Could not create TrackListChange signal object" ) );
     }
 }
 
